@@ -46,8 +46,8 @@ void save(mglGraph *gr,const char *name,const char *suf="")
 		case 1:	// EPS
 			sprintf(buf,"%s%s.eps",name,suf);
 			gr->WriteEPS(buf);
-			sprintf(buf,"%s%s.png",name,suf);
-			gr->WritePNG(buf,0,false);	break;
+//			sprintf(buf,"%s%s.png",name,suf);
+//			gr->WritePNG(buf,0,false);	break;
 		case 2:	// SVG
 			sprintf(buf,"%s%s.svg",name,suf);
 			gr->WriteSVG(buf);	break;
@@ -108,7 +108,7 @@ int sample_map(mglGraph *gr, const void *s)	// full test (in PNG)
 	gr->SubPlot(2, 1, 1);
 	a.Fill("(x^3+y^3)/2", gr->Min, gr->Max);
 	b.Fill("(x-y)/2", gr->Min, gr->Max);
-	gr->Puts(mglPoint(0, 1.1), "\\{\\frac{x^3+y^3}{2}, \\frac{x-y}{2}\\}", "", -2);
+	gr->Puts(mglPoint(0, 1.1), "\\{\\frac{x^3+y^3}{2}, \\frac{x-y}{2}\\}", "C", -2);
 	gr->Box();
 	gr->Map(a, b, "brgk", 0, false);
 	return 0;
@@ -234,7 +234,8 @@ int sample_pde(mglGraph *gr, const void *)	// PDE and Ray sample
 	gr->Dens(a,"wyrRk");
 	gr->Plot("-x", "k|", type==5?0.01:NAN);
 	gr->Puts(mglPoint(0, 0.85), "absorption: (x+z)/2 for x+z>0");
-	gr->Title("Equation: \\i{ik_0\\partial_zu + \\Delta u + x\\cdot u + i \\frac{x+z}{2}\\cdot u = 0}", "C", -float(width)/height);
+	gr->Title("Equation: ik_0\\partial_zu + \\Delta u + x\\cdot u + i \\frac{x+z}{2}\\cdot u = 0", "C", -float(width)/height);
+//	gr->Title("Equation: \\i{ik_0\\partial_zu + \\Delta u + x\\cdot u + i \\frac{x+z}{2}\\cdot u = 0}", "C", -float(width)/height);
 	gr->Compression(false);  //put setting back
 	return 0;
 }
@@ -293,7 +294,7 @@ int sample_ae(mglGraph *gr, const void *)	// TeX sample
 //-----------------------------------------------------------------------------
 int sample_ad(mglGraph *gr, const void *)	// arrow styles
 {
-	float a=0.1,b=0.4,c=0.5;
+	mreal a=0.1,b=0.4,c=0.5;
 	gr->Line(mglPoint(a,1),mglPoint(b,1),"k-A");		gr->Puts(mglPoint(c,1),"Style 'A' or 'A\\_'","rL");
 	gr->Line(mglPoint(a,0.8),mglPoint(b,0.8),"k-V");	gr->Puts(mglPoint(c,0.8),"Style 'V' or 'V\\_'","rL");
 	gr->Line(mglPoint(a,0.6),mglPoint(b,0.6),"k-K");	gr->Puts(mglPoint(c,0.6),"Style 'K' or 'K\\_'","rL");
@@ -336,17 +337,17 @@ int sample_ac(mglGraph *gr, const void *)	// error boxes
 	gr->Error(y,ey,"ko");
 	gr->Plot(y0,"r");
 	gr->Axis();
-	gr->Text(mglPoint(0,1.2,0),"Random {\\i y}");
+	gr->Text(mglPoint(0,1.2,0),"Random \\i{y}");
 	gr->SubPlot(2,2,1);
 	gr->Error(x,y,ex,"ko");
 	gr->Plot(y0,"r");
 	gr->Axis();
-	gr->Text(mglPoint(0,1.2,0),"Random {\\i x, y}");
+	gr->Text(mglPoint(0,1.2,0),"Random \\i{x, y}");
 	gr->SubPlot(2,2,2);
 	gr->Error(x,y,ey,ey,"ko");
 	gr->Plot(y0,"r");
 	gr->Axis();
-	gr->Text(mglPoint(0,1.2,0),"Random {\\i x, y} and 2d boxes");
+	gr->Text(mglPoint(0,1.2,0),"Random \\i{x, y} and 2d boxes");
 	gr->SubPlot(2,2,3);
 	gr->Text(mglPoint(0,1.2,0),"Random point in 3d space");
 	gr->Rotate(40,60);
@@ -360,7 +361,7 @@ int sample_ab(mglGraph *gr, const void *)	// Gaussian beam
 {
 	gr->VertexColor(true);
 	gr->Alpha(true);	gr->Light(true);	gr->Light(0,mglPoint(0,0,1));
-	mglData a(30,30,30),b(30,30,30);
+	mglData a(30,30,30), b(30,30,30);
 	a.Modify("exp(-16*((z-0.5)^2+(y-0.5)^2)/(1+4*x^2))");
 	b.Modify("16*((z-0.5)^2+(y-0.5)^2)*(x)/(1+4*x^2)");
 	gr->CAxis(0,1);
@@ -646,20 +647,23 @@ int sample_a1(mglGraph *gr, const void *)	// transformation
 int sample_1d(mglGraph *gr, const void *s)	// full test (in PNG)
 {
 	const char *suf =(const char *)s;
-	mglData y(50,3), x(50), x2(50), y1(50), y2(50), ys(10,3);
+	mglData y(50,3), x(50), x2(50), y1(50), y2(50), ys(10,3), yr(10,3);
 	ys.Modify("0.8*sin(pi*(2*x+y/2))+0.2*rnd");
+	yr.Modify("0.4*sin(pi*(2*x+y/2))+0.1*rnd");
 	y.Modify("0.7*sin(2*pi*x) + 0.5*cos(3*pi*x) + 0.2*sin(pi*x)",0);
 	y.Modify("sin(2*pi*x)",1);	y.Modify("cos(2*pi*x)",2);
 	y1.Modify("0.5+0.3*cos(2*pi*x)");	y2.Modify("0.3*sin(2*pi*x)");
 	x.Fill(-1,1,'x');			x2.Modify("0.05+0.03*cos(2*pi*x)");
 
-	mglData  x0(10), y0(10), ex0(10), ey0(10),ch(7,2);
+	mglData x0(10), y0(10), ex0(10), ey0(10),ch(7,2);
 	x0.Modify("2*x-1 + 0.1*rnd-0.05");
 	y0.Modify("0.7*sin(2*pi*x) + 0.5*cos(3*pi*x) + 0.2*sin(pi*x) + 0.2*rnd-0.1");
 	ey0.Modify("0.2");	ex0.Modify("0.1");	ch.Modify("rnd+0.1");
 
 	gr->Org = mglPoint();
 	gr->Clf();	gr->Box();	gr->Plot(y);	save(gr,"plot",suf);
+	gr->Clf();	gr->Radar(yr,"#");	save(gr,"radar",suf);
+	gr->Clf();	gr->Box();	gr->Plot(x,y);	gr->Traj(x,y,y1,y2);	save(gr,"traj",suf);
 	gr->Clf();	gr->Box();	gr->Tens(y.SubData(-1,0), y.SubData(-1,1));	save(gr,"tens",suf);
 	gr->Clf();	gr->Box();	gr->Area(y);	save(gr,"area",suf);
 	gr->Clf();	gr->Box();	gr->Stem(y);	save(gr,"stem",suf);
@@ -689,7 +693,7 @@ int sample_1d(mglGraph *gr, const void *s)	// full test (in PNG)
 int sample_2d(mglGraph *gr, const void *s)	// full test (in PNG)
 {
 	const char *suf =(const char *)s;
-	mglData  a(50,40), b(50,40);
+	mglData  a(50,40), b(50,40),v(9);	v.Fill(-1,1);
 	a.Modify("0.6*sin(2*pi*x)*sin(3*pi*y) + 0.4*cos(3*pi*(x*y))");
 	b.Modify("0.6*cos(2*pi*x)*cos(3*pi*y) + 0.4*cos(3*pi*(x*y))");
 
@@ -697,7 +701,7 @@ int sample_2d(mglGraph *gr, const void *s)	// full test (in PNG)
 	gr->Clf();	gr->Box();	gr->Cont(a,"BbcyrRt");	save(gr,"contt",suf);
 	gr->Clf();	gr->Box();	gr->TileS(a,b,"BbcyrR");save(gr,"tiler",suf);
 	gr->Clf();	gr->Box();	gr->Dens(a);	gr->Colorbar();	save(gr,"dens",suf);
-//	gr->Clf();	gr->Box();	gr->Map(a,b,"brg");		save(gr,"map",suf);
+	gr->Clf();	gr->Box();	gr->ContD(v,a);	gr->Colorbar(v);save(gr,"contd",suf);
 	gr->Light(true);	gr->Rotate(40,60);
 	gr->Clf();	gr->Box();	gr->Surf(a,"BbcyrR|");	save(gr,"surf_sl",suf);
 	gr->Clf();	gr->Box();	gr->Surf(a,"BbcyrR");	save(gr,"surf",suf);
@@ -1045,7 +1049,7 @@ int sample_ternary(mglGraph *gr, const void *)	// flag #
 	gr->Axis(); gr->Grid("xyz","B;");
 	gr->Label('x',"x comp.");
 	gr->Label('y',"y comp.");
-	gr->Label('z',"t comp.");
+	gr->Label('t',"t comp.");
 	gr->Ternary(false);
 	return 0;
 }
@@ -1139,7 +1143,7 @@ int sample_fit(mglGraph *gr, const void *s)	// flag #
 	gr->Plot(rnd, ". ");
 	gr->Box();
 
-	float ini[3] = {1,1,3};
+	mreal ini[3] = {1,1,3};
 	gr->Fit(res, rnd, "a+b*sin(c*x)", "abc", ini);
 	gr->Plot(res, "r");
 	gr->Plot(in, "b");
@@ -1153,7 +1157,7 @@ int sample_fit(mglGraph *gr, const void *s)	// flag #
 #include <mgl/mgl_parse.h>
 int sample_parser(mglGraph *gr, const void *)	// flag #
 {
-	float a[100];   // let a_i = sin(4*pi*x), x=0...1
+	mreal a[100];   // let a_i = sin(4*pi*x), x=0...1
 	for(int i=0;i<100;i++)a[i]=sin(4*M_PI*i/99);
 	mglParse *parser = new mglParse;
 	mglData &d =(parser->AddVar("dat"))->d;
@@ -1164,6 +1168,7 @@ int sample_parser(mglGraph *gr, const void *)	// flag #
 	parser->Execute(gr, "xlabel 'x'\nylabel 'y'\nbox");
 	// also you may use cycles or conditions in script
 	parser->Execute(gr, "for $0 -1 1 0.1\nline 0 0 -1 $0 'r'\nnext");
+	delete parser;
 	return 0;
 }
 //-----------------------------------------------------------------------------
@@ -1217,6 +1222,7 @@ static struct option longopts[] =
 	{ "big",			no_argument,	&big,			1 },
 	{ "idtf",			no_argument,	&type,			5 },
 	{ "eps",			no_argument,	&type,			1 },
+	{ "svg",			no_argument,	&type,			2 },
 	{ "gif",			no_argument,	&type,			6 },
 	{ "help",			no_argument,	NULL,			'?' },
 	{ NULL,					0,	NULL,			0 }
@@ -1232,6 +1238,7 @@ void usage()
 		"-big			- png picture is 1200x800\n"			\
 		"-idtf			- output idtf\n"				\
 		"-eps			- output EPS\n"					\
+		"-svg			- output SVG\n"					\
 		"-srnd			- produce the same random numbers in any run\n"	\
 		"-test			- perform test\n"
 		);
@@ -1241,6 +1248,14 @@ void usage()
 #include "mgl/mgl_parse.h"
 int test(mglGraph *gr)
 {
+	mglData  a(50,40),v(9);	v.Fill(-1,1);
+	a.Modify("0.6*sin(2*pi*x)*sin(3*pi*y) + 0.4*cos(3*pi*(x*y))");
+
+	gr->SubPlot(2,1,0);	gr->Box();	gr->ContD(v,a);	gr->Colorbar(v);
+	gr->SubPlot(2,1,1);	gr->Box();	gr->Dens(a);	gr->Colorbar();
+	gr->ShowImage("",false);
+	return 0;
+
 	mglParse par;
 	par.AllowSetSize = true;
 	FILE *fp=fopen("box.mgl","rt");
@@ -1266,20 +1281,25 @@ int main(int argc,char **argv)
 			case 'w':	width = atoi(optarg);	break;
 			case 'h':	height = atoi(optarg);	break;
 			case '?':
-			default:
-				usage();
+			default:	usage();
 		}
 
 	if(dotest)
 	{
+//		test(&ps);
 		test(&zb);
+//		sample_ternary(&zb,"");
 		//sample_crust(&zb,0);
 		zb.WritePNG("test.png","",false);
-		zb.WriteGIF("test.gif","");
+/*		ps.WritePNG("test_2.png","",false);
+		ps.WriteEPS("test.eps");
+		ps.WriteSVG("test.svg");
+		zb.WriteGIF("test.gif","");*/
 		return 0;
 	}
 	if(type==5)			gr = &u3d;
 	else if(type==1){	gr = &ps;	gr->SetSize(width,height);	}
+	else if(type==2){	gr = &ps;	gr->SetSize(width,height);	}
 	else			{	gr = &zb;	gr->SetSize(width,height);	}
 	if(mini)		{	gr->SetSize(200,133);	suf = "_sm";	}
 	if(big)
