@@ -1,5 +1,5 @@
 /***************************************************************************
- * mgl_axis.cpp is part of Math Graphic Library
+ * mgl_axis.cpp is part of Math Graphic Library                            *
  * Copyright (C) 2007 Alexey Balakin <balakin@appl.sci-nnov.ru>            *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -22,15 +22,10 @@
 #include <stdarg.h>
 #include <stdlib.h>
 
-#include "mgl/mgl_eval.h"
 #include "mgl/mgl.h"
+#include "mgl/mgl_eval.h"
 #include "mgl/mgl_c.h"
 #define FLT_EPS	(1.+1.2e-07)
-
-#ifdef WIN32
-#define swprintf    _snwprintf
-#endif
-
 //-----------------------------------------------------------------------------
 wchar_t *mgl_wcsdup(const wchar_t *s)
 {
@@ -62,45 +57,32 @@ void wcstrim_mgl(wchar_t *str)
 }
 //-----------------------------------------------------------------------------
 ///	Round the number to k digits. For example, x = 0.01234 -> r = 0.0123 for k=3.
-float mgl_okrugl(float x,int k)
+mreal mgl_okrugl(mreal x,int k)
 {
 	if(x<=0)	return 0;
-	float r;
+	mreal r;
 	long n = long(log10(x));
 	r = floor(x*pow(10.,k-n))/pow(10.,k-n);
 	return r;
 }
 //-----------------------------------------------------------------------------
-void mglGraph::DrawTick(float *pp,bool sub)
+void mglGraph::DrawTick(mreal *pp,bool sub)
 {
-//	bool tt[3];
-//	float d1,d2,f=sub?0.02:0.04;
 	ScalePoint(pp[0],pp[1],pp[2]);
 	ScalePoint(pp[3],pp[4],pp[5]);
 	ScalePoint(pp[6],pp[7],pp[8]);
-/*	pp[0] -= pp[3];		pp[1] -= pp[4];		pp[2] -= pp[5];
-	pp[6] -= pp[3];		pp[7] -= pp[4];		pp[8] -= pp[5];
-	d1 = sqrt(pp[0]*pp[0] + pp[1]*pp[1] + pp[2]*pp[2]);
-	d2 = sqrt(pp[6]*pp[6] + pp[7]*pp[7] + pp[8]*pp[8]);
-	d1 = (d1>0 ? d1:1)/f;	d2 = (d2>0 ? d2:1)/f;
-	pp[0] = pp[3] + pp[0]/d1;
-	pp[1] = pp[4] + pp[1]/d1;
-	pp[2] = pp[5] + pp[2]/d1;
-	pp[6] = pp[3] + pp[6]/d2;
-	pp[7] = pp[4] + pp[7]/d2;
-	pp[8] = pp[5] + pp[8]/d2;*/
 	curv_plot(3,pp,0);
 }
 //-----------------------------------------------------------------------------
-void mglGraph::DrawXGridLine(float t, float y0, float z0)
+void mglGraph::DrawXGridLine(mreal t, mreal y0, mreal z0)
 {
 	register int i;
-	float pp[3*31];
+	mreal pp[3*31];
 	bool tt[31];
 	for(i=0;i<31;i++)
 	{
-		pp[3*i]=TernAxis ? t-t*i/30. : t;
-		pp[3*i+1]=TernAxis ? t*i/30. : y0;
+		pp[3*i]=t;//TernAxis ? t-t*i/30. : t;
+		pp[3*i+1]=y0;//TernAxis ? t*i/30. : y0;
 		pp[3*i+2]=Min.z+(Max.z-Min.z)*i/30.;
 		tt[i] = ScalePoint(pp[3*i],pp[3*i+1],pp[3*i+2]);
 	}
@@ -114,15 +96,15 @@ void mglGraph::DrawXGridLine(float t, float y0, float z0)
 	curv_plot(31,pp,tt);
 }
 //-----------------------------------------------------------------------------
-void mglGraph::DrawYGridLine(float t, float x0, float z0)
+void mglGraph::DrawYGridLine(mreal t, mreal x0, mreal z0)
 {
 	register int i;
-	float pp[3*31];
+	mreal pp[3*31];
 	bool tt[31];
 	for(i=0;i<31;i++)
 	{
-		pp[3*i]=TernAxis ? t*i/30. : x0;
-		pp[3*i+1]=TernAxis ? t-t*i/30. : t;
+		pp[3*i]=x0;//TernAxis ? t*i/30. : x0;
+		pp[3*i+1]=t;//TernAxis ? t-t*i/30. : t;
 		pp[3*i+2]=Min.z+(Max.z-Min.z)*i/30.;
 		tt[i] = ScalePoint(pp[3*i],pp[3*i+1],pp[3*i+2]);
 	}
@@ -136,10 +118,10 @@ void mglGraph::DrawYGridLine(float t, float x0, float z0)
 	curv_plot(31,pp,tt);
 }
 //-----------------------------------------------------------------------------
-void mglGraph::DrawZGridLine(float t, float x0, float y0)
+void mglGraph::DrawZGridLine(mreal t, mreal x0, mreal y0)
 {
 	register int i;
-	float pp[3*31];
+	mreal pp[3*31];
 	bool tt[31];
 	for(i=0;i<31;i++)
 	{
@@ -157,10 +139,33 @@ void mglGraph::DrawZGridLine(float t, float x0, float y0)
 	curv_plot(31,pp,tt);
 }
 //-----------------------------------------------------------------------------
+void mglGraph::DrawTGridLine(mreal t, mreal z0)
+{
+	if(!TernAxis)	return;
+	register int i;
+	mreal pp[3*31];
+	bool tt[31];
+	for(i=0;i<31;i++)
+	{
+		pp[3*i]= t*i/30.;
+		pp[3*i+1]=t-t*i/30.;
+		pp[3*i+2]=z0;
+		tt[i] = ScalePoint(pp[3*i],pp[3*i+1],pp[3*i+2]);
+	}
+	curv_plot(31,pp,tt);
+	for(i=0;i<31;i++)
+	{
+		pp[3*i]=t-t*i/30.;
+		pp[3*i+1]=t*i/30.;
+		pp[3*i+2]=z0;
+	}
+	curv_plot(31,pp,tt);
+}
+//-----------------------------------------------------------------------------
 void mglGraph::Grid(const char *dir, const char *pen)
 {
 	static int cgid=1;	StartGroup("AxisGrid",cgid++);
-	float x0,y0,z0,ddx,ddy,ddz,t;
+	mreal x0,y0,z0,ddx,ddy,ddz,t;
 	if(pen)	SelectPen(pen);
 	else	SelectPen(TranspType!=2 ? "B;1" : "b;1");
 	ddx = dx>=0 ? dx : mgl_okrugl((Min.x-Max.x)/(dx+1),3);
@@ -205,13 +210,21 @@ void mglGraph::Grid(const char *dir, const char *pen)
 			for(t=z0;t<=Max.z*FLT_EPS;t*=10)	DrawZGridLine(t,x0,y0);
 		}
 	}
+	if(strchr(dir,'t'))
+	{
+		z0 = isnan(OrgT.z) ? GetOrgZ('z') : OrgT.z;
+		z0 = z0 - ddz*floor((z0-Min.z)/ddz);
+		if(xnum)	for(int i=0;i<xnum;i++)		DrawTGridLine(xval[i],z0);
+		else if(dx)	for(t=Min.x;t<=Max.x;t+=ddx)	DrawTGridLine(t,z0);
+		else if(dy)	for(t=Min.y;t<=Max.y;t+=ddy)	DrawTGridLine(t,z0);
+	}
 	SelectPen("k-1");
 	EndGroup();
 }
 //-----------------------------------------------------------------------------
-void mglGraph::Label(char dir,const wchar_t *text,int pos,float size,float shift)
+void mglGraph::Label(char dir,const wchar_t *text,int pos,mreal size,mreal shift)
 {
-	float t, x0, y0, z0;
+	mreal t, x0, y0, z0;
 	SelectPen("k-1");
 	x0 = GetOrgX(dir);	y0 = GetOrgY(dir);	z0 = GetOrgZ(dir);
 	if(size<=0)	size = -size*FontSize;
@@ -222,23 +235,30 @@ void mglGraph::Label(char dir,const wchar_t *text,int pos,float size,float shift
 	}
 	if(dir=='y')
 	{
-		t = pos>0 ? Max.y : (pos<0 ? Min.y : (dy ? (Min.y+Max.y)/2 : sqrt(Min.y*Max.y)));
 		if(TernAxis)
-			Putsw(mglPoint(Max.x-t,t,z0),text,FontDef,size,'Y',shift);
+		{
+			t = pos>0 ? 1 : (pos<0 ? 0.: (dz ? 0.5 : 0.));
+			Putsw(mglPoint(Max.x-t,t,z0),text,FontDef,size,'T',shift);
+		}
 		else
+		{
+			t = pos>0 ? Max.y : (pos<0 ? Min.y : (dy ? (Min.y+Max.y)/2 : sqrt(Min.y*Max.y)));
 			Putsw(mglPoint(x0,t,z0),text,FontDef,size,'Y',shift);
+		}
 	}
 	if(dir=='z')
 	{
 		t = pos>0 ? Max.z : (pos<0 ? Min.z : (dz ? (Min.z+Max.z)/2 : sqrt(Min.z*Max.z)));
-		if(TernAxis)
-			Putsw(mglPoint(x0,Max.y-t,z0),text,FontDef,size,'Z',shift);
-		else
-			Putsw(mglPoint(x0,y0,t),text,FontDef,size,'Z',shift);
+		Putsw(mglPoint(x0,y0,t),text,FontDef,size,'Z',shift);
+	}
+	if(dir=='t' && TernAxis)
+	{
+		t = pos<0 ? Max.y : (pos>0 ? Min.y : (dy ? (Min.y+Max.y)/2 : sqrt(Min.y*Max.y)));
+		Putsw(mglPoint(x0,t,z0),text,FontDef,size,'Y',shift);
 	}
 }
 //-----------------------------------------------------------------------------
-int _mgl_tick_ext(float a, float b, wchar_t s[32], float &v)
+int _mgl_tick_ext(mreal a, mreal b, wchar_t s[32], mreal &v)
 {
 	int kind = 0;
 	if(fabs(a-b)<0.01*fabs(a))
@@ -249,13 +269,13 @@ int _mgl_tick_ext(float a, float b, wchar_t s[32], float &v)
 		{
 			int k=int(log10(v)-0.01);
 			kind=3;		v=mgl_ipow(10,k);
-			swprintf(s, 32, L"(@{\\times{}10^{%d}})", k);
+			mglprintf(s, 32, L"(@{\\times{}10^{%d}})", k);
 		}
 		if(v<1e-2f)
 		{
 			int k=int(log10(v)-0.01)-1;
 			kind=3;		v=mgl_ipow(10,k);
-			swprintf(s, 32, L"(@{\\times{}10^{%d}})", k);
+			mglprintf(s, 32, L"(@{\\times{}10^{%d}})", k);
 		}
 	}
 	else
@@ -266,61 +286,66 @@ int _mgl_tick_ext(float a, float b, wchar_t s[32], float &v)
 			kind = 2;
 			int k=int(log10(v)-0.01);
 			v=mgl_ipow(10,k);
-			swprintf(s, 32, L"\\times 10^{%d}", k);
+			mglprintf(s, 32, L"\\times 10^{%d}", k);
 		}
 		if(v<1e-2f)
 		{
 			kind = 2;
 			int k=int(log10(v)-0.01)-1;
 			v=mgl_ipow(10,k);
-			swprintf(s, 32, L"\\times 10^{%d}", k);
+			mglprintf(s, 32, L"\\times 10^{%d}", k);
 		}
 	}
 	return kind;
 }
 //-----------------------------------------------------------------------------
-void _mgl_tick_text(float z, float z0, float d, float v, int kind, wchar_t str[64])
+void _mgl_tick_text(mreal z, mreal z0, mreal d, mreal v, int kind, wchar_t str[64])
 {
-	float u = fabs(z)<d ? 0:z;
+	mreal u = fabs(z)<d ? 0:z;
 	if((kind&1) && z>z0)	u = fabs(z-z0)<d ? 0:(z-z0);
 	if(kind==2 || (kind==3 && z>z0))	u /= v;
 	if((kind&1) && z>z0)
 	{
 		int n1,n2;
-		swprintf(str, 64, fabs(u)<1 ? L"@{(+%.2g)}" : L"@{(+%.3g)}",u);	n1=wcslen(str);
-		swprintf(str, 64, L"@{(+%g)}",u);	n2=wcslen(str);
-		if(n1<n2)	swprintf(str, 64, L"@{(+%.2g)}",u);
+		mglprintf(str, 64, fabs(u)<1 ? L"@{(+%.2g)}" : L"@{(+%.3g)}",u);	n1=wcslen(str);
+		mglprintf(str, 64, L"@{(+%g)}",u);	n2=wcslen(str);
+		if(n1<n2)	mglprintf(str, 64, L"@{(+%.2g)}",u);
 	}
 	else
 	{
 		int n1,n2;
-		swprintf(str, 64, fabs(u)<1 ? L"%.2g" :  L"%.3g",u);
-		n1=wcslen(str);	swprintf(str, 64, L"%g",u);	n2=wcslen(str);
-		if(n1<n2)	swprintf(str, 64, L"%.2g",u);
+		mglprintf(str, 64, fabs(u)<1 ? L"%.2g" :  L"%.3g",u);
+		n1=wcslen(str);	mglprintf(str, 64, L"%g",u);	n2=wcslen(str);
+		if(n1<n2)	mglprintf(str, 64, L"%.2g",u);
 	}
 }
 //-----------------------------------------------------------------------------
-void mglGraph::DrawXTick(float x, float y0, float z0, float dy, float dz, int f)
+void mglGraph::DrawXTick(mreal x, mreal y0, mreal z0, mreal dy, mreal dz, int f)
 {
-	float pp[9],ff=TickLen/sqrt(1.+f);
+	mreal pp[9],ff=TickLen/sqrt(1.+f*st_t);
 	pp[0]=x;	pp[1]=y0+dy*ff;	pp[2]=z0;
 	pp[3]=x;	pp[4]=y0;	pp[5]=z0;
 	pp[6]=x;	pp[7]=y0;	pp[8]=z0+dz*ff;
+	if(*TickStl && !f)	SelectPen(TickStl);
+	if(*SubTStl && f)	SelectPen(SubTStl);
 	DrawTick(pp,false);
 }
 void mglGraph::AxisX(bool text)
 {
-	float pp[3*31];
+	mreal pp[3*31];
 	bool tt[31];
 	wchar_t str[64];
 	long i;
-	float x0,y0,z0,x,ddx,ddy,ddz,v=0;
+	mreal x0,y0,z0,x,ddx,ddy,ddz,v=0;
 	x0 = GetOrgX('x');	y0 = GetOrgY('x');	z0 = GetOrgZ('x');
 	ddx = dx>=0 ? dx : mgl_okrugl((Min.x-Max.x)/(dx+1),3);
 	ddy = dy>=0 ? dy : mgl_okrugl((Min.y-Max.y)/(dy+1),3);
 	ddz = dz>=0 ? dz : mgl_okrugl((Min.z-Max.z)/(dz+1),3);
-	if(ddy==0)	ddy=y0*3;//(Max.y-Min.y)/2;
- 	if(ddz==0)	ddz=z0*3;//(Max.z-Min.z)/2;
+//	if(ddx>0.1*fabs(Max.x-Min.x)/TickLen)	ddx = 0.1*(Max.x-Min.x)/TickLen;
+//	if(ddy>0.1*fabs(Max.y-Min.y)/TickLen)	ddy = 0.1*(Max.y-Min.y)/TickLen;
+//	if(ddz>0.1*fabs(Max.z-Min.z)/TickLen)	ddz = 0.1*(Max.z-Min.z)/TickLen;
+	if(ddy==0)	ddy=y0*10>fabs(Max.y-Min.y) ? 0.3*fabs(Max.y-Min.y) : y0*3;
+	if(ddz==0)	ddz=z0*10>fabs(Max.z-Min.z) ? 0.3*fabs(Max.z-Min.z) : z0*3;
 	if(y0>(Max.y+Min.y)/2)	ddy = -ddy;
 	if(z0>(Max.z+Min.z)/2)	ddz = -ddz;
 	SelectPen(TranspType!=2 ? "k-1":"w-1");
@@ -352,7 +377,7 @@ void mglGraph::AxisX(bool text)
 			DrawXTick(x,y0,z0,ddy,ddz);
 			if(text)
 			{
-				if(xtt)	swprintf(str, 64, xtt, fabs(x)<ddx/100 ? 0 : x);
+				if(xtt)	mglprintf(str, 64, xtt, fabs(x)<ddx/100 ? 0 : x);
 				else	_mgl_tick_text(x,x0,ddx/100,v,kind,str);
 				wcstrim_mgl(str);
 				Putsw(mglPoint(x,y0,z0),str,FontDef,-1,'x');
@@ -377,7 +402,7 @@ void mglGraph::AxisX(bool text)
 			if(x*FLT_EPS>=Min.x && x<=Max.x*FLT_EPS)
 			{
 				DrawXTick(x,y0,z0,ddy,ddz);
-				swprintf(str,64,L"10^{%d}",int(floor(0.1+log10(x))));
+				mglprintf(str,64,L"10^{%d}",int(floor(0.1+log10(x))));
 				if(text && cur%k==0)
 					Putsw(mglPoint(x,y0,z0),str,FontDef,-1,'x');
 				cur++;
@@ -389,27 +414,32 @@ void mglGraph::AxisX(bool text)
 	}
 }
 //-----------------------------------------------------------------------------
-void mglGraph::DrawYTick(float y, float x0, float z0, float dx, float dz, int f)
+void mglGraph::DrawYTick(mreal y, mreal x0, mreal z0, mreal dx, mreal dz, int f)
 {
-	float pp[9],ff=TickLen/sqrt(1.+f);
+	mreal pp[9],ff=TickLen/sqrt(1.+f*st_t);
 	pp[0]=x0+dx*ff;	pp[1]=y;	pp[2]=z0;
 	pp[3]=x0;		pp[4]=y;	pp[5]=z0;
 	pp[6]=x0;		pp[7]=y;	pp[8]=z0+dz*ff;
+	if(*TickStl && !f)	SelectPen(TickStl);
+	if(*SubTStl && f)	SelectPen(SubTStl);
 	DrawTick(pp,false);
 }
 void mglGraph::AxisY(bool text)
 {
-	float pp[3*31];
+	mreal pp[3*31];
 	bool tt[31];
 	wchar_t str[64];
 	long i;
-	float x0,y0,z0,y,ddx,ddy,ddz,v=0;
+	mreal x0,y0,z0,y,ddx,ddy,ddz,v=0;
 	x0 = GetOrgX('y');	y0 = GetOrgY('y');	z0 = GetOrgZ('y');
 	ddx = dx>=0 ? dx : mgl_okrugl((Min.x-Max.x)/(dx+1),3);
 	ddy = dy>=0 ? dy : mgl_okrugl((Min.y-Max.y)/(dy+1),3);
 	ddz = dz>=0 ? dz : mgl_okrugl((Min.z-Max.z)/(dz+1),3);
-	if(ddx==0)	ddx=x0*3;//(Max.x-Min.x)/2;
-	if(ddz==0)	ddz=z0*3;//(Max.z-Min.z)/2;
+//	if(ddx>0.1*fabs(Max.x-Min.x)/TickLen)	ddx = 0.1*(Max.x-Min.x)/TickLen;
+//	if(ddy>0.1*fabs(Max.y-Min.y)/TickLen)	ddy = 0.1*(Max.y-Min.y)/TickLen;
+//	if(ddz>0.1*fabs(Max.z-Min.z)/TickLen)	ddz = 0.1*(Max.z-Min.z)/TickLen;
+	if(ddx==0)	ddx=x0*10>fabs(Max.x-Min.x) ? 0.3*fabs(Max.x-Min.x) : x0*3;
+	if(ddz==0)	ddz=z0*10>fabs(Max.z-Min.z) ? 0.3*fabs(Max.z-Min.z) : z0*3;
 	if(x0>(Max.x+Min.x)/2)	ddx = -ddx;
 	if(z0>(Max.z+Min.z)/2)	ddz = -ddz;
 	SelectPen(TranspType!=2 ? "k-1":"w-1");
@@ -441,7 +471,7 @@ void mglGraph::AxisY(bool text)
 			DrawYTick(y,x0,z0,ddx,ddz);
 			if(text)
 			{
-				if(ytt)	swprintf(str, 64, ytt, fabs(y)<ddy/100 ? 0 : y);
+				if(ytt)	mglprintf(str, 64, ytt, fabs(y)<ddy/100 ? 0 : y);
 				else	_mgl_tick_text(y,y0,ddy/100,v,kind,str);
 				wcstrim_mgl(str);
 				Putsw(mglPoint(x0,y,z0),str,FontDef,-1,'y');
@@ -466,7 +496,7 @@ void mglGraph::AxisY(bool text)
 			if(y*FLT_EPS>=Min.y && y<=Max.y*FLT_EPS)
 			{
 				DrawYTick(y,x0,z0,ddx,ddz);
-				swprintf(str,64,L"10^{%d}",int(floor(0.1+log10(y))));
+				mglprintf(str,64,L"10^{%d}",int(floor(0.1+log10(y))));
 				if(text && cur%k==0)
 					Putsw(mglPoint(x0,y,z0),str,FontDef,-1,'y');
 				cur++;
@@ -478,27 +508,32 @@ void mglGraph::AxisY(bool text)
 	}
 }
 //-----------------------------------------------------------------------------
-void mglGraph::DrawZTick(float z, float x0, float y0, float dx, float dy, int f)
+void mglGraph::DrawZTick(mreal z, mreal x0, mreal y0, mreal dx, mreal dy, int f)
 {
-	float pp[9],ff=TickLen/sqrt(1.+f);
+	mreal pp[9],ff=TickLen/sqrt(1.+f*st_t);
 	pp[0]=x0;	pp[1]=y0+dy*ff;	pp[2]=z;
 	pp[3]=x0;	pp[4]=y0;		pp[5]=z;
 	pp[6]=x0+dx*ff;	pp[7]=y0;	pp[8]=z;
+	if(*TickStl && !f)	SelectPen(TickStl);
+	if(*SubTStl && f)	SelectPen(SubTStl);
 	DrawTick(pp,false);
 }
 void mglGraph::AxisZ(bool text)
 {
-	float pp[3*31];
+	mreal pp[3*31];
 	bool tt[31];
 	wchar_t str[64];
 	long i;
-	float x0,y0,z0,z,ddx,ddy,ddz,v=0;
+	mreal x0,y0,z0,z,ddx,ddy,ddz,v=0;
 	x0 = GetOrgX('z');	y0 = GetOrgY('z');	z0 = GetOrgZ('z');
 	ddx = dx>=0 ? dx : mgl_okrugl((Min.x-Max.x)/(dx+1),3);
 	ddy = dy>=0 ? dy : mgl_okrugl((Min.y-Max.y)/(dy+1),3);
 	ddz = dz>=0 ? dz : mgl_okrugl((Min.z-Max.z)/(dz+1),3);
-	if(ddx==0)	ddx=x0*3;//(Max.x-Min.x)/2;
-	if(ddy==0)	ddy=y0*3;//(Max.y-Min.y)/2;
+//	if(ddx>0.1*fabs(Max.x-Min.x)/TickLen)	ddx = 0.1*(Max.x-Min.x)/TickLen;
+//	if(ddy>0.1*fabs(Max.y-Min.y)/TickLen)	ddy = 0.1*(Max.y-Min.y)/TickLen;
+//	if(ddz>0.1*fabs(Max.z-Min.z)/TickLen)	ddz = 0.1*(Max.z-Min.z)/TickLen;
+	if(ddx==0)	ddx=x0*10>fabs(Max.x-Min.x) ? 0.3*fabs(Max.x-Min.x) : x0*3;
+	if(ddy==0)	ddy=y0*10>fabs(Max.y-Min.y) ? 0.3*fabs(Max.y-Min.y) : y0*3;
 	if(x0>(Max.x+Min.x)/2)	ddx = -ddx;
 	if(y0>(Max.y+Min.y)/2)	ddy = -ddy;
 	SelectPen(TranspType!=2 ? "k-1":"w-1");
@@ -530,7 +565,7 @@ void mglGraph::AxisZ(bool text)
 			DrawZTick(z,x0,y0,ddx,ddy);
 			if(text)
 			{
-				if(ztt)	swprintf(str,64,ztt,fabs(z)<ddz/100 ? 0 : z);
+				if(ztt)	mglprintf(str,64,ztt,fabs(z)<ddz/100 ? 0 : z);
 				else	_mgl_tick_text(z,z0,ddz/100,v,kind,str);
 				wcstrim_mgl(str);
 				Putsw(mglPoint(x0,y0,z),str,FontDef,-1,'z');
@@ -555,7 +590,7 @@ void mglGraph::AxisZ(bool text)
 			if(z*FLT_EPS>=Min.z && z<=Max.z*FLT_EPS)
 			{
 				DrawZTick(z,x0,y0,ddx,ddy);
-				swprintf(str,64,L"10^{%d}",int(floor(0.1+log10(z))));
+				mglprintf(str,64,L"10^{%d}",int(floor(0.1+log10(z))));
 				if(text && cur%k==0)
 					Putsw(mglPoint(x0,y0,z),str,FontDef,-1,'z');
 				cur++;
@@ -573,22 +608,28 @@ void mglGraph::Axis(const char *dir, bool adjust)
 	if(adjust)	AdjustTicks(dir);
 	static int cgid=1;	StartGroup("Axis",cgid++);
 	bool text = strchr(dir,'_')==NULL;
-	if(strchr(dir,'x'))	{	_sx = 1;	AxisX(text);	}
-	if(strchr(dir,'X'))	{	_sx = -1;	AxisX(text);	}
-	if(strchr(dir,'y'))
-	{	_sy = 1;	if(TernAxis)	AxisTY(text);	else	AxisY(text);	}
-	if(strchr(dir,'Y'))
-	{	_sy = -1;	if(TernAxis)	AxisTY(text);	else	AxisY(text);	}
-	if(strchr(dir,'z'))
-	{	_sz = 1;	if(TernAxis)	AxisTZ(text);	else	AxisZ(text);	}
-	if(strchr(dir,'Z'))
-	{	_sz = -1;	if(TernAxis)	AxisTZ(text);	else	AxisZ(text);	}
+	if(TernAxis)
+	{
+		_sx = _sy = _st = 1;
+		AxisX(text);
+//		AxisY(text);
+		AxisT(text);
+	}
+	else
+	{
+		if(strchr(dir,'x'))	{	_sx = 1;	AxisX(text);	}
+		if(strchr(dir,'X'))	{	_sx = -1;	AxisX(text);	}
+		if(strchr(dir,'y'))	{	_sy = 1;	AxisY(text);	}
+		if(strchr(dir,'Y'))	{	_sy = -1;	AxisY(text);	}
+	}
+	if(strchr(dir,'z'))	{	_sz = 1;	AxisZ(text);	}
+	if(strchr(dir,'Z'))	{	_sz = -1;	AxisZ(text);	}
 	EndGroup();
 }
 //-----------------------------------------------------------------------------
 void mglGraph::TickBox()
 {
-	float x0,y0,z0,x,y,z,ddx,ddy,ddz;
+	mreal x0,y0,z0,x,y,z,ddx,ddy,ddz;
 	x0 = GetOrgX(0);	y0 = GetOrgY(0);	z0 = GetOrgZ(0);
 	ddx = dx>=0 ? dx : mgl_okrugl((Min.x-Max.x)/(dx+1),3);
 	ddy = dy>=0 ? dy : mgl_okrugl((Min.y-Max.y)/(dy+1),3);
@@ -704,7 +745,7 @@ void mglGraph::Box(const char *col,bool ticks)
 void mglGraph::Box(mglColor p,bool ticks)
 {
 	static int cgid=1;	StartGroup("Box",cgid++);
-	float x1=Min.x,x2=Max.x,y1=Min.y,y2=Max.y,z1=Min.z,z2=Max.z;
+	mreal x1=Min.x,x2=Max.x,y1=Min.y,y2=Max.y,z1=Min.z,z2=Max.z;
 	Arrow1 = Arrow2 = '_';
 	if(p.Valid())	Pen(p,'-',BaseLineWidth);
 	else			Pen(TranspType!=2 ? BC:WC,'-',BaseLineWidth);
@@ -744,10 +785,10 @@ void mglGraph::AddLegend(const wchar_t *text,const char *style)
 	NumLeg++;
 }
 //-----------------------------------------------------------------------------
-void mglGraph::Legend(float x, float y, const char *font, float size, float llen)
+void mglGraph::Legend(mreal x, mreal y, const char *font, mreal size, mreal llen)
 {	Legend(NumLeg,LegStr,LegStl,x,y,font,size,llen);	}
 //-----------------------------------------------------------------------------
-void mglGraph::Legend(int where, const char *font, float size, float llen)
+void mglGraph::Legend(int where, const char *font, mreal size, mreal llen)
 {	Legend(NumLeg,LegStr,LegStl,where,font,size,llen);	}
 //-----------------------------------------------------------------------------
 void mglGraph::ClearLegend()
@@ -756,118 +797,74 @@ void mglGraph::ClearLegend()
 	NumLeg = 0;
 }
 //-----------------------------------------------------------------------------
-void mglGraph::Legend(int n, wchar_t **text,char **style, float x, float y,
-					const char *font, float size, float llen)
-{
-	if(n<1)	{	SetWarn(mglWarnLeg);	return;	}
-	if(isnan(llen))	llen=0.1;
-	static int cgid=1;	StartGroup("Legend",cgid++);
-	float pp[15], r=GetRatio(), rh, rw;
-	if(size<=0)	size = -size*FontSize;
-
-	rh=(r<1?r:1.)*size/12.*(Max.y-Min.y);
-	rw=(r>1?1./r:1.)*size/16.;
-	float w=0, h=fnt->Height(font)*rh, j, dx=Max.x-Min.x;
-	register long i;
-	for(i=0;i<n;i++)
-	{
-		j = fnt->Width(text[i],font)*rw;
-		if(style[i][0]==0)	j -= llen;
-		w = w>j ? w:j;
-	}
-	w = (w + llen*1.1f)*dx;	//   
-
-	if(LegendBox)
-	{
-		pp[2] = pp[5] = pp[8] = pp[11] = pp[14] = Max.z;
-		pp[0] = pp[9] = pp[12] = x;		pp[3] = pp[6] = x+w;
-		pp[1] = pp[4] = pp[13] = y-0.*h;		pp[7] = pp[10] = y+h*n;
-		for(i=0;i<5;i++)	ScalePoint(pp[3*i],pp[3*i+1],pp[3*i+2]);
-		SelectPen(TranspType!=2 ? "k-1":"w-1");
-		curv_plot(5,pp,0);	// bounding rectangle
-		pp[2] = pp[5] = pp[8] = pp[11] = Max.z;
-		pp[0] = pp[6] = x;			pp[3] = pp[9] = x+w;
-		pp[1] = pp[4] = y-0.*h;		pp[7] = pp[10] = y+h*n;
-		DefColor(mglColor(1,1,1),1);
-		for(i=0;i<4;i++)	ScalePoint(pp[3*i],pp[3*i+1],pp[3*i+2]);
-		surf_plot(2,2,pp,0,0);		// white rectangle below it
-	}
-
-	for(i=0;i<n;i++)
-	{
-		char m=SelectPen(style[i]);
-		pp[2] = pp[5] = pp[8] = Max.z;
-		pp[1] = pp[4] = pp[7] = y+i*h+0.45f*h;
-		pp[0] = x+0.1f*llen*dx;	pp[3] = x+0.9f*llen*dx;	pp[6] = x+0.5f*llen*dx;
-		ScalePoint(pp[0],pp[1],pp[2]);
-		ScalePoint(pp[3],pp[4],pp[5]);
-		ScalePoint(pp[6],pp[7],pp[8]);
-
-		curv_plot(2,pp,0);
-		if(m)	Mark(pp[6],pp[7],pp[8],m);
-		SelectPen(TranspType!=2 ? "k-1":"w-1");
-		Putsw(mglPoint(x+(style[i][0]!=0?llen:0)*dx, y+i*h+0.3f*h, Max.z),text[i], font, size,'n');
-	}
-	EndGroup();
-}
-//-----------------------------------------------------------------------------
 void mglGraph::Legend(int n, wchar_t **text,char **style, int where,
-					const char *font, float size, float llen)
+					const char *font, mreal size, mreal llen)
 {
 	if(n<1)	{	SetWarn(mglWarnLeg);	return;	}
 	if(isnan(llen))	llen=0.1;
 	if(size<0)	size = -size*FontSize;
-	float w=0, r=GetRatio(), rh, rw;
+	mreal w=0, r=GetRatio(), rh, rw;
 
-	rh=(r<1?r:1.)*size/12.*(Max.y-Min.y);	// 12 = 16/1.4
-	rw=(r>1?1./r:1.)*size/16.;
-	float h=fnt->Height(font)*rh, x, y, dx = 0.03*(Max.x-Min.x), dy = 0.03*(Max.y-Min.y);
-	for(long i=0;i<n;i++)
+	rh=(r<1?r:1.)*size/6.;	// 6 = 12/2 = (16/1.4)/2
+	rw=(r>1?1./r:1.)*size/8.;
+	mreal h=fnt->Height(font)*rh, x, y, dx = 0.06, dy = 0.06;
+	for(long i=0;i<n;i++)		// find text length
 	{
 		x = fnt->Width(text[i],font)*rw;
 		if(style[i][0]==0)	x -= llen;
 		w = w>x ? w:x;
 	}
-	w = (w + 1.1f*llen)*(Max.x-Min.x);	//   
+	w = (w + 1.1f*llen*1.5);	// add space for lines
 
 	switch(where)
 	{
-	case 0:		x = Min.x+dx;	y = Min.y+dy;		break;
-	case 1:		x = Max.x-w-dx;	y = Min.y+dy;		break;
-	case 2:		x = Min.x+dx;	y = Max.y-h*n-dy;	break;
-	default:	x = Max.x-w-dx;	y = Max.y-h*n-dy;	break;
+	case 0:		x = dx-1;	y = dy-1;		break;
+	case 1:		x = 1-w-dx;	y = dy-1;		break;
+	case 2:		x = dx-1;	y = 1-h*n-dy;	break;
+	default:	x = 1-w-dx;	y = 1-h*n-dy;	break;
 	}
-	Legend(n,text,style,x,y,font,size,llen);
+	Legend(n,text,style,(x+1)/2,(y+1)/2,font,size,llen);
 }
 //-----------------------------------------------------------------------------
 void mglGraph::Ternary(bool t)
 {
 	static mglPoint x1(-1,-1,-1),x2(1,1,1),o(NAN,NAN,NAN);
+	static bool c = true;
 	TernAxis = t;
 	if(t)
 	{
-		x1 = Min;	x2 = Max;	o = Org;
+		x1 = Min;	x2 = Max;	o = Org;	c = Cut;
 		Cut = false;	OrgT = mglPoint(NAN,NAN,NAN);
 		Axis(mglPoint(0,0,0),mglPoint(1,1,1),mglPoint(0,0,0));
 	}
-	else	Axis(x1,x2,o);
+	else	{	Axis(x1,x2,o);	Cut = c;	}
 }
 //-----------------------------------------------------------------------------
-void mglGraph::AxisTY(bool text)
+void mglGraph::DrawTTick(mreal y, mreal x0, mreal z0, mreal dx, mreal dz, int f)
 {
-	float pp[3*31];
+	mreal pp[9],ff=TickLen/sqrt(1.+f*st_t);
+	pp[0]=x0-y-dx*ff;	pp[1]=y;	pp[2]=z0;
+	pp[3]=x0-y;			pp[4]=y;	pp[5]=z0;
+	pp[6]=x0-y;			pp[7]=y;	pp[8]=z0+dz*ff;
+	if(*TickStl && !f)	SelectPen(TickStl);
+	if(*SubTStl && f)	SelectPen(SubTStl);
+	DrawTick(pp,false);
+}
+void mglGraph::AxisT(bool text)
+{
+	mreal pp[3*31];
 	bool tt[31];
 	wchar_t str[64];
 	long i;
-	float y0,z0,y,ddx,ddy,ddz;
-	y0 = GetOrgY('y');	z0 = GetOrgZ('y');
+	mreal z0=GetOrgZ('z'), y, ddx, ddy, ddz;
 	ddx = dx>=0 ? dx : mgl_okrugl((Min.x-Max.x)/(dx+1),3);
 	ddy = dy>=0 ? dy : mgl_okrugl((Min.y-Max.y)/(dy+1),3);
 	ddz = dz>=0 ? dz : mgl_okrugl((Min.z-Max.z)/(dz+1),3);
 	if(ddx==0)	ddx=(Max.x-Min.x)/2;
-//	if(ddy==0)	ddy=(Max.y-Min.y)/2;
+	if(ddy==0)	ddy=(Max.y-Min.y)/2;
 	if(ddz==0)	ddz=(Max.z-Min.z)/2;
 	SelectPen(TranspType!=2 ? "k-1":"w-1");
+
 	for(i=0;i<31;i++)	//  
 	{
 		pp[3*i]=Max.x+(Min.x-Max.x)*i/30.;
@@ -876,85 +873,38 @@ void mglGraph::AxisTY(bool text)
 		tt[i] = ScalePoint(pp[3*i],pp[3*i+1],pp[3*i+2]);
 	}
 	curv_plot(31,pp,tt);
-
-	if(ddy>0)	// ticks drawing
-	{
-		NSy = NSy<0 ? 0 : NSy;
-		for(y=0;y<=Max.y+(Max.y-Min.y)*1e-6;y+=ddy)
-		{
-			pp[0]=Max.x-y-ddx/10;pp[1]=y;	pp[2]=z0;
-			pp[3]=Max.x-y;		pp[4]=y;	pp[5]=z0;
-			pp[6]=Max.x-y;		pp[7]=y;	pp[8]=z0+ddz/10;
-			if(y>Min.y && y<Max.y)	DrawTick(pp,false);
-			swprintf(str,64,L"%.2g",y);		wcstrim_mgl(str);
-			if(text)	Putsw(mglPoint(Max.x-y,y,z0),str,FontDef,FontSize,'y');
-		}
-		if(NSy>0)
-		{
-			ddy /= (NSy+1);	// subticks drawing
-			for(y=0;y<=Max.y;y+=ddy)
-			{
-				pp[0]=Max.x-y-ddx/14;pp[1]=y;	pp[2]=z0;
-				pp[3]=Max.x-y;		pp[4]=y;	pp[5]=z0;
-				pp[6]=Max.x-y;		pp[7]=y;	pp[8]=z0+ddz/14;
-				DrawTick(pp,true);
-			}
-		}
-	}
-}
-//-----------------------------------------------------------------------------
-void mglGraph::AxisTZ(bool text)
-{
-	float pp[3*31];
-	bool tt[31];
-	wchar_t str[64];
-	long i;
-	float x0,z0,z,ddx,ddy,ddz;
-	x0 = GetOrgX('t');	z0 = GetOrgZ('t');
-	ddx = dx>=0 ? dx : mgl_okrugl((Min.x-Max.x)/(dx+1),3);
-	ddy = dy>=0 ? dy : mgl_okrugl((Min.y-Max.y)/(dy+1),3);
-	ddz = dz>=0 ? dz : mgl_okrugl((Min.z-Max.z)/(dz+1),3);
-	if(ddx==0)	ddx=(Max.x-Min.x)/2;
-	if(ddy==0)	ddy=(Max.y-Min.y)/2;
-//	if(ddz==0)	ddz=(Max.z-Min.z)/2;
-	SelectPen(TranspType!=2 ? "k-1":"w-1");
 	for(i=0;i<31;i++)	//  
 	{
-		pp[3*i]=x0;
+		pp[3*i]=Min.x;
 		pp[3*i+1]=Max.y+(Min.y-Max.y)*i/30.;
 		pp[3*i+2]=z0;
 		tt[i] = ScalePoint(pp[3*i],pp[3*i+1],pp[3*i+2]);
 	}
 	curv_plot(31,pp,tt);
 
-	if(ddz>0)	// ticks drawing
+	NSy = NSy<0 ? 0 : NSy;
+	for(y=0;y<=Max.y+(Max.y-Min.y)*1e-6;y+=ddy)
 	{
-		NSz = NSz<0 ? 0 : NSz;
-//		z0 = z0 - ddz*floor((z0-Min.z)/ddz+1e-3);
-		for(z=0;z<=Max.z+(Max.z-Min.z)*1e-6;z+=ddz)
+		if(y>Min.y && y<Max.y)	DrawTTick(y,Max.x,z0,ddx,ddz);
+		mglprintf(str,64,L"%.2g",y);		wcstrim_mgl(str);
+		if(text)	Putsw(mglPoint(Max.x-y,y,z0),str,FontDef,FontSize,'t');
+
+		if(y>Min.y && y<Max.y)	DrawYTick(Max.y-y,Min.x,z0,ddx,ddz);
+		mglprintf(str,64,L"%.2g",y);		wcstrim_mgl(str);
+		if(text)	Putsw(mglPoint(Min.x,Max.y-y,z0),str,FontDef,FontSize,'y');
+	}
+	if(NSy>0)
+	{
+		ddy /= (NSy+1);	// subticks drawing
+		for(y=0;y<=Max.y;y+=ddy)
 		{
-			pp[0]=x0+ddx/10;	pp[1]=Max.y-z-ddy/10;pp[2]=z0;
-			pp[3]=x0;			pp[4]=Max.y-z;		pp[5]=z0;
-			pp[6]=x0+ddx/10;	pp[7]=Max.y-z;		pp[8]=z0;
-			if(z>Min.z && z<Max.z)	DrawTick(pp,false);
-			swprintf(str, 64, L"%.2g",z);	wcstrim_mgl(str);
-			if(text)	Putsw(mglPoint(x0,Max.y-z,z0),str,FontDef,FontSize,'z');
-		}
-		if(NSz>0)
-		{
-			ddz /= (NSz+1);	// subticks drawing
-			for(z=0;z<=Max.z;z+=ddz)
-			{
-				pp[0]=x0+ddx/14;	pp[1]=Max.y-z-ddy/14;	pp[2]=z0;
-				pp[3]=x0;	pp[4]=Max.y-z;			pp[5]=z0;
-				pp[6]=x0+ddx/14;	pp[7]=Max.y-z;	pp[8]=z0;
-				DrawTick(pp,true);
-			}
+			DrawTTick(y,Max.x,z0,ddx,ddz,1);
+			DrawYTick(Max.y-y,Min.x,z0,ddx,ddz,1);
 		}
 	}
 }
 //-----------------------------------------------------------------------------
-void mglGraph::SetTicks(char dir, float d, int ns, float org)
+void mglGraph::SetTicks(char dir, mreal d, int ns, mreal org)
 {
 	if(dir=='x')
 	{	dx = d;	NSx = ns;	OrgT.x = org;	if(xnum) delete []xbuf;	xnum=0;	}
@@ -964,7 +914,7 @@ void mglGraph::SetTicks(char dir, float d, int ns, float org)
 	{	dz = d;	NSz = ns;	OrgT.z = org;	if(znum) delete []zbuf;	znum=0;	}
 }
 //-----------------------------------------------------------------------------
-void mglGraph::SetTicksVal(char dir, int n, float *val, const wchar_t **lbl)
+void mglGraph::SetTicksVal(char dir, int n, mreal *val, const wchar_t **lbl)
 {
 	long len=0;
 	register int i;
@@ -1009,7 +959,7 @@ void mglGraph::SetTicksVal(char dir, int n, float *val, const wchar_t **lbl)
 	}
 }
 //-----------------------------------------------------------------------------
-void mglGraph::SetTicksVal(char dir, int n, float *val, const char **lbl)
+void mglGraph::SetTicksVal(char dir, int n, mreal *val, const char **lbl)
 {
 	long len=0;
 	register int i,ll;
@@ -1060,7 +1010,7 @@ void mglGraph::SetTicksVal(char dir, int n, float *val, const char **lbl)
 void mglGraph::SetTicksVal(char dir, int n, double val, const char *lbl, ...)
 {
 	if(n<1)	return;
-	float *v = new float[n];
+	mreal *v = new mreal[n];
 	const char **l = (const char **)malloc(n*sizeof(const char *));
 	v[0] = val;	l[0] = lbl;
 	va_list ap;
@@ -1078,7 +1028,7 @@ void mglGraph::SetTicksVal(char dir, int n, double val, const char *lbl, ...)
 void mgl_set_ticks_val(HMGL gr, char dir, int n, double val, const char *lbl, ...)
 {	// NOTE: I have to repeat the function because I don't know how to pass variable arguments to C++ function
 	if(n<1)	return;
-	float *v = new float[n];
+	mreal *v = new mreal[n];
 	const char **l = (const char **)malloc(n*sizeof(const char *));
 	v[0] = val;	l[0] = lbl;
 	va_list ap;
@@ -1093,9 +1043,9 @@ void mgl_set_ticks_val(HMGL gr, char dir, int n, double val, const char *lbl, ..
 	delete []v;		free(l);
 }
 //-----------------------------------------------------------------------------
-void mglGraph::adjust(char dir, float d)
+void mglGraph::adjust(char dir, mreal d)
 {
-	float n;
+	mreal n;
 	d = fabs(d);	n = floor(log10(d));	d = floor(d*pow(10,-n));
 	if(d<4)		SetTicks(dir,0.5*pow(10,n),4,0);
 	else if(d<7)SetTicks(dir,pow(10,n),4,0);
