@@ -25,7 +25,7 @@
 //	DensX, DensY, DensZ series
 //
 //-----------------------------------------------------------------------------
-void mglGraph::DensX(const mglData &a, const char *sch,float sVal)
+void mglGraph::DensX(const mglData &a, const char *sch,mreal sVal)
 {
 	register long i,j,i0,k,n=a.nx,m=a.ny;
 	mglColor col;
@@ -34,7 +34,7 @@ void mglGraph::DensX(const mglData &a, const char *sch,float sVal)
 	if(sVal<Min.x || sVal>Max.x){	SetWarn(mglWarnSlc,"DensX");	return;	}
 	static int cgid=1;	StartGroup("DensX",cgid++);
 	SetScheme(sch);
-	float *b = a.a, d;
+	mreal *b = a.a, d;
 	bool bb = false;
 	if(a.nz>1)
 	{
@@ -42,20 +42,20 @@ void mglGraph::DensX(const mglData &a, const char *sch,float sVal)
 		n = a.nz;
 		d = (a.nx-1)*(sVal - Min.x)/(Max.x - Min.x);
 		k = long(d);		d = d - k;
-		b = new float[n*m];
+		b = new mreal[n*m];
 		if(d>0) for(i=0;i<n;i++)	for(j=0;j<m;j++)
 			b[i+n*j] = a.a[k+a.nx*(j+m*i)]*(1-d) + d*a.a[k+1+a.nx*(j+m*i)];
 		else	for(i=0;i<n;i++)	for(j=0;j<m;j++)
 			b[i+n*j] = a.a[k+a.nx*(j+m*i)];
 	}
 
-	float *pp = new float[3*n*m], *cc = new float[4*n*m];
+	mreal *pp = new mreal[3*n*m], *cc = new mreal[4*n*m];
 	for(i=0;i<n;i++)	for(j=0;j<m;j++)	//   
 	{
 		i0 = i+n*j;
 		pp[3*i0+0] = sVal;
-		pp[3*i0+2] = Min.y + (Max.y-Min.y)*float(j)/(m-1.);
-		pp[3*i0+1] = Min.z + (Max.z-Min.z)*float(i)/(n-1.);
+		pp[3*i0+2] = Min.y + (Max.y-Min.y)*mreal(j)/(m-1.);
+		pp[3*i0+1] = Min.z + (Max.z-Min.z)*mreal(i)/(n-1.);
 		col = GetC(b[i+n*j]);
 		cc[4*i0+0] = col.r;	cc[4*i0+1] = col.g;
 		cc[4*i0+2] = col.b;	cc[4*i0+3] = Transparent ? AlphaDef : 1;
@@ -65,7 +65,7 @@ void mglGraph::DensX(const mglData &a, const char *sch,float sVal)
 	delete []pp;	delete []cc;	if(bb)	delete []b;
 }
 //-----------------------------------------------------------------------------
-void mglGraph::DensY(const mglData &a, const char *sch,float sVal)
+void mglGraph::DensY(const mglData &a, const char *sch,mreal sVal)
 {
 	register long i,j,i0,k,n=a.nx,m=a.ny;
 	mglColor col;
@@ -74,7 +74,7 @@ void mglGraph::DensY(const mglData &a, const char *sch,float sVal)
 	if(sVal<Min.y || sVal>Max.y){	SetWarn(mglWarnSlc,"DensY");	return;	}
 	static int cgid=1;	StartGroup("DensY",cgid++);
 	SetScheme(sch);
-	float *b = a.a, d;
+	mreal *b = a.a, d;
 	bool bb = false;
 	if(a.nz>1)
 	{
@@ -82,20 +82,20 @@ void mglGraph::DensY(const mglData &a, const char *sch,float sVal)
 		m = a.nz;
 		d = (a.ny-1)*(sVal - Min.y)/(Max.y - Min.y);
 		k = long(d);		d = d - k;
-		b = new float[n*m];
+		b = new mreal[n*m];
 		if(d>0) for(i=0;i<n;i++)	for(j=0;j<m;j++)
 			b[i+n*j] = a.a[i+n*(k+a.ny*j)]*(1-d) + d*a.a[i+n*(k+1+a.ny*j)];
 		else	for(i=0;i<n;i++)	for(j=0;j<m;j++)
 			b[i+n*j] = a.a[i+n*(k+a.ny*j)];
 	}
 
-	float *pp = new float[3*n*m], *cc = new float[4*n*m];
+	mreal *pp = new mreal[3*n*m], *cc = new mreal[4*n*m];
 	for(i=0;i<n;i++)	for(j=0;j<m;j++)	//   
 	{
 		i0 = i+n*j;
-		pp[3*i0+0] = Min.x + (Max.x-Min.x)*float(i)/(n-1.);
+		pp[3*i0+0] = Min.x + (Max.x-Min.x)*mreal(i)/(n-1.);
 		pp[3*i0+1] = sVal;
-		pp[3*i0+2] = Min.z + (Max.z-Min.z)*float(j)/(m-1.);
+		pp[3*i0+2] = Min.z + (Max.z-Min.z)*mreal(j)/(m-1.);
 		col = GetC(b[i+n*j]);
 		cc[4*i0+0] = col.r;	cc[4*i0+1] = col.g;
 		cc[4*i0+2] = col.b;	cc[4*i0+3] = Transparent ? AlphaDef : 1;
@@ -105,7 +105,7 @@ void mglGraph::DensY(const mglData &a, const char *sch,float sVal)
 	delete []pp;	delete []cc;	if(bb)	delete []b;
 }
 //-----------------------------------------------------------------------------
-void mglGraph::DensZ(const mglData &a, const char *sch,float sVal)
+void mglGraph::DensZ(const mglData &a, const char *sch,mreal sVal)
 {
 	register long i,j,i0,k,n=a.nx,m=a.ny;
 	mglColor col;
@@ -114,26 +114,26 @@ void mglGraph::DensZ(const mglData &a, const char *sch,float sVal)
 	if(sVal<Min.z || sVal>Max.z){	SetWarn(mglWarnSlc,"DensZ");	return;	}
 	static int cgid=1;	StartGroup("DensZ",cgid++);
 	SetScheme(sch);
-	float *b = a.a, d;
+	mreal *b = a.a, d;
 	bool bb = false;
 	if(a.nz>1)
 	{
 		bb = true;
 		d = (a.nz-1)*(sVal - Min.z)/(Max.y - Min.z);
 		k = long(d);		d = d - k;
-		b = new float[n*m];
+		b = new mreal[n*m];
 		if(d>0) for(i=0;i<n;i++)	for(j=0;j<m;j++)
 			b[i+n*j] = a.a[i+n*(j+m*k)]*(1-d) + d*a.a[i+n*(j+m*k+m)];
 		else	for(i=0;i<n;i++)	for(j=0;j<m;j++)
 			b[i+n*j] = a.a[i+n*(j+m*k)];
 	}
 
-	float *pp = new float[3*n*m], *cc = new float[4*n*m];
+	mreal *pp = new mreal[3*n*m], *cc = new mreal[4*n*m];
 	for(i=0;i<n;i++)	for(j=0;j<m;j++)	//   
 	{
 		i0 = i+n*j;
-		pp[3*i0+0] = Min.x + (Max.x-Min.x)*float(i)/(n-1.);
-		pp[3*i0+1] = Min.y + (Max.y-Min.y)*float(j)/(m-1.);
+		pp[3*i0+0] = Min.x + (Max.x-Min.x)*mreal(i)/(n-1.);
+		pp[3*i0+1] = Min.y + (Max.y-Min.y)*mreal(j)/(m-1.);
 		pp[3*i0+2] = sVal;
 		col = GetC(b[i+n*j]);
 		cc[4*i0+0] = col.r;	cc[4*i0+1] = col.g;
@@ -148,31 +148,31 @@ void mglGraph::DensZ(const mglData &a, const char *sch,float sVal)
 //	ContX, ContY, ContZ series
 //
 //-----------------------------------------------------------------------------
-void mglGraph::ContX(const mglData &z, const char *sch, float zVal, int Num)
+void mglGraph::ContX(const mglData &z, const char *sch, mreal zVal, int Num)
 {
 	if(Num<1)	{	SetWarn(mglWarnCnt,"ContX");	return;	}
 	mglData v(Num);
-	for(long i=0;i<Num;i++)	v.a[i] = Cmin + (Cmax-Cmin)*float(i+1)/(Num+1);
+	for(long i=0;i<Num;i++)	v.a[i] = Cmin + (Cmax-Cmin)*mreal(i+1)/(Num+1);
 	ContX(v,z,sch,zVal);
 }
 //-----------------------------------------------------------------------------
-void mglGraph::ContY(const mglData &z, const char *sch, float zVal, int Num)
+void mglGraph::ContY(const mglData &z, const char *sch, mreal zVal, int Num)
 {
 	if(Num<1)	{	SetWarn(mglWarnCnt,"ContY");	return;	}
 	mglData v(Num);
-	for(long i=0;i<Num;i++)	v.a[i] = Cmin + (Cmax-Cmin)*float(i+1)/(Num+1);
+	for(long i=0;i<Num;i++)	v.a[i] = Cmin + (Cmax-Cmin)*mreal(i+1)/(Num+1);
 	ContY(v,z,sch,zVal);
 }
 //-----------------------------------------------------------------------------
-void mglGraph::ContZ(const mglData &z, const char *sch, float zVal, int Num)
+void mglGraph::ContZ(const mglData &z, const char *sch, mreal zVal, int Num)
 {
 	if(Num<1)	{	SetWarn(mglWarnCnt,"ContZ");	return;	}
 	mglData v(Num);
-	for(long i=0;i<Num;i++)	v.a[i] = Cmin + (Cmax-Cmin)*float(i+1)/(Num+1);
+	for(long i=0;i<Num;i++)	v.a[i] = Cmin + (Cmax-Cmin)*mreal(i+1)/(Num+1);
 	ContZ(v,z,sch,zVal);
 }
 //-----------------------------------------------------------------------------
-void mglGraph::ContX(const mglData &v, const mglData &a, const char *sch, float sVal)
+void mglGraph::ContX(const mglData &v, const mglData &a, const char *sch, mreal sVal)
 {
 	register long i,j,k,n=a.nx,m=a.ny;
 	mglColor col;
@@ -184,7 +184,7 @@ void mglGraph::ContX(const mglData &v, const mglData &a, const char *sch, float
 	if(sch && strchr(sch,'t'))	text=-1;
 	if(sch && strchr(sch,'T'))	text=+1;
 	SetScheme(sch);
-	float *b = a.a, d;
+	mreal *b = a.a, d;
 	bool bb = false;
 	if(a.nz>1)
 	{
@@ -192,13 +192,13 @@ void mglGraph::ContX(const mglData &v, const mglData &a, const char *sch, float
 		n = a.nz;
 		d = (a.ny-1)*(sVal - Min.x)/(Max.x - Min.x);
 		k = long(d);		d = d - k;
-		b = new float[n*m];
+		b = new mreal[n*m];
 		if(d>0)	for(j=0;j<m;j++) for(i=0;i<n;i++)
 			b[i+n*j] = a.a[k+a.nx*(j+m*i)]*(1-d) + d*a.a[k+1+a.nx*(j+m*i)];
 		else	for(i=0;i<n;i++)	for(j=0;j<m;j++)
 			b[i+n*j] = a.a[k+a.nx*(j+m*i)];
 	}
-	float *xx = new float[n*m], *yy = new float[n*m], *zz = new float[n*m];
+	mreal *xx = new mreal[n*m], *yy = new mreal[n*m], *zz = new mreal[n*m];
 	for(i=0;i<n;i++)	for(j=0;j<m;j++)
 	{
 		k = i+n*j;		yy[k] = Min.y + (Max.y-Min.y)*i/(n-1.);
@@ -213,7 +213,7 @@ void mglGraph::ContX(const mglData &v, const mglData &a, const char *sch, float
 	delete []xx;	delete []yy;	delete []zz;	if(bb)	delete []b;
 }
 //-----------------------------------------------------------------------------
-void mglGraph::ContY(const mglData &v, const mglData &a, const char *sch, float sVal)
+void mglGraph::ContY(const mglData &v, const mglData &a, const char *sch, mreal sVal)
 {
 	register long i,j,k,n=a.nx,m=a.ny;
 	mglColor col;
@@ -225,7 +225,7 @@ void mglGraph::ContY(const mglData &v, const mglData &a, const char *sch, float
 	if(sch && strchr(sch,'t'))	text=-1;
 	if(sch && strchr(sch,'T'))	text=+1;
 	SetScheme(sch);
-	float *b = a.a, d;
+	mreal *b = a.a, d;
 	bool bb = false;
 	if(a.nz>1)
 	{
@@ -233,13 +233,13 @@ void mglGraph::ContY(const mglData &v, const mglData &a, const char *sch, float
 		m = a.nz;
 		d = (a.ny-1)*(sVal - Min.y)/(Max.y - Min.y);
 		k = long(d);		d = d - k;
-		b = new float[n*m];
+		b = new mreal[n*m];
 		if(d>0) for(i=0;i<n;i++)	for(j=0;j<m;j++)
 			b[i+n*j] = a.a[i+n*(k+a.ny*j)]*(1-d) + d*a.a[i+n*(k+1+a.ny*j)];
 		else	for(i=0;i<n;i++)	for(j=0;j<m;j++)
 			b[i+n*j] = a.a[i+n*(k+a.ny*j)];
 	}
-	float *xx = new float[n*m], *yy = new float[n*m], *zz = new float[n*m];
+	mreal *xx = new mreal[n*m], *yy = new mreal[n*m], *zz = new mreal[n*m];
 	for(i=0;i<n;i++)	for(j=0;j<m;j++)
 	{
 		k = i+n*j;		zz[k] = Min.z + (Max.z-Min.z)*j/(m-1.);
@@ -254,7 +254,7 @@ void mglGraph::ContY(const mglData &v, const mglData &a, const char *sch, float
 	delete []xx;	delete []yy;	delete []zz;	if(bb)	delete []b;
 }
 //-----------------------------------------------------------------------------
-void mglGraph::ContZ(const mglData &v, const mglData &a, const char *sch, float sVal)
+void mglGraph::ContZ(const mglData &v, const mglData &a, const char *sch, mreal sVal)
 {
 	register long i,j,k,n=a.nx,m=a.ny;
 	mglColor col;
@@ -266,20 +266,20 @@ void mglGraph::ContZ(const mglData &v, const mglData &a, const char *sch, float
 	if(sch && strchr(sch,'t'))	text=-1;
 	if(sch && strchr(sch,'T'))	text=+1;
 	SetScheme(sch);
-	float *b = a.a, d;
+	mreal *b = a.a, d;
 	bool bb = false;
 	if(a.nz>1)
 	{
 		bb = true;
 		d = (a.nz-1)*(sVal - Min.z)/(Max.y - Min.z);
 		k = long(d);		d = d - k;
-		b = new float[n*m];
+		b = new mreal[n*m];
 		if(d>0) for(i=0;i<n;i++)	for(j=0;j<m;j++)
 			b[i+n*j] = a.a[i+n*(j+m*k)]*(1-d) + d*a.a[i+n*(j+m*k+m)];
 		else	for(i=0;i<n;i++)	for(j=0;j<m;j++)
 			b[i+n*j] = a.a[i+n*(j+m*k)];
 	}
-	float *xx = new float[n*m], *yy = new float[n*m], *zz = new float[n*m];
+	mreal *xx = new mreal[n*m], *yy = new mreal[n*m], *zz = new mreal[n*m];
 	for(i=0;i<n;i++)	for(j=0;j<m;j++)
 	{
 		k = i+n*j;		yy[k] = Min.y + (Max.y-Min.y)*j/(m-1.);
@@ -298,31 +298,31 @@ void mglGraph::ContZ(const mglData &v, const mglData &a, const char *sch, float
 //	ContFX, ContFY, ContFZ series
 //
 //-----------------------------------------------------------------------------
-void mglGraph::ContFX(const mglData &z, const char *sch, float zVal, int Num)
+void mglGraph::ContFX(const mglData &z, const char *sch, mreal zVal, int Num)
 {
 	if(Num<1)	{	SetWarn(mglWarnCnt,"ContFX");	return;	}
 	mglData v(Num);
-	for(long i=0;i<Num;i++)	v.a[i] = Cmin + (Cmax-Cmin)*float(i+1)/(Num+1);
+	for(long i=0;i<Num;i++)	v.a[i] = Cmin + (Cmax-Cmin)*mreal(i+1)/(Num+1);
 	ContFX(v,z,sch,zVal);
 }
 //-----------------------------------------------------------------------------
-void mglGraph::ContFY(const mglData &z, const char *sch, float zVal, int Num)
+void mglGraph::ContFY(const mglData &z, const char *sch, mreal zVal, int Num)
 {
 	if(Num<1)	{	SetWarn(mglWarnCnt,"ContFYX");	return;	}
 	mglData v(Num);
-	for(long i=0;i<Num;i++)	v.a[i] = Cmin + (Cmax-Cmin)*float(i+1)/(Num+1);
+	for(long i=0;i<Num;i++)	v.a[i] = Cmin + (Cmax-Cmin)*mreal(i+1)/(Num+1);
 	ContFY(v,z,sch,zVal);
 }
 //-----------------------------------------------------------------------------
-void mglGraph::ContFZ(const mglData &z, const char *sch, float zVal, int Num)
+void mglGraph::ContFZ(const mglData &z, const char *sch, mreal zVal, int Num)
 {
 	if(Num<1)	{	SetWarn(mglWarnCnt,"ContFZ");	return;	}
 	mglData v(Num);
-	for(long i=0;i<Num;i++)	v.a[i] = Cmin + (Cmax-Cmin)*float(i+1)/(Num+1);
+	for(long i=0;i<Num;i++)	v.a[i] = Cmin + (Cmax-Cmin)*mreal(i+1)/(Num+1);
 	ContFZ(v,z,sch,zVal);
 }
 //-----------------------------------------------------------------------------
-void mglGraph::ContFX(const mglData &v, const mglData &a, const char *sch, float sVal)
+void mglGraph::ContFX(const mglData &v, const mglData &a, const char *sch, mreal sVal)
 {
 	register long i,j,k,n=a.nx,m=a.ny;
 	mglColor col;
@@ -334,7 +334,7 @@ void mglGraph::ContFX(const mglData &v, const mglData &a, const char *sch, float
 	if(sch && strchr(sch,'t'))	text=-1;
 	if(sch && strchr(sch,'T'))	text=+1;
 	SetScheme(sch);
-	float *b = a.a, d;
+	mreal *b = a.a, d;
 	bool bb = false;
 	if(a.nz>1)
 	{
@@ -342,13 +342,13 @@ void mglGraph::ContFX(const mglData &v, const mglData &a, const char *sch, float
 		n = a.nz;
 		d = (a.ny-1)*(sVal - Min.x)/(Max.x - Min.x);
 		k = long(d);		d = d - k;
-		b = new float[n*m];
+		b = new mreal[n*m];
 		if(d>0)	for(j=0;j<m;j++) for(i=0;i<n;i++)
 			b[i+n*j] = a.a[k+a.nx*(j+m*i)]*(1-d) + d*a.a[k+1+a.nx*(j+m*i)];
 		else	for(i=0;i<n;i++)	for(j=0;j<m;j++)
 			b[i+n*j] = a.a[k+a.nx*(j+m*i)];
 	}
-	float *xx = new float[n*m], *yy = new float[n*m], *zz = new float[n*m];
+	mreal *xx = new mreal[n*m], *yy = new mreal[n*m], *zz = new mreal[n*m];
 	for(i=0;i<n;i++)	for(j=0;j<m;j++)
 	{
 		k = i+n*j;		yy[k] = Min.y + (Max.y-Min.y)*i/(n-1.);
@@ -363,7 +363,7 @@ void mglGraph::ContFX(const mglData &v, const mglData &a, const char *sch, float
 	delete []xx;	delete []yy;	delete []zz;	if(bb)	delete []b;
 }
 //-----------------------------------------------------------------------------
-void mglGraph::ContFY(const mglData &v, const mglData &a, const char *sch, float sVal)
+void mglGraph::ContFY(const mglData &v, const mglData &a, const char *sch, mreal sVal)
 {
 	register long i,j,k,n=a.nx,m=a.ny;
 	mglColor col;
@@ -375,7 +375,7 @@ void mglGraph::ContFY(const mglData &v, const mglData &a, const char *sch, float
 	if(sch && strchr(sch,'t'))	text=-1;
 	if(sch && strchr(sch,'T'))	text=+1;
 	SetScheme(sch);
-	float *b = a.a, d;
+	mreal *b = a.a, d;
 	bool bb = false;
 	if(a.nz>1)
 	{
@@ -383,13 +383,13 @@ void mglGraph::ContFY(const mglData &v, const mglData &a, const char *sch, float
 		m = a.nz;
 		d = (a.ny-1)*(sVal - Min.y)/(Max.y - Min.y);
 		k = long(d);		d = d - k;
-		b = new float[n*m];
+		b = new mreal[n*m];
 		if(d>0) for(i=0;i<n;i++)	for(j=0;j<m;j++)
 			b[i+n*j] = a.a[i+n*(k+a.ny*j)]*(1-d) + d*a.a[i+n*(k+1+a.ny*j)];
 		else	for(i=0;i<n;i++)	for(j=0;j<m;j++)
 			b[i+n*j] = a.a[i+n*(k+a.ny*j)];
 	}
-	float *xx = new float[n*m], *yy = new float[n*m], *zz = new float[n*m];
+	mreal *xx = new mreal[n*m], *yy = new mreal[n*m], *zz = new mreal[n*m];
 	for(i=0;i<n;i++)	for(j=0;j<m;j++)
 	{
 		k = i+n*j;		zz[k] = Min.z + (Max.z-Min.z)*j/(m-1.);
@@ -404,7 +404,7 @@ void mglGraph::ContFY(const mglData &v, const mglData &a, const char *sch, float
 	delete []xx;	delete []yy;	delete []zz;	if(bb)	delete []b;
 }
 //-----------------------------------------------------------------------------
-void mglGraph::ContFZ(const mglData &v, const mglData &a, const char *sch, float sVal)
+void mglGraph::ContFZ(const mglData &v, const mglData &a, const char *sch, mreal sVal)
 {
 	register long i,j,k,n=a.nx,m=a.ny;
 	mglColor col;
@@ -416,20 +416,20 @@ void mglGraph::ContFZ(const mglData &v, const mglData &a, const char *sch, float
 	if(sch && strchr(sch,'t'))	text=-1;
 	if(sch && strchr(sch,'T'))	text=+1;
 	SetScheme(sch);
-	float *b = a.a, d;
+	mreal *b = a.a, d;
 	bool bb = false;
 	if(a.nz>1)
 	{
 		bb = true;
 		d = (a.nz-1)*(sVal - Min.z)/(Max.y - Min.z);
 		k = long(d);		d = d - k;
-		b = new float[n*m];
+		b = new mreal[n*m];
 		if(d>0) for(i=0;i<n;i++)	for(j=0;j<m;j++)
 			b[i+n*j] = a.a[i+n*(j+m*k)]*(1-d) + d*a.a[i+n*(j+m*k+m)];
 		else	for(i=0;i<n;i++)	for(j=0;j<m;j++)
 			b[i+n*j] = a.a[i+n*(j+m*k)];
 	}
-	float *xx = new float[n*m], *yy = new float[n*m], *zz = new float[n*m];
+	mreal *xx = new mreal[n*m], *yy = new mreal[n*m], *zz = new mreal[n*m];
 	for(i=0;i<n;i++)	for(j=0;j<m;j++)
 	{
 		k = i+n*j;		yy[k] = Min.y + (Max.y-Min.y)*j/(m-1.);
@@ -447,153 +447,153 @@ void mglGraph::ContFZ(const mglData &v, const mglData &a, const char *sch, float
 //		Combined plotting functions
 //-----------------------------------------------------------------------------
 /// Draw density plot for data a at x = sVal
-void mgl_dens_x(HMGL gr, const HMDT a, const char *sch, float sVal)
+void mgl_dens_x(HMGL gr, const HMDT a, const char *sch, mreal sVal)
 {	if(gr && a)	gr->DensX(*a, sch, sVal);	}
 /// Draw density plot for data a at y = sVal
-void mgl_dens_y(HMGL gr, const HMDT a, const char *sch, float sVal)
+void mgl_dens_y(HMGL gr, const HMDT a, const char *sch, mreal sVal)
 {	if(gr && a)	gr->DensY(*a, sch, sVal);	}
 /// Draw density plot for data a at z = sVal
-void mgl_dens_z(HMGL gr, const HMDT a, const char *sch, float sVal)
+void mgl_dens_z(HMGL gr, const HMDT a, const char *sch, mreal sVal)
 {	if(gr && a)	gr->DensZ(*a, sch, sVal);	}
 /// Draw several contour plots for data a at x = sVal
-void mgl_cont_x(HMGL gr, const HMDT a, const char *sch, float sVal, int Num)
+void mgl_cont_x(HMGL gr, const HMDT a, const char *sch, mreal sVal, int Num)
 {	if(gr && a)	gr->ContX(*a, sch, sVal, Num);	}
 /// Draw several contour plots for data a at y = sVal
-void mgl_cont_y(HMGL gr, const HMDT a, const char *sch, float sVal, int Num)
+void mgl_cont_y(HMGL gr, const HMDT a, const char *sch, mreal sVal, int Num)
 {	if(gr && a)	gr->ContY(*a, sch, sVal, Num);	}
 /// Draw several contour plots for data a at z = sVal
-void mgl_cont_z(HMGL gr, const HMDT a, const char *sch, float sVal, int Num)
+void mgl_cont_z(HMGL gr, const HMDT a, const char *sch, mreal sVal, int Num)
 {	if(gr && a)	gr->ContZ(*a, sch, sVal, Num);	}
 /// Draw contour plots for data a at x = sVal
-void mgl_cont_x_val(HMGL gr, const HMDT v, const HMDT a, const char *sch, float sVal)
+void mgl_cont_x_val(HMGL gr, const HMDT v, const HMDT a, const char *sch, mreal sVal)
 {	if(gr && a && v)	gr->ContX(*v, *a, sch, sVal);	}
 /// Draw contour plots for data a at y = sVal
-void mgl_cont_y_val(HMGL gr, const HMDT v, const HMDT a, const char *sch, float sVal)
+void mgl_cont_y_val(HMGL gr, const HMDT v, const HMDT a, const char *sch, mreal sVal)
 {	if(gr && a && v)	gr->ContY(*v, *a, sch, sVal);	}
 /// Draw contour plots for data a at z = sVal
-void mgl_cont_z_val(HMGL gr, const HMDT v, const HMDT a, const char *sch, float sVal)
+void mgl_cont_z_val(HMGL gr, const HMDT v, const HMDT a, const char *sch, mreal sVal)
 {	if(gr && a && v)	gr->ContZ(*v, *a, sch, sVal);	}
 /// Draw several contour plots for data a at x = sVal
-void mgl_contf_x(HMGL gr, const HMDT a, const char *sch, float sVal, int Num)
+void mgl_contf_x(HMGL gr, const HMDT a, const char *sch, mreal sVal, int Num)
 {	if(gr && a)	gr->ContFX(*a, sch, sVal, Num);	}
 /// Draw several contour plots for data a at y = sVal
-void mgl_contf_y(HMGL gr, const HMDT a, const char *sch, float sVal, int Num)
+void mgl_contf_y(HMGL gr, const HMDT a, const char *sch, mreal sVal, int Num)
 {	if(gr && a)	gr->ContFY(*a, sch, sVal, Num);	}
 /// Draw several contour plots for data a at z = sVal
-void mgl_contf_z(HMGL gr, const HMDT a, const char *sch, float sVal, int Num)
+void mgl_contf_z(HMGL gr, const HMDT a, const char *sch, mreal sVal, int Num)
 {	if(gr && a)	gr->ContFZ(*a, sch, sVal, Num);	}
 /// Draw contour plots for data a at x = sVal
-void mgl_contf_x_val(HMGL gr, const HMDT v, const HMDT a, const char *sch, float sVal)
+void mgl_contf_x_val(HMGL gr, const HMDT v, const HMDT a, const char *sch, mreal sVal)
 {	if(gr && a && v)	gr->ContFX(*v, *a, sch, sVal);	}
 /// Draw contour plots for data a at y = sVal
-void mgl_contf_y_val(HMGL gr, const HMDT v, const HMDT a, const char *sch, float sVal)
+void mgl_contf_y_val(HMGL gr, const HMDT v, const HMDT a, const char *sch, mreal sVal)
 {	if(gr && a && v)	gr->ContFY(*v, *a, sch, sVal);	}
 /// Draw contour plots for data a at z = sVal
-void mgl_contf_z_val(HMGL gr, const HMDT v, const HMDT a, const char *sch, float sVal)
+void mgl_contf_z_val(HMGL gr, const HMDT v, const HMDT a, const char *sch, mreal sVal)
 {	if(gr && a && v)	gr->ContFZ(*v, *a, sch, sVal);	}
 //-----------------------------------------------------------------------------
 //		Combined plotting functions (Fortran)
 //-----------------------------------------------------------------------------
 /// Draw density plot for data a at x = *sVal
-void mgl_dens_x_(uintptr_t *gr, uintptr_t *a, const char *sch, float *sVal,int l)
+void mgl_dens_x_(uintptr_t *gr, uintptr_t *a, const char *sch, mreal *sVal,int l)
 {
 	char *s=new char[l+1];	memcpy(s,sch,l);	s[l]=0;
 	if(gr && a)	_GR_->DensX(_D_(a), s, *sVal);
 	delete []s;
 }
 /// Draw density plot for data a at y = *sVal
-void mgl_dens_y_(uintptr_t *gr, uintptr_t *a, const char *sch, float *sVal,int l)
+void mgl_dens_y_(uintptr_t *gr, uintptr_t *a, const char *sch, mreal *sVal,int l)
 {
 	char *s=new char[l+1];	memcpy(s,sch,l);	s[l]=0;
 	if(gr && a)	_GR_->DensY(_D_(a), s, *sVal);
 	delete []s;
 }
 /// Draw density plot for data a at z = *sVal
-void mgl_dens_z_(uintptr_t *gr, uintptr_t *a, const char *sch, float *sVal,int l)
+void mgl_dens_z_(uintptr_t *gr, uintptr_t *a, const char *sch, mreal *sVal,int l)
 {
 	char *s=new char[l+1];	memcpy(s,sch,l);	s[l]=0;
 	if(gr && a)	_GR_->DensZ(_D_(a), s, *sVal);
 	delete []s;
 }
 /// Draw several contour plots for data a at x = *sVal
-void mgl_cont_x_(uintptr_t *gr, uintptr_t *a, const char *sch, float *sVal, int *Num,int l)
+void mgl_cont_x_(uintptr_t *gr, uintptr_t *a, const char *sch, mreal *sVal, int *Num,int l)
 {
 	char *s=new char[l+1];	memcpy(s,sch,l);	s[l]=0;
 	if(gr && a)	_GR_->ContX(_D_(a), s, *sVal, *Num);
 	delete []s;
 }
 /// Draw several contour plots for data a at y = *sVal
-void mgl_cont_y_(uintptr_t *gr, uintptr_t *a, const char *sch, float *sVal, int *Num,int l)
+void mgl_cont_y_(uintptr_t *gr, uintptr_t *a, const char *sch, mreal *sVal, int *Num,int l)
 {
 	char *s=new char[l+1];	memcpy(s,sch,l);	s[l]=0;
 	if(gr && a)	_GR_->ContY(_D_(a), s, *sVal, *Num);
 	delete []s;
 }
 /// Draw several contour plots for data a at z = *sVal
-void mgl_cont_z_(uintptr_t *gr, uintptr_t *a, const char *sch, float *sVal, int *Num,int l)
+void mgl_cont_z_(uintptr_t *gr, uintptr_t *a, const char *sch, mreal *sVal, int *Num,int l)
 {
 	char *s=new char[l+1];	memcpy(s,sch,l);	s[l]=0;
 	if(gr && a)	_GR_->ContZ(_D_(a), s, *sVal, *Num);
 	delete []s;
 }
 /// Draw contour plots for data a at x = *sVal
-void mgl_cont_x_val_(uintptr_t *gr, uintptr_t *v, uintptr_t *a, const char *sch, float *sVal,int l)
+void mgl_cont_x_val_(uintptr_t *gr, uintptr_t *v, uintptr_t *a, const char *sch, mreal *sVal,int l)
 {
 	char *s=new char[l+1];	memcpy(s,sch,l);	s[l]=0;
 	if(gr && a && v)	_GR_->ContX(_D_(v), _D_(a), s, *sVal);
 	delete []s;
 }
 /// Draw contour plots for data a at y = *sVal
-void mgl_cont_y_val_(uintptr_t *gr, uintptr_t *v, uintptr_t *a, const char *sch, float *sVal,int l)
+void mgl_cont_y_val_(uintptr_t *gr, uintptr_t *v, uintptr_t *a, const char *sch, mreal *sVal,int l)
 {
 	char *s=new char[l+1];	memcpy(s,sch,l);	s[l]=0;
 	if(gr && a && v)	_GR_->ContY(_D_(v), _D_(a), s, *sVal);
 	delete []s;
 }
 /// Draw contour plots for data a at z = *sVal
-void mgl_cont_z_val_(uintptr_t *gr, uintptr_t *v, uintptr_t *a, const char *sch, float *sVal,int l)
+void mgl_cont_z_val_(uintptr_t *gr, uintptr_t *v, uintptr_t *a, const char *sch, mreal *sVal,int l)
 {
 	char *s=new char[l+1];	memcpy(s,sch,l);	s[l]=0;
 	if(gr && a && v)	_GR_->ContZ(_D_(v), _D_(a), s, *sVal);
 	delete []s;
 }
 /// Draw several contour plots for data a at x = *sVal
-void mgl_contf_x_(uintptr_t *gr, uintptr_t *a, const char *sch, float *sVal, int *Num,int l)
+void mgl_contf_x_(uintptr_t *gr, uintptr_t *a, const char *sch, mreal *sVal, int *Num,int l)
 {
 	char *s=new char[l+1];	memcpy(s,sch,l);	s[l]=0;
 	if(gr && a)	_GR_->ContFX(_D_(a), s, *sVal, *Num);
 	delete []s;
 }
 /// Draw several contour plots for data a at y = *sVal
-void mgl_contf_y_(uintptr_t *gr, uintptr_t *a, const char *sch, float *sVal, int *Num,int l)
+void mgl_contf_y_(uintptr_t *gr, uintptr_t *a, const char *sch, mreal *sVal, int *Num,int l)
 {
 	char *s=new char[l+1];	memcpy(s,sch,l);	s[l]=0;
 	if(gr && a)	_GR_->ContFY(_D_(a), s, *sVal, *Num);
 	delete []s;
 }
 /// Draw several contour plots for data a at z = *sVal
-void mgl_contf_z_(uintptr_t *gr, uintptr_t *a, const char *sch, float *sVal, int *Num,int l)
+void mgl_contf_z_(uintptr_t *gr, uintptr_t *a, const char *sch, mreal *sVal, int *Num,int l)
 {
 	char *s=new char[l+1];	memcpy(s,sch,l);	s[l]=0;
 	if(gr && a)	_GR_->ContFZ(_D_(a), s, *sVal, *Num);
 	delete []s;
 }
 /// Draw contour plots for data a at x = *sVal
-void mgl_contf_x_val_(uintptr_t *gr, uintptr_t *v, uintptr_t *a, const char *sch, float *sVal,int l)
+void mgl_contf_x_val_(uintptr_t *gr, uintptr_t *v, uintptr_t *a, const char *sch, mreal *sVal,int l)
 {
 	char *s=new char[l+1];	memcpy(s,sch,l);	s[l]=0;
 	if(gr && a && v)	_GR_->ContFX(_D_(v), _D_(a), s, *sVal);
 	delete []s;
 }
 /// Draw contour plots for data a at y = *sVal
-void mgl_contf_y_val_(uintptr_t *gr, uintptr_t *v, uintptr_t *a, const char *sch, float *sVal,int l)
+void mgl_contf_y_val_(uintptr_t *gr, uintptr_t *v, uintptr_t *a, const char *sch, mreal *sVal,int l)
 {
 	char *s=new char[l+1];	memcpy(s,sch,l);	s[l]=0;
 	if(gr && a && v)	_GR_->ContFY(_D_(v), _D_(a), s, *sVal);
 	delete []s;
 }
 /// Draw contour plots for data a at z = *sVal
-void mgl_contf_z_val_(uintptr_t *gr, uintptr_t *v, uintptr_t *a, const char *sch, float *sVal,int l)
+void mgl_contf_z_val_(uintptr_t *gr, uintptr_t *v, uintptr_t *a, const char *sch, mreal *sVal,int l)
 {
 	char *s=new char[l+1];	memcpy(s,sch,l);	s[l]=0;
 	if(gr && a && v)	_GR_->ContFZ(_D_(v), _D_(a), s, *sVal);
