@@ -26,17 +26,17 @@
 //	Flow series
 //
 //-----------------------------------------------------------------------------
-void mglGraph::flow(bool simple, float zVal, float u, float v,
+void mglGraph::flow(bool simple, mreal zVal, mreal u, mreal v,
 					const mglData &x, const mglData &y, const mglData &ax, const mglData &ay)
 {
 	long n=10*(ax.nx+ax.ny);
 	bool both = x.nx==ax.nx && y.nx==ax.nx && x.ny==ax.ny && y.ny==ax.ny;
-	float *pp, *cc;
+	mreal *pp, *cc;
 	bool *tt;
-	pp = (float *)malloc(3*n*sizeof(float));
-	cc = (float *)malloc(n*sizeof(float));
+	pp = (mreal *)malloc(3*n*sizeof(mreal));
+	cc = (mreal *)malloc(n*sizeof(mreal));
 	tt = (bool  *)malloc(n*sizeof(bool));
-	float dt = 0.5/(ax.nx > ax.ny ? ax.nx : ax.ny),e,f,g,ff[4],gg[4],h,s=1;
+	mreal dt = 0.5/(ax.nx > ax.ny ? ax.nx : ax.ny),e,f,g,ff[4],gg[4],h,s=1;
 	if(u<0 || v<0)	{	dt = -dt;	u = -u;		v = -v;		s = -1;}
 	register long k=0,m;
 	bool end = false;
@@ -44,8 +44,8 @@ void mglGraph::flow(bool simple, float zVal, float u, float v,
 		if(k>=n)	// resize array if need
 		{
 			n += 10*(ax.nx+ax.ny);
-			pp = (float *)realloc(pp,3*n*sizeof(float));
-			cc = (float *)realloc(cc,n*sizeof(float));
+			pp = (mreal *)realloc(pp,3*n*sizeof(mreal));
+			cc = (mreal *)realloc(cc,n*sizeof(mreal));
 			tt = (bool  *)realloc(tt,n*sizeof(bool));
 		}
 		// insert point
@@ -93,9 +93,9 @@ void mglGraph::flow(bool simple, float zVal, float u, float v,
 	free(pp);	free(cc);	free(tt);
 }
 //-----------------------------------------------------------------------------
-void mglGraph::Flow(const mglData &x, const mglData &y, const mglData &ax, const mglData &ay, const char *sch, int num, bool cnt, float zVal)
+void mglGraph::Flow(const mglData &x, const mglData &y, const mglData &ax, const mglData &ay, const char *sch, int num, bool cnt, mreal zVal)
 {
-	float u,v;
+	mreal u,v;
 	long n=ax.nx, m=ax.ny;
 	if(n*m*ax.nz!=ay.nx*ay.ny*ay.nz)	{	SetWarn(mglWarnDim,"Flow");	return;	}
 	if(n<2 || m<2)						{	SetWarn(mglWarnLow,"Flow");	return;	}
@@ -110,7 +110,7 @@ void mglGraph::Flow(const mglData &x, const mglData &y, const mglData &ax, const
 
 	for(long k=0;k<ax.nz;k++)
 	{
-		if(ax.nz>1)	zVal = Min.z+(Max.z-Min.z)*float(k)/(ax.nz-1);
+		if(ax.nz>1)	zVal = Min.z+(Max.z-Min.z)*mreal(k)/(ax.nz-1);
 		for(long i=0;i<num;i++)
 		{
 			u = 0;	v = (i+1.)/(num+1.);
@@ -140,7 +140,7 @@ void mglGraph::Flow(const mglData &x, const mglData &y, const mglData &ax, const
 }
 //-----------------------------------------------------------------------------
 void mglGraph::Flow(const mglData &ax, const mglData &ay, const char *sch, int num,
-					bool cnt, float zVal)
+					bool cnt, mreal zVal)
 {
 	if(ax.nx*ax.ny!=ay.nx*ay.ny){	SetWarn(mglWarnDim,"Flow");	return;	}
 	if(ax.nx<2 || ax.ny<2)		{	SetWarn(mglWarnLow,"Flow");	return;	}
@@ -150,21 +150,87 @@ void mglGraph::Flow(const mglData &ax, const mglData &ay, const char *sch, int n
 	Flow(x,y,ax,ay,sch,num,cnt,zVal);
 }
 //-----------------------------------------------------------------------------
-void mglGraph::flow(bool simple, float u, float v, float w,
+void mglGraph::Flow(mglPoint p, const mglData &x, const mglData &y, const mglData &ax, const mglData &ay, const char *sch)
+{
+	mreal u,v;
+	long n=ax.nx, m=ax.ny;
+	if(n*m*ax.nz!=ay.nx*ay.ny*ay.nz)	{	SetWarn(mglWarnDim,"Flow");	return;	}
+	if(n<2 || m<2)						{	SetWarn(mglWarnLow,"Flow");	return;	}
+	static int cgid=1;	StartGroup("FlowP",cgid++);
+	bool both = x.nx==n && y.nx==n && x.ny==m && y.ny==m;
+	if(!(both || (x.nx==n && y.nx==m)))	{	SetWarn(mglWarnDim,"Flow");	return;	}
+
+	Arrow1 = Arrow2 = '_';
+	SetScheme(sch);
+	// find coordinates u, v
+	register long i,j,ii;
+	register mreal d, dm=1e7;
+	long i0=0,j0=0;
+	for(i=0;i<n;i++)	for(j=0;j<m;j++)	// first find closest
+	{
+		ii = i+n*j;
+		d = both ? hypot(x.a[ii]-p.x,y.a[ii]-p.y) : hypot(x.a[i]-p.x,y.a[j]-p.y);
+		if(d<dm)	{	i0=i;	j0=j;	dm=d;	}
+	}
+	if(dm==0)	{	u = i0/mreal(n);	v = j0/mreal(m);	}	// we find it
+	else
+	{
+		mreal dxu,dxv,dyu,dyv, dx, dy;
+		if(both)
+		{
+			ii = i0+n*j0;
+			dx = x.a[ii]-p.x;	dy = y.a[ii]-p.y;
+			if(i0<n-1)
+			{	dxu = x.a[ii+1]-x.a[ii];	dyu = y.a[ii+1]-y.a[ii];	}
+			else
+			{	dxu = x.a[ii]-x.a[ii-1];	dyu = y.a[ii]-y.a[ii-1];	}
+			if(j0<m-1)
+			{	dxv = x.a[ii+n]-x.a[ii];	dyv = y.a[ii+n]-y.a[ii];	}
+			else
+			{	dxv = x.a[ii]-x.a[ii-n];	dyv = y.a[ii]-y.a[ii-n];	}
+			d = dxv*dyu-dxu*dyv;
+			u = (i0+(dxv*dy-dx*dyv)/d)/n;
+			v = (j0-(dxu*dy-dx*dyu)/d)/m;
+		}
+		else
+		{
+			dx = x.a[i0]-p.x;	dy = y.a[j0]-p.y;
+			if(i0<n-1)	dxu = x.a[i0+1]-x.a[i0];
+			else		dxu = x.a[i0]-x.a[i0-1];
+			if(j0<m-1)	dyv = y.a[j0+1]-y.a[j0];
+			else		dyv = y.a[j0]-y.a[j0-1];
+			u = (i0+dx/dxu)/n;	v = (j0+dy/dyv)/m;
+		}
+	}
+	flow(false, p.z, u, v, x, y, ax, ay);
+	EndGroup();
+}
+//-----------------------------------------------------------------------------
+void mglGraph::Flow(mglPoint p, const mglData &ax, const mglData &ay, const char *sch)
+{
+	if(ax.nx*ax.ny!=ay.nx*ay.ny){	SetWarn(mglWarnDim,"Flow");	return;	}
+	if(ax.nx<2 || ax.ny<2)		{	SetWarn(mglWarnLow,"Flow");	return;	}
+	mglData x(ax.nx), y(ax.ny);
+	x.Fill(Min.x,Max.x);
+	y.Fill(Min.y,Max.y);
+	Flow(p,x,y,ax,ay,sch);
+}
+//-----------------------------------------------------------------------------
+void mglGraph::flow(bool simple, mreal u, mreal v, mreal w,
 					const mglData &x, const mglData &y, const mglData &z,
 					const mglData &ax, const mglData &ay, const mglData &az)
 {
 	static long n=10*(ax.nx+ax.ny);
 	long nn = ax.nx*ax.ny*ax.nz;
 	bool both = x.nx*x.ny*x.nz==nn && y.nx*y.ny*y.nz==nn && z.nx*z.ny*z.nz==nn;
-	float *pp, *cc;
+	mreal *pp, *cc;
 	bool *tt;
-	pp = (float *)malloc(3*n*sizeof(float));
-	cc = (float *)malloc(n*sizeof(float));
+	pp = (mreal *)malloc(3*n*sizeof(mreal));
+	cc = (mreal *)malloc(n*sizeof(mreal));
 	tt = (bool  *)malloc(n*sizeof(bool));
 	nn = (ax.nx > ax.ny ? ax.nx : ax.ny);
 	nn = (nn > ax.nz ? nn : ax.nz);
-	float dt = 0.2/nn, e,f,g,ee[4],ff[4],gg[4],h,s=1,u1,v1,w1;
+	mreal dt = 0.2/nn, e,f,g,ee[4],ff[4],gg[4],h,s=1,u1,v1,w1;
 	if(u<0 || v<0 || w<0)
 	{	dt = -dt;	u = -u;		v = -v;		w = -w;		s = -1;}
 	register long k=0,m;
@@ -173,8 +239,8 @@ void mglGraph::flow(bool simple, float u, float v, float w,
 		if(k>=n)	// resize array if need
 		{
 			n += 10*(ax.nx+ax.ny);
-			pp = (float *)realloc(pp,3*n*sizeof(float));
-			cc = (float *)realloc(cc,n*sizeof(float));
+			pp = (mreal *)realloc(pp,3*n*sizeof(mreal));
+			cc = (mreal *)realloc(cc,n*sizeof(mreal));
 			tt = (bool  *)realloc(tt,n*sizeof(bool));
 		}
 		// insert point
@@ -233,7 +299,7 @@ void mglGraph::flow(bool simple, float u, float v, float w,
 void mglGraph::Flow(const mglData &x, const mglData &y, const mglData &z, const mglData &ax, const mglData &ay,
 					const mglData &az, const char *sch, int num, bool cnt)
 {
-	float u,v,w;
+	mreal u,v,w;
 	long i,j,n=ax.nx,m=ax.ny,l=ax.nz;
 	if(ax.nx*ax.ny*ax.nz!=ay.nx*ay.ny*ay.nz || ax.nx*ax.ny*ax.nz!=az.nx*az.ny*az.nz)
 	{	SetWarn(mglWarnDim,"Flow");	return;	}
@@ -297,20 +363,105 @@ void mglGraph::Flow(const mglData &ax, const mglData &ay, const mglData &az,
 	Flow(x,y,z,ax,ay,az,sch,num,cnt);
 }
 //-----------------------------------------------------------------------------
+void mglGraph::Flow(mglPoint p, const mglData &x, const mglData &y, const mglData &z, const mglData &ax, const mglData &ay, const mglData &az, const char *sch)
+{
+	mreal u,v,w;
+	long n=ax.nx,m=ax.ny,l=ax.nz;
+	if(ax.nx*ax.ny*ax.nz!=ay.nx*ay.ny*ay.nz || ax.nx*ax.ny*ax.nz!=az.nx*az.ny*az.nz)
+	{	SetWarn(mglWarnDim,"Flow");	return;	}
+	if(ax.nx<2 || ax.ny<2 || ax.nz<2)
+	{	SetWarn(mglWarnLow,"Flow");	return;	}
+	bool both = x.nx*x.ny*x.nz==n*m*l && y.nx*y.ny*y.nz==n*m*l && z.nx*z.ny*z.nz==n*m*l;
+	if(!(both || (x.nx==n && y.nx==m && z.nx==l)))
+	{	SetWarn(mglWarnDim,"Flow");	return;	}
+	static int cgid=1;	StartGroup("FlowP3",cgid++);
+	Arrow1 = Arrow2 = '_';	SetScheme(sch);
+
+	// find coordinates u, v, w
+	register long i,j,k,ii;
+	register mreal d, dm=1e7;
+	long i0=0,j0=0,k0=0;
+	mreal dx,dy,dz;
+	for(i=0;i<n;i++)	for(j=0;j<m;j++)	for(k=0;k<l;k++)	// first find closest
+	{
+		ii = i+n*(j+m*k);
+		if(both)
+		{	dx = x.a[ii]-p.x;	dy = y.a[ii]-p.y;	dz = x.a[ii]-p.z;	}
+		else
+		{	dx = x.a[i]-p.x;	dy = y.a[j]-p.y;	dz = x.a[k]-p.z;	}
+		d = sqrt(dx*dx+dy*dy+dz*dz);
+		if(d<dm)	{	i0=i;	j0=j;	k0=k;	dm=d;	}
+	}
+	if(dm==0)	// we find it
+	{	u=i0/mreal(n);	v=j0/mreal(m);	w=k0/mreal(l);	}
+	else
+	{
+		mreal dxu,dxv,dxw,dyu,dyv,dyw,dzu,dzv,dzw;
+		if(both)
+		{
+			ii = i0+n*j0;
+			dx = x.a[ii]-p.x;	dy = y.a[ii]-p.y;	dz = z.a[ii]-p.z;
+			if(i0<n-1)
+			{	dxu = x.a[ii+1]-x.a[ii];	dyu = y.a[ii+1]-y.a[ii];	dzu = z.a[ii+1]-z.a[ii];	}
+			else
+			{	dxu = x.a[ii]-x.a[ii-1];	dyu = y.a[ii]-y.a[ii-1];	dzu = z.a[ii]-z.a[ii-1];	}
+			if(j0<m-1)
+			{	dxv = x.a[ii+n]-x.a[ii];	dyv = y.a[ii+n]-y.a[ii];	dzv = z.a[ii+n]-z.a[ii];	}
+			else
+			{	dxv = x.a[ii]-x.a[ii-n];	dyv = y.a[ii]-y.a[ii-n];	dzv = z.a[ii]-z.a[ii-n];	}
+			if(k0<l-1)
+			{	dxw = x.a[ii+n*m]-x.a[ii];	dyw = y.a[ii+n*m]-y.a[ii];	dzw = z.a[ii+n*m]-z.a[ii];	}
+			else
+			{	dxw = x.a[ii]-x.a[ii-n*m];	dyw = y.a[ii]-y.a[ii-n*m];	dzw = z.a[ii]-z.a[ii-n*m];	}
+			d = dxu*(dyw*dzv-dyv*dzw)+dxv*(dyu*dzw-dyw*dzu)+dxw*(dyv*dzu-dyu*dzv);
+			u = (i0+(dx*(dyw*dzv-dyv*dzw)+dxv*(dy*dzw-dyw*dz)+dxw*(dyv*dz-dy*dzv))/d)/n;
+			v = (j0-(dx*(dyw*dzu-dyu*dzw)+dxu*(dy*dzw-dyw*dz)+dxw*(dyu*dz-dy*dzu))/d)/m;
+			w = (i0+(dx*(dyv*dzu-dyu*dzv)+dxu*(dy*dzv-dyv*dz)+dxv*(dyu*dz-dy*dzu))/d)/l;
+		}
+		else
+		{
+			dx = x.a[i0]-p.x;	dy = y.a[j0]-p.y;
+			if(i0<n-1)	dxu = x.a[i0+1]-x.a[i0];
+			else		dxu = x.a[i0]-x.a[i0-1];
+			if(j0<m-1)	dyv = y.a[j0+1]-y.a[j0];
+			else		dyv = y.a[j0]-y.a[j0-1];
+			if(k0<l-1)	dzw = z.a[k0+1]-z.a[k0];
+			else		dzw = z.a[k0]-z.a[k0-1];
+			u = (i0+dx/dxu)/n;	v = (j0+dy/dyv)/m;	w = (k0+dz/dzw)/m;
+		}
+	}
+	flow(false, u, v, w, x, y, z, ax, ay, az);
+	EndGroup();
+}
+//-----------------------------------------------------------------------------
+void mglGraph::Flow(mglPoint p, const mglData &ax, const mglData &ay, const mglData &az,
+					const char *sch)
+{
+	if(ax.nx*ax.ny*ax.nz!=ay.nx*ay.ny*ay.nz || ax.nx*ax.ny*ax.nz!=az.nx*az.ny*az.nz)
+	{	SetWarn(mglWarnDim,"Flow");	return;	}
+	if(ax.nx<2 || ax.ny<2 || ax.nz<2)
+	{	SetWarn(mglWarnLow,"Flow");	return;	}
+	mglData x(ax.nx), y(ax.ny), z(ax.nz);
+	x.Fill(Min.x,Max.x);
+	y.Fill(Min.y,Max.y);
+	z.Fill(Min.z,Max.z);
+	Flow(p, x,y,z,ax,ay,az,sch);
+}
+//-----------------------------------------------------------------------------
 //
 //	Pipe series
 //
 //-----------------------------------------------------------------------------
-void mglGraph::flowr(bool simple, float zVal, float u, float v, const mglData &x, const mglData &y, const mglData &ax, const mglData &ay, float r0)
+void mglGraph::flowr(bool simple, mreal zVal, mreal u, mreal v, const mglData &x, const mglData &y, const mglData &ax, const mglData &ay, mreal r0)
 {
 	long n=10*(ax.nx+ax.ny);
 	bool both = x.nx==ax.nx && y.nx==ax.nx && x.ny==ax.ny && y.ny==ax.ny;
-	float *pp, *cc, *rr;
+	mreal *pp, *cc, *rr;
 
-	pp = (float *)malloc(3*n*sizeof(float));
-	cc = (float *)malloc(4*n*sizeof(float));
-	rr = (float *)malloc(n*sizeof(float));
-	float dt = 0.5/(ax.nx > ax.ny ? ax.nx : ax.ny),e,f,g,ff[4],gg[4],h,s=1,ss;
+	pp = (mreal *)malloc(3*n*sizeof(mreal));
+	cc = (mreal *)malloc(4*n*sizeof(mreal));
+	rr = (mreal *)malloc(n*sizeof(mreal));
+	mreal dt = 0.5/(ax.nx > ax.ny ? ax.nx : ax.ny),e,f,g,ff[4],gg[4],h,s=1,ss;
 	if(u<0 || v<0)	{	dt = -dt;	u = -u;		v = -v;		s = -1;}
 	ss = 4/((Cmax-Cmin)*(Cmax-Cmin));
 	if(r0==0)	r0=0.05;
@@ -320,9 +471,9 @@ void mglGraph::flowr(bool simple, float zVal, float u, float v, const mglData &x
 		if(k>=n)	// resize array if need
 		{
 			n += 10*(ax.nx+ax.ny);
-			pp = (float *)realloc(pp,3*n*sizeof(float));
-			cc = (float *)realloc(cc,4*n*sizeof(float));
-			rr = (float *)realloc(rr,n*sizeof(float));
+			pp = (mreal *)realloc(pp,3*n*sizeof(mreal));
+			cc = (mreal *)realloc(cc,4*n*sizeof(mreal));
+			rr = (mreal *)realloc(rr,n*sizeof(mreal));
 		}
 		// insert point
 		pp[3*k] = simple ? Min.x + (Max.x-Min.x)*u : (both ? x.Spline1(u,v,0):x.Spline1(u,0,0));
@@ -340,7 +491,7 @@ void mglGraph::flowr(bool simple, float zVal, float u, float v, const mglData &x
 		h = hypot(f,g);
 		rr[k] = r0>0 ? r0*sqrt(1e-2+ss*h*h)/2 : -r0/sqrt(1e-2+ss*h*h)/5;
 		{
-			float a = (2*s*h-Cmin-Cmax)/(Cmax-Cmin);
+			mreal a = (2*s*h-Cmin-Cmax)/(Cmax-Cmin);
 			if(a>1)	a=1;	if(a<-1)	a=-1;
 			mglColor c = GetC(a,false);
 			cc[4*k] = c.r;	cc[4*k+1] = c.g;	cc[4*k+2] = c.b;	cc[4*k+3] = AlphaDef;
@@ -369,9 +520,9 @@ void mglGraph::flowr(bool simple, float zVal, float u, float v, const mglData &x
 	free(pp);	free(cc);	free(rr);
 }
 //-----------------------------------------------------------------------------
-void mglGraph::Pipe(const mglData &x, const mglData &y, const mglData &ax, const mglData &ay, const char *sch, float r0, int num, bool cnt, float zVal)
+void mglGraph::Pipe(const mglData &x, const mglData &y, const mglData &ax, const mglData &ay, const char *sch, mreal r0, int num, bool cnt, mreal zVal)
 {
-	float u,v;
+	mreal u,v;
 	long n=ax.nx, m=ax.ny;
 	if(n*m*ax.nz!=ay.nx*ay.ny*ay.nz)	{	SetWarn(mglWarnDim,"Pipe");	return;	}
 	if(n<2 || m<2)						{	SetWarn(mglWarnLow,"Pipe");	return;	}
@@ -386,7 +537,7 @@ void mglGraph::Pipe(const mglData &x, const mglData &y, const mglData &ax, const
 
 	for(long k=0;k<ax.nz;k++)
 	{
-		if(ax.nz>1)	zVal = Min.z+(Max.z-Min.z)*float(k)/(ax.nz-1);
+		if(ax.nz>1)	zVal = Min.z+(Max.z-Min.z)*mreal(k)/(ax.nz-1);
 		for(long i=0;i<num;i++)
 		{
 			u = 0;	v = (i+1.)/(num+1.);
@@ -415,7 +566,7 @@ void mglGraph::Pipe(const mglData &x, const mglData &y, const mglData &ax, const
 	EndGroup();
 }
 //-----------------------------------------------------------------------------
-void mglGraph::Pipe(const mglData &ax, const mglData &ay, const char *sch, float r0, int num, bool cnt, float zVal)
+void mglGraph::Pipe(const mglData &ax, const mglData &ay, const char *sch, mreal r0, int num, bool cnt, mreal zVal)
 {
 	if(ax.nx*ax.ny!=ay.nx*ay.ny){	SetWarn(mglWarnDim,"Pipe");	return;	}
 	if(ax.nx<2 || ax.ny<2)		{	SetWarn(mglWarnLow,"Pipe");	return;	}
@@ -425,21 +576,21 @@ void mglGraph::Pipe(const mglData &ax, const mglData &ay, const char *sch, float
 	Pipe(x,y,ax,ay,sch,r0,num,cnt,zVal);
 }
 //-----------------------------------------------------------------------------
-void mglGraph::flowr(bool simple, float u, float v, float w,
+void mglGraph::flowr(bool simple, mreal u, mreal v, mreal w,
 					const mglData &x, const mglData &y, const mglData &z,
-					const mglData &ax, const mglData &ay, const mglData &az, float r0)
+					const mglData &ax, const mglData &ay, const mglData &az, mreal r0)
 {
 	static long n=10*(ax.nx+ax.ny);
 	long nn = ax.nx*ax.ny*ax.nz;
 	bool both = x.nx*x.ny*x.nz==nn && y.nx*y.ny*y.nz==nn && z.nx*z.ny*z.nz==nn;
-	float *pp, *cc, *rr;
+	mreal *pp, *cc, *rr;
 
-	pp = (float *)malloc(3*n*sizeof(float));
-	cc = (float *)malloc(4*n*sizeof(float));
-	rr = (float *)malloc(n*sizeof(float));
+	pp = (mreal *)malloc(3*n*sizeof(mreal));
+	cc = (mreal *)malloc(4*n*sizeof(mreal));
+	rr = (mreal *)malloc(n*sizeof(mreal));
 	nn = (ax.nx > ax.ny ? ax.nx : ax.ny);
 	nn = (nn > ax.nz ? nn : ax.nz);
-	float dt = 0.2/nn, e,f,g,ee[4],ff[4],gg[4],h,s=1,u1,v1,w1,ss;
+	mreal dt = 0.2/nn, e,f,g,ee[4],ff[4],gg[4],h,s=1,u1,v1,w1,ss;
 	if(u<0 || v<0 || w<0)
 	{	dt = -dt;	u = -u;		v = -v;		w = -w;		s = -1;}
 	ss = 4/((Cmax-Cmin)*(Cmax-Cmin));
@@ -450,9 +601,9 @@ void mglGraph::flowr(bool simple, float u, float v, float w,
 		if(k>=n)	// resize array if need
 		{
 			n += 10*(ax.nx+ax.ny);
-			pp = (float *)realloc(pp,3*n*sizeof(float));
-			cc = (float *)realloc(cc,4*n*sizeof(float));
-			rr = (float *)realloc(rr,n*sizeof(float));
+			pp = (mreal *)realloc(pp,3*n*sizeof(mreal));
+			cc = (mreal *)realloc(cc,4*n*sizeof(mreal));
+			rr = (mreal *)realloc(rr,n*sizeof(mreal));
 		}
 		// insert point
 		pp[3*k] = simple ? Min.x + (Max.x-Min.x)*u : (both ? x.Spline1(u,v,w):x.Spline1(u,0,0));
@@ -470,7 +621,7 @@ void mglGraph::flowr(bool simple, float u, float v, float w,
 		h = sqrt(e*e+f*f+g*g);
 		rr[k] = r0>0 ? r0*sqrt(1e-2+ss*h*h)/2 : -r0/sqrt(1e-2+ss*h*h)/5;
 		{
-			float a = (2*s*h-Cmin-Cmax)/(Cmax-Cmin);
+			mreal a = (2*s*h-Cmin-Cmax)/(Cmax-Cmin);
 			if(a>1)	a=1;	if(a<-1)	a=-1;
 			mglColor c = GetC(a,false);
 			cc[4*k] = c.r;	cc[4*k+1] = c.g;	cc[4*k+2] = c.b;	cc[4*k+3] = AlphaDef;
@@ -506,9 +657,9 @@ void mglGraph::flowr(bool simple, float u, float v, float w,
 	free(pp);	free(cc);	free(rr);
 }
 //-----------------------------------------------------------------------------
-void mglGraph::Pipe(const mglData &x, const mglData &y, const mglData &z, const mglData &ax, const mglData &ay, const mglData &az, const char *sch, float r0, int num, bool cnt)
+void mglGraph::Pipe(const mglData &x, const mglData &y, const mglData &z, const mglData &ax, const mglData &ay, const mglData &az, const char *sch, mreal r0, int num, bool cnt)
 {
-	float u,v,w;
+	mreal u,v,w;
 	long i,j,n=ax.nx,m=ax.ny,l=ax.nz;
 	if(ax.nx*ax.ny*ax.nz!=ay.nx*ay.ny*ay.nz || ax.nx*ax.ny*ax.nz!=az.nx*az.ny*az.nz)
 	{	SetWarn(mglWarnDim,"Pipe");	return;	}
@@ -559,7 +710,7 @@ void mglGraph::Pipe(const mglData &x, const mglData &y, const mglData &z, const
 }
 //-----------------------------------------------------------------------------
 void mglGraph::Pipe(const mglData &ax, const mglData &ay, const mglData &az,
-					const char *sch, float r0, int num, bool cnt)
+					const char *sch, mreal r0, int num, bool cnt)
 {
 	if(ax.nx*ax.ny*ax.nz!=ay.nx*ay.ny*ay.nz || ax.nx*ax.ny*ax.nz!=az.nx*az.ny*az.nz)
 	{	SetWarn(mglWarnDim,"Pipe");	return;	}
@@ -573,10 +724,10 @@ void mglGraph::Pipe(const mglData &ax, const mglData &ay, const mglData &az,
 }
 //-----------------------------------------------------------------------------
 /// Plot flows for vector field {ax,ay} parametrically depended on coordinate {x,y} with color proportional to value |a|
-void mgl_flow_xy(HMGL gr, const HMDT x, const HMDT y, const HMDT ax, const HMDT ay, const char *sch, int num, int central, float zVal)
+void mgl_flow_xy(HMGL gr, const HMDT x, const HMDT y, const HMDT ax, const HMDT ay, const char *sch, int num, int central, mreal zVal)
 {	if(gr && ay && ax && x && y)	gr->Flow(*x, *y, *ax, *ay, sch, num, central, zVal);	}
 /// Plot flows for vector field {ax,ay} with color proportional to value |a|
-void mgl_flow_2d(HMGL gr, const HMDT ax, const HMDT ay, const char *sch, int num, int central, float zVal)
+void mgl_flow_2d(HMGL gr, const HMDT ax, const HMDT ay, const char *sch, int num, int central, mreal zVal)
 {	if(gr && ay && ax)	gr->Flow(*ax, *ay, sch, num, central, zVal);	}
 /// Plot flows for 3d vector field {ax,ay,ay} parametrically depended on coordinate {x,y,z} with color proportional to value |a|
 void mgl_flow_xyz(HMGL gr, const HMDT x, const HMDT y, const HMDT z, const HMDT ax, const HMDT ay, const HMDT az, const char *sch, int num, int central)
@@ -585,31 +736,44 @@ void mgl_flow_xyz(HMGL gr, const HMDT x, const HMDT y, const HMDT z, const HMDT
 void mgl_flow_3d(HMGL gr, const HMDT ax, const HMDT ay, const HMDT az, const char *sch, int num, int central)
 {	if(gr && ay && ax && az)	gr->Flow(*ax, *ay, *az, sch, num, central);	}
 //-----------------------------------------------------------------------------
+/// Plot flows for vector field {ax,ay} parametrically depended on coordinate {x,y} with color proportional to value |a|
+void mgl_flowp_xy(HMGL gr, mreal x0, mreal y0, mreal z0, const HMDT x, const HMDT y, const HMDT ax, const HMDT ay, const char *sch)
+{	if(gr && ay && ax && x && y)	gr->Flow(mglPoint(x0,y0,z0), *x, *y, *ax, *ay, sch);	}
+/// Plot flows for vector field {ax,ay} with color proportional to value |a|
+void mgl_flowp_2d(HMGL gr, mreal x0, mreal y0, mreal z0, const HMDT ax, const HMDT ay, const char *sch)
+{	if(gr && ay && ax)	gr->Flow(mglPoint(x0,y0,z0), *ax, *ay, sch);	}
+/// Plot flows for 3d vector field {ax,ay,ay} parametrically depended on coordinate {x,y,z} with color proportional to value |a|
+void mgl_flowp_xyz(HMGL gr, mreal x0, mreal y0, mreal z0, const HMDT x, const HMDT y, const HMDT z, const HMDT ax, const HMDT ay, const HMDT az, const char *sch)
+{	if(gr && ay && ax && az && x && y && z)	gr->Flow(mglPoint(x0,y0,z0), *x, *y, *z, *ax, *ay, *az, sch);	}
+/// Plot flows for 3d vector field {ax,ay,ay} with color proportional to value |a|
+void mgl_flowp_3d(HMGL gr, mreal x0, mreal y0, mreal z0, const HMDT ax, const HMDT ay, const HMDT az, const char *sch)
+{	if(gr && ay && ax && az)	gr->Flow(mglPoint(x0,y0,z0), *ax, *ay, *az, sch);	}
+//-----------------------------------------------------------------------------
 /// Plot flow pipes for vector field {ax,ay} parametrically depended on coordinate {x,y} with color proportional to value |a|
-void mgl_pipe_xy(HMGL gr, const HMDT x, const HMDT y, const HMDT ax, const HMDT ay, const char *sch, float r0, int num, int central, float zVal)
+void mgl_pipe_xy(HMGL gr, const HMDT x, const HMDT y, const HMDT ax, const HMDT ay, const char *sch, mreal r0, int num, int central, mreal zVal)
 {	if(gr && ay && ax && x && y)	gr->Pipe(*x, *y, *ax, *ay, sch, r0, num, central, zVal);	}
 /// Plot flow pipes for vector field {ax,ay} with color proportional to value |a|
-void mgl_pipe_2d(HMGL gr, const HMDT ax, const HMDT ay, const char *sch, float r0, int num, int central, float zVal)
+void mgl_pipe_2d(HMGL gr, const HMDT ax, const HMDT ay, const char *sch, mreal r0, int num, int central, mreal zVal)
 {	if(gr && ay && ax)	gr->Pipe(*ax, *ay, sch, r0, num, central, zVal);	}
 /// Plot flow pipes for 3d vector field {ax,ay,ay} parametrically depended on coordinate {x,y,z} with color proportional to value |a|
-void mgl_pipe_xyz(HMGL gr, const HMDT x, const HMDT y, const HMDT z, const HMDT ax, const HMDT ay, const HMDT az, const char *sch, float r0, int num, int central)
+void mgl_pipe_xyz(HMGL gr, const HMDT x, const HMDT y, const HMDT z, const HMDT ax, const HMDT ay, const HMDT az, const char *sch, mreal r0, int num, int central)
 {	if(gr && ay && ax && az && x && y && z)	gr->Pipe(*x, *y, *z, *ax, *ay, *az, sch, r0, num, central);	}
 /// Plot flow pipes for 3d vector field {ax,ay,ay} with color proportional to value |a|
-void mgl_pipe_3d(HMGL gr, const HMDT ax, const HMDT ay, const HMDT az, const char *sch, float r0, int num, int central)
+void mgl_pipe_3d(HMGL gr, const HMDT ax, const HMDT ay, const HMDT az, const char *sch, mreal r0, int num, int central)
 {	if(gr && ay && ax && az)	gr->Pipe(*ax, *ay, *az, sch, r0, num, central);	}
 //-----------------------------------------------------------------------------
 //	Fortran interface
 //-----------------------------------------------------------------------------
 /// Plot flows for vector field {ax,ay} parametrically depended on coordinate {x,y} with color proportional to value |a|
 void mgl_flow_xy_(uintptr_t *gr, uintptr_t *x, uintptr_t *y, uintptr_t *ax, uintptr_t *ay, const char *sch,
-					int *num, int *central, float *zVal,int l)
+					int *num, int *central, mreal *zVal,int l)
 {
 	char *s=new char[l+1];	memcpy(s,sch,l);	s[l]=0;
 	if(gr && ay && ax && x && y)	_GR_->Flow(_D_(x), _D_(y), _D_(ax), _D_(ay), s, *num, *central, *zVal);
 	delete []s;
 }
 /// Plot flows for vector field {ax,ay} with color proportional to value |a|
-void mgl_flow_2d_(uintptr_t *gr, uintptr_t *ax, uintptr_t *ay, const char *sch, int *num, int *central, float *zVal,int l)
+void mgl_flow_2d_(uintptr_t *gr, uintptr_t *ax, uintptr_t *ay, const char *sch, int *num, int *central, mreal *zVal,int l)
 {
 	char *s=new char[l+1];	memcpy(s,sch,l);	s[l]=0;
 	if(gr && ay && ax)	_GR_->Flow(_D_(ax), _D_(ay), s, *num, *central, *zVal);
@@ -632,9 +796,42 @@ void mgl_flow_3d_(uintptr_t *gr, uintptr_t *ax, uintptr_t *ay, uintptr_t *az, co
 	delete []s;
 }
 //-----------------------------------------------------------------------------
+/// Plot flows for vector field {ax,ay} parametrically depended on coordinate {x,y} with color proportional to value |a|
+void mgl_flowp_xy_(uintptr_t *gr, mreal *x0, mreal *y0, mreal *z0, uintptr_t *x, uintptr_t *y, uintptr_t *ax, uintptr_t *ay, const char *sch, int l)
+{
+	char *s=new char[l+1];	memcpy(s,sch,l);	s[l]=0;
+	if(gr && ay && ax && x && y)
+		_GR_->Flow(mglPoint(*x0,*y0,*z0), _D_(x), _D_(y), _D_(ax), _D_(ay), s);
+	delete []s;
+}
+/// Plot flows for vector field {ax,ay} with color proportional to value |a|
+void mgl_flowp_2d_(uintptr_t *gr, mreal *x0, mreal *y0, mreal *z0, uintptr_t *ax, uintptr_t *ay, const char *sch, int l)
+{
+	char *s=new char[l+1];	memcpy(s,sch,l);	s[l]=0;
+	if(gr && ay && ax)
+		_GR_->Flow(mglPoint(*x0,*y0,*z0), _D_(ax), _D_(ay), s);
+	delete []s;
+}
+/// Plot flows for 3d vector field {ax,ay,ay} parametrically depended on coordinate {x,y,z} with color proportional to value |a|
+void mgl_flowp_xyz_(uintptr_t *gr, mreal *x0, mreal *y0, mreal *z0, uintptr_t *x, uintptr_t *y, uintptr_t *z, uintptr_t *ax, uintptr_t *ay, uintptr_t *az, const char *sch, int l)
+{
+	char *s=new char[l+1];	memcpy(s,sch,l);	s[l]=0;
+	if(gr && ay && ax && az && x && y && z)
+		_GR_->Flow(mglPoint(*x0,*y0,*z0), _D_(x), _D_(y), _D_(z), _D_(ax), _D_(ay), _D_(az), s);
+	delete []s;
+}
+/// Plot flows for 3d vector field {ax,ay,ay} with color proportional to value |a|
+void mgl_flowp_3d_(uintptr_t *gr, mreal *x0, mreal *y0, mreal *z0, uintptr_t *ax, uintptr_t *ay, uintptr_t *az, const char *sch, int l)
+{
+	char *s=new char[l+1];	memcpy(s,sch,l);	s[l]=0;
+	if(gr && ay && ax && az)
+		_GR_->Flow(mglPoint(*x0,*y0,*z0), _D_(ax), _D_(ay), _D_(az), s);
+	delete []s;
+}
+//-----------------------------------------------------------------------------
 /// Plot flow pipes for vector field {ax,ay} parametrically depended on coordinate {x,y} with color proportional to value |a|
-void mgl_pipe_xy_(uintptr_t *gr, uintptr_t *x, uintptr_t *y, uintptr_t *ax, uintptr_t *ay, const char *sch, float *r0,
-					int *num, int *central, float *zVal,int l)
+void mgl_pipe_xy_(uintptr_t *gr, uintptr_t *x, uintptr_t *y, uintptr_t *ax, uintptr_t *ay, const char *sch, mreal *r0,
+					int *num, int *central, mreal *zVal,int l)
 {
 	char *s=new char[l+1];	memcpy(s,sch,l);	s[l]=0;
 	if(gr && ay && ax && x && y)
@@ -642,7 +839,7 @@ void mgl_pipe_xy_(uintptr_t *gr, uintptr_t *x, uintptr_t *y, uintptr_t *ax, uint
 	delete []s;
 }
 /// Plot flow pipes for vector field {ax,ay} with color proportional to value |a|
-void mgl_pipe_2d_(uintptr_t *gr, uintptr_t *ax, uintptr_t *ay, const char *sch, float *r0, int *num, int *central, float *zVal,int l)
+void mgl_pipe_2d_(uintptr_t *gr, uintptr_t *ax, uintptr_t *ay, const char *sch, mreal *r0, int *num, int *central, mreal *zVal,int l)
 {
 	char *s=new char[l+1];	memcpy(s,sch,l);	s[l]=0;
 	if(gr && ay && ax)	_GR_->Pipe(_D_(ax), _D_(ay), s, *r0, *num, *central, *zVal);
@@ -650,7 +847,7 @@ void mgl_pipe_2d_(uintptr_t *gr, uintptr_t *ax, uintptr_t *ay, const char *sch,
 }
 /// Plot flow pipes for 3d vector field {ax,ay,ay} parametrically depended on coordinate {x,y,z} with color proportional to value |a|
 void mgl_pipe_xyz_(uintptr_t *gr, uintptr_t *x, uintptr_t *y, uintptr_t *z, uintptr_t *ax, uintptr_t *ay, uintptr_t *az,
-					const char *sch, float *r0, int *num, int *central,int l)
+					const char *sch, mreal *r0, int *num, int *central,int l)
 {
 	char *s=new char[l+1];	memcpy(s,sch,l);	s[l]=0;
 	if(gr && ay && ax && az && x && y && z)
@@ -658,7 +855,7 @@ void mgl_pipe_xyz_(uintptr_t *gr, uintptr_t *x, uintptr_t *y, uintptr_t *z, uint
 	delete []s;
 }
 /// Plot flow pipes for 3d vector field {ax,ay,ay} with color proportional to value |a|
-void mgl_pipe_3d_(uintptr_t *gr, uintptr_t *ax, uintptr_t *ay, uintptr_t *az, const char *sch, float *r0, int *num, int *central,int l)
+void mgl_pipe_3d_(uintptr_t *gr, uintptr_t *ax, uintptr_t *ay, uintptr_t *az, const char *sch, mreal *r0, int *num, int *central,int l)
 {
 	char *s=new char[l+1];	memcpy(s,sch,l);	s[l]=0;
 	if(gr && ay && ax && az)	_GR_->Pipe(_D_(ax), _D_(ay), _D_(az), s, *r0, *num, *central);
