@@ -36,20 +36,21 @@
 #endif
 //-----------------------------------------------------------------------------
 extern unsigned mgl_numg, mgl_cur;
-extern float mgl_fact;
+extern mreal mgl_fact;
 extern long mgl_gen_fnt[516][6];
 extern short mgl_buf_fnt[246080];
-const float mgl_fgen = 4*14;
+const mreal mgl_fgen = 4*14;
 extern mglTeXsymb mgl_tex_symb[];
 mglFont mglDefFont;
 //-----------------------------------------------------------------------------
-float mglFont::Puts(const char *str,const char *how, char col)
+mreal mglFont::Puts(const char *str,const char *how, char col)
 {
-	int font=0, align=0;
+	int font=0, align=1;
 	if(how)
 	{
 		if(strchr(how,'R'))	align = 2;
 		if(strchr(how,'C'))	align = 1;
+		if(strchr(how,'L'))	align = 0;
 		if(strchr(how,'D'))	align+= 4;
 		if(strchr(how,'b'))	font = font|MGL_FONT_BOLD;
 		if(strchr(how,'i'))	font = font|MGL_FONT_ITAL;
@@ -60,7 +61,7 @@ float mglFont::Puts(const char *str,const char *how, char col)
 	return Puts(str, font, align, col);
 }
 //-----------------------------------------------------------------------------
-float mglFont::Width(const char *str,const char *how)
+mreal mglFont::Width(const char *str,const char *how)
 {
 	int font=0;
 	if(how)
@@ -74,13 +75,14 @@ float mglFont::Width(const char *str,const char *how)
 	return Width(str, font);
 }
 //-----------------------------------------------------------------------------
-float mglFont::Puts(const wchar_t *str,const char *how, char col)
+mreal mglFont::Puts(const wchar_t *str,const char *how, char col)
 {
-	int font=0, align=0;
+	int font=0, align=1;
 	if(how)
 	{
 		if(strchr(how,'R'))	align = 2;
 		if(strchr(how,'C'))	align = 1;
+		if(strchr(how,'L'))	align = 0;
 		if(strchr(how,'D'))	align+= 4;
 		if(strchr(how,'b'))	font = font|MGL_FONT_BOLD;
 		if(strchr(how,'i'))	font = font|MGL_FONT_ITAL;
@@ -91,7 +93,7 @@ float mglFont::Puts(const wchar_t *str,const char *how, char col)
 	return Puts(str, font, align,col);
 }
 //-----------------------------------------------------------------------------
-float mglFont::Width(const wchar_t *str,const char *how)
+mreal mglFont::Width(const wchar_t *str,const char *how)
 {
 	int font=0;
 	if(how)
@@ -105,30 +107,30 @@ float mglFont::Width(const wchar_t *str,const char *how)
 	return Width(str, font);
 }
 //-----------------------------------------------------------------------------
-float mglFont::Puts(const char *str,int font,int align, char col)
+mreal mglFont::Puts(const char *str,int font,int align, char col)
 {
 	unsigned size = strlen(str)+1;
 	wchar_t *wcs = new wchar_t[size];
 	mbstowcs(wcs,str,size);
-	float w = Puts(wcs,font,align,col);
+	mreal w = Puts(wcs,font,align,col);
 	delete []wcs;
 	return w;
 }
 //-----------------------------------------------------------------------------
-float mglFont::Width(const char *str,int font)
+mreal mglFont::Width(const char *str,int font)
 {
 	unsigned size = strlen(str)+1;
 	wchar_t *wcs = new wchar_t[size];
 	mbstowcs(wcs,str,size);
-	float w = Width(wcs,font);
+	mreal w = Width(wcs,font);
 	delete []wcs;
 	return w;
 }
 //-----------------------------------------------------------------------------
-float mglFont::Puts(const wchar_t *str,int font,int align, char col)
+mreal mglFont::Puts(const wchar_t *str,int font,int align, char col)
 {
 	if(numg==0)	return 0;
-	float ww=0,w=0,h = (align&4) ? 500./fact[0] : 0;
+	mreal ww=0,w=0,h = (align&4) ? 500./fact[0] : 0;
 	unsigned size = wcslen(str)+1;
 	if(parse)
 	{
@@ -157,10 +159,7 @@ float mglFont::Puts(const wchar_t *str,int font,int align, char col)
 			{
 				j = Internal(str[i]);
 				if(j==-1)	continue;
-				if(font & MGL_FONT_WIRE)
-					gr->Glyph(w, -h, 1.f/fact[s], 0, 0, numl[s][j], buf+ln[s][j], col);
-				else
-					gr->Glyph(w, -h, 1.f/fact[s], numt[s][j], buf+tr[s][j], numl[s][j], buf+ln[s][j], col);
+				gr->Glyph(w, -h, 1, s+(font&MGL_FONT_WIRE)?4:0, j, col);
 			}
 			else	j = 0;//Internal('!');
 			w+= width[s][j]/fact[s];
@@ -169,10 +168,10 @@ float mglFont::Puts(const wchar_t *str,int font,int align, char col)
 	return ww;
 }
 //-----------------------------------------------------------------------------
-float mglFont::Width(const wchar_t *str,int font)
+mreal mglFont::Width(const wchar_t *str,int font)
 {
 	if(numg==0)	return 0;
-	float w=0;
+	mreal w=0;
 	unsigned size = wcslen(str)+1;
 	if(parse)
 	{
@@ -196,14 +195,14 @@ float mglFont::Width(const wchar_t *str,int font)
 	return w;
 }
 //-----------------------------------------------------------------------------
-float mglFont::Height(int font)
+mreal mglFont::Height(int font)
 {
 	if(numg==0)	return 0;
 	int s = (font/MGL_FONT_BOLD)&3;
 	return (500.f)/fact[s];
 }
 //-----------------------------------------------------------------------------
-float mglFont::Height(const char *how)
+mreal mglFont::Height(const char *how)
 {
 	if(numg==0)	return 0;
 	int s=0;
@@ -328,7 +327,7 @@ void mglFont::Convert(const wchar_t *str, unsigned *res)
 	res[j] = 0;
 }
 //-----------------------------------------------------------------------------
-float mglFont::get_ptr(long &i,unsigned *str, unsigned **b1, unsigned **b2,float &w1,float &w2, float f1, float f2, int st)
+mreal mglFont::get_ptr(long &i,unsigned *str, unsigned **b1, unsigned **b2,mreal &w1,mreal &w2, mreal f1, mreal f2, int st)
 {
 	static unsigned s1[2]={0,0}, s2[2]={0,0};
 	register long k;
@@ -361,16 +360,24 @@ float mglFont::get_ptr(long &i,unsigned *str, unsigned **b1, unsigned **b2,float
 	return w1>w2 ? w1 : w2;
 }
 //-----------------------------------------------------------------------------
-float mglFont::Puts(const unsigned *text, float x,float y,float f,int style,char col)
+void mglFont::draw_ouline(mglGraph *gr, int st, mreal x, mreal y, mreal f, mreal g, mreal ww, char ccol)
+{
+	if(st&MGL_FONT_OLINE)
+		gr->Glyph(x,y+499*f/g, ww*g, (st&MGL_FONT_WIRE)?12:8, 0, ccol);
+	if(st&MGL_FONT_ULINE)
+		gr->Glyph(x,y-200*f/g, ww*g, (st&MGL_FONT_WIRE)?12:8, 0, ccol);
+}
+//-----------------------------------------------------------------------------
+mreal mglFont::Puts(const unsigned *text, mreal x,mreal y,mreal f,int style,char col)
 {
 	if(numg==0)	return 0;
 	register long i,j,k;
 	register unsigned s,ss;
-	float w=0;				// string width
+	mreal w=0;				// string width
 	int st = style;			// current style
 	unsigned *b1, *b2;		// pointer to substring
 	unsigned *str;			// string itself
-	float yy=y, ff=f, ww, w1, w2;
+	mreal yy=y, ff=f, ww, w1, w2;
 	char ccol=col;
 	int a = (st/MGL_FONT_BOLD)&3;
 	for(i=0;text[i];i++);
@@ -391,10 +398,7 @@ float mglFont::Puts(const unsigned *text, float x,float y,float f,int style,char
 			str[i-1]=0;	i--;
 			ww = Puts(b1, x, yy, ff, (st&(~MGL_FONT_OLINE)&(~MGL_FONT_ULINE)), ccol);
 			if(gr && !(style&0x10))	// add under-/over- line now
-			{
-				if(st&MGL_FONT_OLINE)	gr->Glyph(x,y+499*f/fact[a], ww, !(st&MGL_FONT_WIRE), 0,-int(100*ff+1),0, ccol);
- 				if(st&MGL_FONT_ULINE)	gr->Glyph(x,y-200*f/fact[a],ww, !(st&MGL_FONT_WIRE),0,-int(100*ff+1),0, ccol);
-			}
+				draw_ouline(gr,st,x,y,f,fact[a],ww,ccol);
 			st = style;	yy = y;	ff = f;		// clear style modification
 		}
 		else if(s==unsigned(-9))	// underset
@@ -403,10 +407,7 @@ float mglFont::Puts(const unsigned *text, float x,float y,float f,int style,char
 			Puts(b1, x+(ww-w1)/2, yy, ff, (st&(~MGL_FONT_OLINE)&(~MGL_FONT_ULINE)), ccol);
 			Puts(b2, x+(ww-w2)/2, yy-150*ff/fact[a], ff/4, (st&(~MGL_FONT_OLINE)&(~MGL_FONT_ULINE)), ccol);
 			if(gr && !(style&0x10))	// add under-/over- line now
-			{
-				if(st&MGL_FONT_OLINE)	gr->Glyph(x,y+499*f/fact[a],ww, !(st&MGL_FONT_WIRE),0,-int(100*ff+1),0, ccol);
-				if(st&MGL_FONT_ULINE)	gr->Glyph(x,y-200*f/fact[a],ww, !(st&MGL_FONT_WIRE),0,-int(100*ff+1),0, ccol);
-			}
+				draw_ouline(gr,st,x,y,f,fact[a],ww,ccol);
 			st = style;	yy = y;	ff = f;		// clear style modification
 		}
 		else if(s==unsigned(-8))	// overset
@@ -415,10 +416,7 @@ float mglFont::Puts(const unsigned *text, float x,float y,float f,int style,char
 			Puts(b1, x+(ww-w1)/2, yy, ff, (st&(~MGL_FONT_OLINE)&(~MGL_FONT_ULINE)), ccol);
 			Puts(b2, x+(ww-w2)/2, yy+375*ff/fact[a], ff/4, (st&(~MGL_FONT_OLINE)&(~MGL_FONT_ULINE)), ccol);
 			if(gr && !(style&0x10))	// add under-/over- line now
-			{
-				if(st&MGL_FONT_OLINE)	gr->Glyph(x,y+499*f/fact[a],ww, !(st&MGL_FONT_WIRE),0,-int(100*ff+1),0, ccol);
-				if(st&MGL_FONT_ULINE)	gr->Glyph(x,y-200*f/fact[a],ww, !(st&MGL_FONT_WIRE),0,-int(100*ff+1),0, ccol);
-			}
+				draw_ouline(gr,st,x,y,f,fact[a],ww,ccol);
 			st = style;	yy = y;	ff = f;		// clear style modification
 		}
 		else if(s==unsigned(-11))	// stackl
@@ -427,10 +425,7 @@ float mglFont::Puts(const unsigned *text, float x,float y,float f,int style,char
 			Puts(b1, x, yy+250*ff/fact[a], ff*0.45, (st&(~MGL_FONT_OLINE)&(~MGL_FONT_ULINE)), ccol);
 			Puts(b2, x, yy-110*ff/fact[a], ff*0.45, (st&(~MGL_FONT_OLINE)&(~MGL_FONT_ULINE)), ccol);
 			if(gr && !(style&0x10))	// add under-/over- line now
-			{
-				if(st&MGL_FONT_OLINE)	gr->Glyph(x,y+499*f/fact[a],ww, !(st&MGL_FONT_WIRE),0,-int(100*ff+1),0, ccol);
-				if(st&MGL_FONT_ULINE)	gr->Glyph(x,y-200*f/fact[a],ww, !(st&MGL_FONT_WIRE),0,-int(100*ff+1),0, ccol);
-			}
+				draw_ouline(gr,st,x,y,f,fact[a],ww,ccol);
 			st = style;	yy = y;	ff = f;		// clear style modification
 		}
 		else if(s==unsigned(-10))	// stacr
@@ -439,10 +434,7 @@ float mglFont::Puts(const unsigned *text, float x,float y,float f,int style,char
 			Puts(b1, x+(ww-w1), yy+250*ff/fact[a], ff*0.45, (st&(~MGL_FONT_OLINE)&(~MGL_FONT_ULINE)), ccol);
 			Puts(b2, x+(ww-w2), yy-110*ff/fact[a], ff*0.45, (st&(~MGL_FONT_OLINE)&(~MGL_FONT_ULINE)), ccol);
 			if(gr && !(style&0x10))	// add under-/over- line now
-			{
-				if(st&MGL_FONT_OLINE)	gr->Glyph(x,y+499*f/fact[a],ww, !(st&MGL_FONT_WIRE),0,-int(100*ff+1),0, ccol);
-				if(st&MGL_FONT_ULINE)	gr->Glyph(x,y-200*f/fact[a],ww, !(st&MGL_FONT_WIRE),0,-int(100*ff+1),0, ccol);
-			}
+				draw_ouline(gr,st,x,y,f,fact[a],ww,ccol);
 			st = style;	yy = y;	ff = f;		// clear style modification
 		}
 		else if(s==unsigned(-7))	// stack
@@ -451,10 +443,7 @@ float mglFont::Puts(const unsigned *text, float x,float y,float f,int style,char
 			Puts(b1, x+(ww-w1)/2, yy+250*ff/fact[a], ff*0.45, (st&(~MGL_FONT_OLINE)&(~MGL_FONT_ULINE)), ccol);
 			Puts(b2, x+(ww-w2)/2, yy-110*ff/fact[a], ff*0.45, (st&(~MGL_FONT_OLINE)&(~MGL_FONT_ULINE)), ccol);
 			if(gr && !(style&0x10))	// add under-/over- line now
-			{
-				if(st&MGL_FONT_OLINE)	gr->Glyph(x,y+499*f/fact[a],ww, !(st&MGL_FONT_WIRE),0,-int(100*ff+1),0, ccol);
-				if(st&MGL_FONT_ULINE)	gr->Glyph(x,y-200*f/fact[a],ww, !(st&MGL_FONT_WIRE),0,-int(100*ff+1),0, ccol);
-			}
+				draw_ouline(gr,st,x,y,f,fact[a],ww,ccol);
 			st = style;	yy = y;	ff = f;		// clear style modification
 		}
 		else if(s==unsigned(-6))	// frac
@@ -464,9 +453,8 @@ float mglFont::Puts(const unsigned *text, float x,float y,float f,int style,char
 			Puts(b2, x+(ww-w2)/2, yy-110*ff/fact[a], ff*0.45, (st&(~MGL_FONT_OLINE)&(~MGL_FONT_ULINE)), ccol);
 			if(gr && !(style&0x10))	// add under-/over- line now
 			{
-				gr->Glyph(x,y+150*f/fact[a],ww, !(st&MGL_FONT_WIRE),0,-int(100*ff+1),0, ccol);
-				if(st&MGL_FONT_OLINE)	gr->Glyph(x,y+499*f/fact[a],ww, !(st&MGL_FONT_WIRE),0,-int(100*ff+1),0, ccol);
-				if(st&MGL_FONT_ULINE)	gr->Glyph(x,y-200*f/fact[a],ww, !(st&MGL_FONT_WIRE),0,-int(100*ff+1),0, ccol);
+				draw_ouline(gr,st,x,y,f,fact[a],ww,ccol);
+				gr->Glyph(x,y+150*f/fact[a], ww*fact[a], (st&MGL_FONT_WIRE)?12:8, 0, ccol);
 			}
 			st = style;	yy = y;	ff = f;		// clear style modification
 		}
@@ -490,21 +478,16 @@ float mglFont::Puts(const unsigned *text, float x,float y,float f,int style,char
 					if(gr && !(style&0x10))
 					{
 						if(st & MGL_FONT_WIRE)
-							gr->Glyph(x, yy, ff/fact[a], 0, 0, numl[a][j], buf+ln[a][j], ccol);
+							gr->Glyph(x,yy,ff,a+4,j,ccol);
 						else
-							gr->Glyph(x, yy, ff/fact[a], numt[a][j], buf+tr[a][j], numl[a][j], buf+ln[a][j], ccol);
+							gr->Glyph(x,yy,ff,a,j,ccol);
 					}
 				}
 				else
 					j = Internal('!');
 				ww = ff*width[a][j]/fact[a];
 				if(gr && !(style&0x10))	// add under-/over- line now
-				{
-					if(st&MGL_FONT_OLINE)
-						gr->Glyph(x,y+499*f/fact[a],ww, !(st&MGL_FONT_WIRE),0,-int(100*ff+1),0, ccol);
-					if(st&MGL_FONT_ULINE)
-						gr->Glyph(x,y-200*f/fact[a],ww, !(st&MGL_FONT_WIRE),0,-int(100*ff+1),0, ccol);
-				}
+					draw_ouline(gr,st,x,y,f,fact[a],ww,ccol);
 				if(s & MGL_FONT_ZEROW)	ww = 0;
 				st = style;	yy = y;	ff = f;	ccol=col;	// clear style modification
 			}
@@ -526,7 +509,7 @@ float mglFont::Puts(const unsigned *text, float x,float y,float f,int style,char
 	return w;
 }
 //-----------------------------------------------------------------------------
-void mglFont::read_data(FILE *fp, float *ff, short *wdt, short *numl,
+void mglFont::read_data(FILE *fp, mreal *ff, short *wdt, short *numl,
 						unsigned *posl, short *numt, unsigned *post, unsigned &cur)
 {
 	char str[256];
@@ -654,6 +637,7 @@ bool mglFont::read_def(unsigned &cur)
 bool mglFont::Load(const char *base, const char *path)
 {
 //	base = 0;
+	char *buf=0,sep='/';
 	FILE *fp;
 	char str[256];
 	setlocale(LC_NUMERIC,"C");
@@ -661,7 +645,6 @@ bool mglFont::Load(const char *base, const char *path)
 	if(!path)	path = MGL_FONT_PATH;
 	if(base)
 	{
-		char *buf,sep='/';
 #ifdef WIN32
 		sep='\\';
 #endif
@@ -676,7 +659,8 @@ bool mglFont::Load(const char *base, const char *path)
 	}
 	Clear();							// first clear old
 
-	if(!base || !read_main(base,path,cur))	{	read_def(cur);	return true;	}
+	if(!base || !read_main(base,path,cur))
+	{	read_def(cur);	if(buf)	delete []buf;	return true;	}
 
 	//================== bold ===========================================
 	sprintf(str,"%s/%s_b.vfm",path,base);
@@ -707,12 +691,13 @@ bool mglFont::Load(const char *base, const char *path)
 	// Finally normalize all factors
 	fact[0] *= mgl_fgen;	fact[1] *= mgl_fgen;
 	fact[2] *= mgl_fgen;	fact[3] *= mgl_fgen;
+	if(buf)	delete []buf;
 	return true;
 }
 //-----------------------------------------------------------------------------
 mglFont::mglFont(const char *name, const char *path)
 {
-	parse = true;	numg=0;
+	parse = true;	numg=0;	gr=0;
 	if(this==&mglDefFont)	Load(name, path);
 	else	Copy(&mglDefFont);
 }
@@ -721,6 +706,7 @@ void mglFont::Restore()	{	Copy(&mglDefFont);	}
 //-----------------------------------------------------------------------------
 void mglFont::Clear()
 {
+//	if(gr)	gr->Clf();
 	if(numg)
 	{
 		delete []id;		free(buf);			numg = 0;
@@ -760,7 +746,7 @@ void mglFont::Copy(mglFont *f)
 	memcpy(numl[1],f->numl[1],numg*sizeof(short));
 	memcpy(numl[2],f->numl[2],numg*sizeof(short));
 	memcpy(numl[3],f->numl[3],numg*sizeof(short));
-	memcpy(fact,f->fact,4*sizeof(float));
+	memcpy(fact,f->fact,4*sizeof(mreal));
 	// now copy symbols descriptions
 	buf = (short *)malloc(numb*sizeof(short));
 	memcpy(buf, f->buf, numb*sizeof(short));
