@@ -21,8 +21,13 @@
 #include <unistd.h>
 #include <stdarg.h>
 #include <wchar.h>
-#include "mgl/mgl_eval.h"
 #include "mgl/mgl.h"
+#include "mgl/mgl_eval.h"
+#ifndef MGL_DEF_VIEWER
+//#define MGL_DEF_VIEWER "kuickshow"
+#define MGL_DEF_VIEWER "gthumb"
+//#define MGL_DEF_VIEWER "gwenview";
+#endif
 //-----------------------------------------------------------------------------
 const char *mglWarn[mglWarnEnd] = {"%s: data dimension(s) is incompatible",
 								"%s: data dimension(s) is too small",
@@ -87,23 +92,23 @@ void mglGraph::RecalcBorder()
 	}
 }
 //-----------------------------------------------------------------------------
-void mglGraph::SetFBord(float x,float y,float z)
+void mglGraph::SetFBord(mreal x,mreal y,mreal z)
 {
 	if(fx)
 	{
-		float v = fx->Calc(x,y,z);
+		mreal v = fx->Calc(x,y,z);
 		if(FMax.x < v)	FMax.x = v;
 		if(FMin.x > v)	FMin.x = v;
 	}
 	if(fy)
 	{
-		float v = fy->Calc(x,y,z);
+		mreal v = fy->Calc(x,y,z);
 		if(FMax.y < v)	FMax.y = v;
 		if(FMin.y > v)	FMin.y = v;
 	}
 	if(fz)
 	{
-		float v = fz->Calc(x,y,z);
+		mreal v = fz->Calc(x,y,z);
 		if(FMax.z < v)	FMax.z = v;
 		if(FMin.z > v)	FMin.z = v;
 	}
@@ -151,11 +156,11 @@ void mglGraph::CutOff(const char *EqC)
 //-----------------------------------------------------------------------------
 //#define FLT_EPS	1.1920928955078125e-07
 #define FLT_EPS	(1.+2e-07)
-bool mglGraph::ScalePoint(float &x,float &y,float &z)
+bool mglGraph::ScalePoint(mreal &x,mreal &y,mreal &z)
 {
-//	float x1=x,y1=y,z1=z;
+//	mreal x1=x,y1=y,z1=z;
 	if(isnan(x) || isnan(y) || isnan(z))	return false;
-	float x1,y1,z1,x2,y2,z2;
+	mreal x1,y1,z1,x2,y2,z2;
 	x1 = x>0?x*FLT_EPS:x/FLT_EPS;	x2 = x<0?x*FLT_EPS:x/FLT_EPS;
 	y1 = y>0?y*FLT_EPS:y/FLT_EPS;	y2 = y<0?y*FLT_EPS:y/FLT_EPS;
 	z1 = z>0?z*FLT_EPS:z/FLT_EPS;	z2 = z<0?z*FLT_EPS:z/FLT_EPS;
@@ -201,11 +206,11 @@ bool mglGraph::ScalePoint(float &x,float &y,float &z)
 	return res;
 }
 //-----------------------------------------------------------------------------
-void mglGraph::Color(float a,float a1,float a2)
+void mglGraph::Color(mreal a,mreal a1,mreal a2)
 {
 	mglColor c;
 	if(a1==a2)	{	a1 = Cmin;	a2 = Cmax;	}
-	float z = (2*a-a1-a2)/(a2-a1);
+	mreal z = (2*a-a1-a2)/(a2-a1);
 	if(z>1)	z=1;	if(z<-1)	z=-1;
 	c = GetC(z,false);
 	DefColor(c,AlphaDef);
@@ -249,10 +254,10 @@ void mglGraph::SetScheme(const char *s)
 	if(NumCol==0)	NumCol = 2;
 }
 //-----------------------------------------------------------------------------
-mglColor mglGraph::GetC(float x,float y,float z,bool simple)
+mglColor mglGraph::GetC(mreal x,mreal y,mreal z,bool simple)
 {
 	mglColor c,m;
-	float n;
+	mreal n;
 	if(OnCoord && !simple && NumCol>2)
 	{
 		m = cmap[0]+cmap[1]+cmap[2];		n = m.Norm();
@@ -262,14 +267,14 @@ mglColor mglGraph::GetC(float x,float y,float z,bool simple)
 	return c;
 }
 //-----------------------------------------------------------------------------
-float mglGraph::GetA(float a)
+mreal mglGraph::GetA(mreal a)
 {
 	a = (2*a-Cmin-Cmax)/(Cmax-Cmin);
 	a = fabs(a)>1 ? copysignf(1,a) : a;
 	return a;
 }
 //-----------------------------------------------------------------------------
-mglColor mglGraph::GetC(float z, bool scale)
+mglColor mglGraph::GetC(mreal z, bool scale)
 {
 	mglColor c;
 	register long n = NumCol-1;
@@ -286,10 +291,10 @@ mglColor mglGraph::GetC(float z, bool scale)
 	return c;
 }
 //-----------------------------------------------------------------------------
-mglColor mglGraph::GetC2(float x,float y)
+mglColor mglGraph::GetC2(mreal x,mreal y)
 {
 	mglColor c;
-	float m=1;
+	mreal m=1;
 	x = (2*x-Min.x)/(Max.x-Min.x);
 	y = (2*y-Min.y)/(Max.y-Min.y);
 	if(NumCol<1)		c = mglColor(x-x*y,y-x*y,x*y);
@@ -311,7 +316,7 @@ mglColor mglGraph::GetC2(float x,float y)
 	return c;
 }
 //-----------------------------------------------------------------------------
-void mglGraph::CRange(const mglData &a,bool add, float fact)
+void mglGraph::CRange(const mglData &a,bool add, mreal fact)
 {
 	long n = a.nx*a.ny*a.nz;
 	register long i;
@@ -323,11 +328,11 @@ void mglGraph::CRange(const mglData &a,bool add, float fact)
 		Cmax = Cmax>a.a[i] ? Cmax : a.a[i];
 	}
 	if(Cmin==Cmax)	Cmax += 1;
-	float dc = (Cmax-Cmin)*fact;
+	mreal dc = (Cmax-Cmin)*fact;
 	Cmax+=dc;	Cmin-=dc;
 }
 //-----------------------------------------------------------------------------
-void mglGraph::XRange(const mglData &a,bool add,float fact)
+void mglGraph::XRange(const mglData &a,bool add,mreal fact)
 {
 	long n = a.nx*a.ny*a.nz;
 	register long i;
@@ -338,14 +343,14 @@ void mglGraph::XRange(const mglData &a,bool add,float fact)
 		Min.x = Min.x<a.a[i] ? Min.x : a.a[i];
 		Max.x = Max.x>a.a[i] ? Max.x : a.a[i];
 	}
-	float dc = (Max.x-Min.x)*fact;
+	mreal dc = (Max.x-Min.x)*fact;
 	Max.x+=dc;	Min.x-=dc;
 	if(AutoOrg && Org.x<Min.x && !isnan(Org.x))	Org.x = Min.x;
 	if(AutoOrg && Org.x>Max.x && !isnan(Org.x))	Org.x = Max.x;
 	RecalcBorder();
 }
 //-----------------------------------------------------------------------------
-void mglGraph::YRange(const mglData &a,bool add,float fact)
+void mglGraph::YRange(const mglData &a,bool add,mreal fact)
 {
 	long n = a.nx*a.ny*a.nz;
 	register long i;
@@ -356,14 +361,14 @@ void mglGraph::YRange(const mglData &a,bool add,float fact)
 		Min.y = Min.y<a.a[i] ? Min.y : a.a[i];
 		Max.y = Max.y>a.a[i] ? Max.y : a.a[i];
 	}
-	float dc = (Max.y-Min.y)*fact;
+	mreal dc = (Max.y-Min.y)*fact;
 	Max.y+=dc;	Min.y-=dc;
 	if(AutoOrg && Org.y<Min.y && !isnan(Org.y))	Org.y = Min.y;
 	if(AutoOrg && Org.y>Max.y && !isnan(Org.y))	Org.y = Max.y;
 	RecalcBorder();
 }
 //-----------------------------------------------------------------------------
-void mglGraph::ZRange(const mglData &a,bool add,float fact)
+void mglGraph::ZRange(const mglData &a,bool add,mreal fact)
 {
 	long n = a.nx*a.ny*a.nz;
 	register long i;
@@ -374,7 +379,7 @@ void mglGraph::ZRange(const mglData &a,bool add,float fact)
 		Min.z = Min.z<a.a[i] ? Min.z : a.a[i];
 		Max.z = Max.z>a.a[i] ? Max.z : a.a[i];
 	}
-	float dc = (Max.z-Min.z)*fact;
+	mreal dc = (Max.z-Min.z)*fact;
 	Max.z+=dc;	Min.z-=dc;
 	if(AutoOrg && Org.z<Min.z && !isnan(Org.z))	Org.z = Min.z;
 	if(AutoOrg && Org.z>Max.z && !isnan(Org.z))	Org.z = Max.z;
@@ -428,9 +433,9 @@ char mglGraph::SelectPen(const char *p)
 	return mk;
 }
 //-----------------------------------------------------------------------------
-void mglGraph::SubPlot(int nx,int ny,int m, float dx, float dy)
+void mglGraph::SubPlot(int nx,int ny,int m, mreal dx, mreal dy)
 {
-	float x1,x2,y1,y2;
+	mreal x1,x2,y1,y2;
 	int mx = m%nx, my = m/nx;
 	if(AutoPlotFactor)	{	dx /= 1.55;	dy /= 1.55;	}
 	else	{	dx /= 2;	dy /= 2;	}
@@ -450,7 +455,7 @@ void mglGraph::Printf(mglPoint p,const char *str,...)
 	Puts(p,text,FontDef);
 }
 //-----------------------------------------------------------------------------
-float GetX(const mglData &x, int i, int j, int k)
+mreal GetX(const mglData &x, int i, int j, int k)
 {
 	k = k<x.nz ? k : 0;
 	if(x.ny>j && x.nx>i && x.ny>1)	return x.v(i,j,k);
@@ -458,7 +463,7 @@ float GetX(const mglData &x, int i, int j, int k)
 	return 0;
 }
 //-----------------------------------------------------------------------------
-float GetY(const mglData &y, int i, int j, int k)
+mreal GetY(const mglData &y, int i, int j, int k)
 {
 	k = k<y.nz ? k : 0;
 	if(y.ny>j && y.nx>i && y.ny>1)	return y.v(i,j,k);
@@ -466,7 +471,7 @@ float GetY(const mglData &y, int i, int j, int k)
 	return 0;
 }
 //-----------------------------------------------------------------------------
-float GetZ(const mglData &z, int i, int j, int k)
+mreal GetZ(const mglData &z, int i, int j, int k)
 {
 	k = k<z.nz ? k : 0;
 	if(z.ny>j && z.nx>i && z.ny>1)	return z.v(i,j,k);
@@ -474,7 +479,7 @@ float GetZ(const mglData &z, int i, int j, int k)
 	return 0;
 }
 //-----------------------------------------------------------------------------
-void mglColor::Set(mglColor c, float br)
+void mglColor::Set(mglColor c, mreal br)
 {
 	if(br<0)	br=0;	if(br>2.f)	br=2.f;
 	r= br<=1.f ? c.r*br : 1 - (1-c.r)*(2-br);
@@ -482,7 +487,7 @@ void mglColor::Set(mglColor c, float br)
 	b= br<=1.f ? c.b*br : 1 - (1-c.b)*(2-br);
 }
 //-----------------------------------------------------------------------------
-void mglColor::Set(char p, float bright)
+void mglColor::Set(char p, mreal bright)
 {
 	Set(-1,-1,-1);
 	for(long i=0; mglColorIds[i].id; i++)
@@ -490,7 +495,7 @@ void mglColor::Set(char p, float bright)
 		{	Set(mglColorIds[i].col, bright);	break;	}
 }
 //-----------------------------------------------------------------------------
-void mglGraph::ball(float *p,float *c)
+void mglGraph::ball(mreal *p,mreal *c)
 {
 	Ball(p[0],p[1],p[2],mglColor(c[0],c[1],c[2]),c[3]);
 }
@@ -502,26 +507,28 @@ void mglGraph::ClearEq()
 	RecalcBorder();
 }
 //-----------------------------------------------------------------------------
-void mglGraph::SetPalette(const char *colors)
+void mglGraph::SetPal(const char *colors)
 {
-	if(!colors || !colors[0])	return;	// do nothing if empty
+	if(!colors || !colors[0])	colors = MGL_DEF_PAL;	// restore default
 	memset(Pal,0,100*sizeof(mglColor));
 	memset(PalNames,0,101*sizeof(char));
-	int i,n = strlen(colors);
+	int i,n = strlen(colors),k;
 	n = n<100? n:100;
-	for(i=0;i<n;i++)
+	for(i=k=0;i<n;i++)
 	{
-		PalNames[i] = colors[i];
-		Pal[i] = mglColor(colors[i]);
+		if(!strchr("wkrgbcymhRGBCYMHWlenuqpLENUQP",colors[i]))	continue;
+		PalNames[k] = colors[i];	Pal[k] = mglColor(colors[i]);	k++;
 	}
-	CurrPal = 0;	NumPal = n;
+	NumPal = k;	//	CurrPal = 0;
+	if(k==0)	SetPal(0);	// if no colors then set default palette
 }
 //-----------------------------------------------------------------------------
 void mglGraph::DefaultPlotParam()
 {
+	FontSize = 5;			BaseLineWidth = 1;
 	Ambient();				Ternary(false);
-	PlotId = "frame";		SetPalette("Hbgrcmyhlnqeup");
-	SetScheme("BbcyrR");	SelectPen("k-1");
+	PlotId = "frame";		SelectPen("k-1");
+	SetScheme("BbcyrR");	SetPalette(MGL_DEF_PAL);
 	Cut = true;				InPlot(0,1,0,1);
 	SetTicks('x');	SetTicks('y');	SetTicks('z');
 	Axis(mglPoint(-1,-1,-1), mglPoint(1,1,1));
@@ -530,8 +537,7 @@ void mglGraph::DefaultPlotParam()
 	BarWidth = 0.7;			fit_res[0] = 0;
 	MarkSize = 0.02;		ArrowSize = 0.03;
 	AlphaDef = 0.5;			Transparent = true;
-	FontSize = 5;			BaseLineWidth = 1;
-	strcpy(FontDef,"rC");	AxialDir = 'y';
+	FontDef[0]=0;			AxialDir = 'y';
 	UseAlpha = false;		TranspType = 0;
 	RotatedText = true;		fnt->gr = this;
 	CloudFactor = 1;		MeshNum = 0;
@@ -544,13 +550,30 @@ void mglGraph::DefaultPlotParam()
 	Alpha(false);			Fog(0);
 	xtt=ytt=ztt=ctt=0;		FactorPos = 1.15;
 	AutoPlotFactor = true;	ScalePuts = true;
-	TickLen = 0.1;
+	TickLen = 0.1;			st_t = 1;
+	TickStl[0] = SubTStl[0] = 'k';
+	TickStl[1] = SubTStl[1] = 0;
 	PlotFactor = AutoPlotFactor ? 1.55f :2.f;
 	for(int i=0;i<10;i++)
 	{	Light(i, mglPoint(0,0,1));	Light(i,false);	}
 	Light(0,true);			Light(false);
 }
 //-----------------------------------------------------------------------------
+void mglGraph::SetTickStl(const char *stl, const char *sub)
+{
+	if(!stl || !(*stl))
+	{	TickStl[0] = SubTStl[0] = 0;	}
+	else if(strlen(stl)<32)
+	{
+		strcpy(TickStl,stl);
+		if(!sub || !(*sub))		strcpy(SubTStl,stl);
+		else if(strlen(sub)<32)	strcpy(SubTStl,sub);
+	}
+}
+//-----------------------------------------------------------------------------
+void mglGraph::SetTickLen(mreal tlen, mreal stt)
+{	TickLen=fabs(tlen)>0?tlen:0.1;	st_t=stt>0?stt:1;	}
+//-----------------------------------------------------------------------------
 void mglGraph::SimplePlot(const mglData &a, int type, const char *stl)
 {
 	if(a.nx<2)	{	SetWarn(mglWarnLow);	return;	}
@@ -579,7 +602,7 @@ void mglGraph::SimplePlot(const mglData &a, int type, const char *stl)
 	}
 }
 //-----------------------------------------------------------------------------
-void mglGraph::Zoom(float x1, float y1, float x2, float y2)
+void mglGraph::Zoom(mreal x1, mreal y1, mreal x2, mreal y2)
 {
 	if(x1==x2 || y1==y2)	{	x1=y1=0;	x2=y2=1;	}
 	Clf();
@@ -589,14 +612,14 @@ void mglGraph::Zoom(float x1, float y1, float x2, float y2)
 	else		{	zoomy1=y2;	zoomy2=y1-y2;	}
 }
 //-----------------------------------------------------------------------------
-void mglGraph::font_curve(long n,float *pp,bool *,long *nn,const wchar_t *text,
-				int pos,float size)
+void mglGraph::font_curve(long n,mreal *pp,bool *,long *nn,const wchar_t *text,
+				int pos,mreal size)
 {
 	if(n<2 || text==0 || wcslen(text)==0)	return;
 	if(!pp || !nn || n<2 || nn[0]<0)	return;
 	if(size<0)	size = -size*FontSize;
 	mglPoint p[25],t;
-	float del = fnt->Width(text,"rL")*size/8.;
+	mreal del = fnt->Width(text,"rL")*size/8.;
 	del = del>1 ? del:1;
 	long k=1,i,j;
 	bool less;
@@ -626,12 +649,12 @@ bool same_chain(long f,long i,long *nn)
 	}
 }
 //-----------------------------------------------------------------------------
-void mglGraph::string_curve(long f,long n,float *pp,long *nn,const wchar_t *text, float size, int pos)
+void mglGraph::string_curve(long f,long n,mreal *pp,long *nn,const wchar_t *text, mreal size, int pos)
 {
 	wchar_t L[2]=L"a";
 	mglPoint p1,n1,p2;
 
-	float w, r, ww, wg=fnt->Height("")*size/8.;//*font_factor;
+	mreal w, r, ww, wg=fnt->Height("")*size/8.;//*font_factor;
 	register long i,k,h;
 	ScalePuts = false;
 
@@ -724,39 +747,39 @@ void mglGraph::string_curve(long f,long n,float *pp,long *nn,const wchar_t *text
 }
 //-----------------------------------------------------------------------------
 // Pure virtual functions below are defined here for compatibility with MinGW
-void mglGraph::Pen(mglColor , char ,float ){}
+void mglGraph::Pen(mglColor , char ,mreal ){}
 void mglGraph::Light(int ,bool ){}
-void mglGraph::Light(int , mglPoint , mglColor , float , bool ){}
+void mglGraph::Light(int , mglPoint , mglColor , mreal , bool ){}
 void mglGraph::Clf(mglColor ){}
-void mglGraph::InPlot(float ,float ,float ,float ,bool ){}
+void mglGraph::InPlot(mreal ,mreal ,mreal ,mreal ,bool ){}
 //-----------------------------------------------------------------------------
 // These functions can be pure virtual but it should be empty (do nothing)
 // in some of inherit classes so they are defined empty here
 void mglGraph::SetSize(int ,int ){}
-float mglGraph::GetRatio(){return 1.;}
+mreal mglGraph::GetRatio(){return 1.;}
 void mglGraph::Flush(){}
 void mglGraph::Finish(){}
 //-----------------------------------------------------------------------------
 void mglGraph::Mark(mglPoint p,char t)
 {	ScalePoint(p.x,p.y,p.z);	Mark(p.x,p.y,p.z,t);	}
 //-----------------------------------------------------------------------------
-void mglGraph::View(float tetx,float tetz,float tety)
+void mglGraph::View(mreal tetx,mreal tetz,mreal tety)
 {	_tetx=tetx;	_tety=tety;	_tetz=tetz;	}
 void mglGraph::Identity()	{	InPlot(0,1,0,1);	}
 //-----------------------------------------------------------------------------
-void mglGraph::Rotate(float TetX,float TetZ,float TetY)
+void mglGraph::Rotate(mreal TetX,mreal TetZ,mreal TetY)
 {
 	RotateN(TetX+_tetx,1.,0.,0.);
 	RotateN(TetY+_tety,0.,1.,0.);
 	RotateN(TetZ+_tetz,0.,0.,1.);
 }
 //-----------------------------------------------------------------------------
-void mglGraph::Fog(float d, float dz)	{	FogDist=d;	FogDz = dz;	}
+void mglGraph::Fog(mreal d, mreal dz)	{	FogDist=d;	FogDz = dz;	}
 //-----------------------------------------------------------------------------
-void mglGraph::Light(int n,mglPoint p, char c, float bright, bool infty)
+void mglGraph::Light(int n,mglPoint p, char c, mreal bright, bool infty)
 {	Light(n,p,mglColor(c),bright,infty);	}
 //-----------------------------------------------------------------------------
-void mglGraph::Ambient(float bright)	{	AmbBr = bright;	}
+void mglGraph::Ambient(mreal bright)	{	AmbBr = bright;	}
 //-----------------------------------------------------------------------------
 mglGraph::mglGraph()
 {
@@ -788,18 +811,21 @@ void mglGraph::SetWarn(int code, const char *who)
 //-----------------------------------------------------------------------------
 void mglGraph::SetFont(mglFont *f)
 {
-	if(f)	fnt->Copy(f);					// set new typeface and delete existed
-	else	fnt->Load(MGL_DEF_FONT_NAME,0);	//	restore (reload) default typeface
+	if(f)	fnt->Copy(f);		// set new typeface and delete existed
+	else	fnt->Load(MGL_DEF_FONT_NAME,0);	// restore default typeface
 }
 //-----------------------------------------------------------------------------
-void mglGraph::Title(const wchar_t *str,const char *font,float size)
+void mglGraph::Title(const wchar_t *str,const char *font,mreal size)
 {
 	Identity();
-	Text(mglPoint(0,1.3,0), str, font, size);
+	mglFormula *ox=fx, *oy=fy, *oz=fz;
+	fx = fy = fz = NULL;
+	Text(mglPoint((Min.x+Max.x)*0.5, Max.y+(Max.y-Min.y)*0.15, (Min.z+Max.z)*0.5), str, font, size);
+	fx=ox;	fy=oy;	fz=oz;
 	RestoreM();
 }
 //-----------------------------------------------------------------------------
-void mglGraph::Title(const char *str,const char *font,float size)
+void mglGraph::Title(const char *str,const char *font,mreal size)
 {
 	unsigned s = strlen(str)+1;
 	wchar_t *wcs = new wchar_t[s];
@@ -808,7 +834,7 @@ void mglGraph::Title(const char *str,const char *font,float size)
 	delete []wcs;
 }
 //-----------------------------------------------------------------------------
-void mglGraph::Label(char dir, const char *str, int pos, float size, float shift)
+void mglGraph::Label(char dir, const char *str, int pos, mreal size, mreal shift)
 {
 	unsigned s = strlen(str)+1;
 	wchar_t *wcs = new wchar_t[s];
@@ -817,7 +843,7 @@ void mglGraph::Label(char dir, const char *str, int pos, float size, float shift
 	delete []wcs;
 }
 //-----------------------------------------------------------------------------
-void mglGraph::Puts(mglPoint p,const char *str,const char *font,float size,char dir,float shift)
+void mglGraph::Puts(mglPoint p,const char *str,const char *font,mreal size,char dir,mreal shift)
 {
 	unsigned s = strlen(str)+1;
 	wchar_t *wcs = new wchar_t[s];
@@ -826,17 +852,17 @@ void mglGraph::Puts(mglPoint p,const char *str,const char *font,float size,char
 	delete []wcs;
 }
 //-----------------------------------------------------------------------------
-float mglGraph::Puts(mglPoint p,mglPoint l,const char *str,char font,float size)
+mreal mglGraph::Puts(mglPoint p,mglPoint l,const char *str,char font,mreal size)
 {
 	unsigned s = strlen(str)+1;
 	wchar_t *wcs = new wchar_t[s];
 	mbstowcs(wcs,str,s);
-	float res = Putsw(p, l, wcs, font, size);
+	mreal res = Putsw(p, l, wcs, font, size);
 	delete []wcs;
 	return res;
 }
 //-----------------------------------------------------------------------------
-void mglGraph::Text(mglPoint p,const char *text,const char *font,float size,char dir)
+void mglGraph::Text(mglPoint p,const char *text,const char *font,mreal size,char dir)
 {
 	bool rt = RotatedText;
 	RotatedText = false;
@@ -844,7 +870,7 @@ void mglGraph::Text(mglPoint p,const char *text,const char *font,float size,char
 	RotatedText = rt;
 }
 //-----------------------------------------------------------------------------
-void mglGraph::Text(mglPoint p,const wchar_t *text,const char *font,float size,char dir)
+void mglGraph::Text(mglPoint p,const wchar_t *text,const char *font,mreal size,char dir)
 {
 	bool rt = RotatedText;
 	RotatedText = false;
@@ -852,7 +878,7 @@ void mglGraph::Text(mglPoint p,const wchar_t *text,const char *font,float size,c
 	RotatedText = rt;
 }
 //-----------------------------------------------------------------------------
-void mglGraph::Text(const mglData &y,const char *str,const char *font,float size,float zVal)
+void mglGraph::Text(const mglData &y,const char *str,const char *font,mreal size,mreal zVal)
 {
 	unsigned s = strlen(str)+1;
 	wchar_t *wcs = new wchar_t[s];
@@ -861,7 +887,7 @@ void mglGraph::Text(const mglData &y,const char *str,const char *font,float size
 	delete []wcs;
 }
 //-----------------------------------------------------------------------------
-void mglGraph::Text(const mglData &x,const mglData &y,const char *str,const char *font,float size,float zVal)
+void mglGraph::Text(const mglData &x,const mglData &y,const char *str,const char *font,mreal size,mreal zVal)
 {
 	unsigned s = strlen(str)+1;
 	wchar_t *wcs = new wchar_t[s];
@@ -870,7 +896,7 @@ void mglGraph::Text(const mglData &x,const mglData &y,const char *str,const char
 	delete []wcs;
 }
 //-----------------------------------------------------------------------------
-void mglGraph::Text(const mglData &x,const mglData &y,const mglData &z,const char *str,const char *font,float size)
+void mglGraph::Text(const mglData &x,const mglData &y,const mglData &z,const char *str,const char *font,mreal size)
 {
 	unsigned s = strlen(str)+1;
 	wchar_t *wcs = new wchar_t[s];
@@ -888,7 +914,7 @@ void mglGraph::TextMark(const mglData &x, const mglData &y, const mglData &z, co
 	delete []wcs;
 }
 //-----------------------------------------------------------------------------
-void mglGraph::TextMark(const mglData &x, const mglData &y, const mglData &r, const char *str, const char *fnt, float zVal)
+void mglGraph::TextMark(const mglData &x, const mglData &y, const mglData &r, const char *str, const char *fnt, mreal zVal)
 {
 	unsigned s = strlen(str)+1;
 	wchar_t *wcs = new wchar_t[s];
@@ -897,7 +923,7 @@ void mglGraph::TextMark(const mglData &x, const mglData &y, const mglData &r, co
 	delete []wcs;
 }
 //-----------------------------------------------------------------------------
-void mglGraph::TextMark(const mglData &y, const mglData &r, const char *str, const char *fnt, float zVal)
+void mglGraph::TextMark(const mglData &y, const mglData &r, const char *str, const char *fnt, mreal zVal)
 {
 	unsigned s = strlen(str)+1;
 	wchar_t *wcs = new wchar_t[s];
@@ -906,7 +932,7 @@ void mglGraph::TextMark(const mglData &y, const mglData &r, const char *str, con
 	delete []wcs;
 }
 //-----------------------------------------------------------------------------
-void mglGraph::TextMark(const mglData &y, const char *str, const char *fnt, float zVal)
+void mglGraph::TextMark(const mglData &y, const char *str, const char *fnt, mreal zVal)
 {
 	unsigned s = strlen(str)+1;
 	wchar_t *wcs = new wchar_t[s];
@@ -925,24 +951,24 @@ void mglGraph::AddLegend(const char *str,const char *style)
 	delete []wcs;
 }
 //-----------------------------------------------------------------------------
-void mglGraph::SetFontSizePT(float pt, int dpi)
+void mglGraph::SetFontSizePT(mreal pt, int dpi)
 {	FontSize = pt*27.f/dpi;	}
 //-----------------------------------------------------------------------------
-float mglGraph::GetOrgX(char)	{	return isnan(Org.x) ? Min.x : Org.x;	}
-float mglGraph::GetOrgY(char)	{	return isnan(Org.y) ? Min.y : Org.y;	}
-float mglGraph::GetOrgZ(char)	{	return isnan(Org.z) ? Min.z : Org.z;	}
+mreal mglGraph::GetOrgX(char)	{	return isnan(Org.x) ? Min.x : Org.x;	}
+mreal mglGraph::GetOrgY(char)	{	return isnan(Org.y) ? Min.y : Org.y;	}
+mreal mglGraph::GetOrgZ(char)	{	return isnan(Org.z) ? Min.z : Org.z;	}
 //-----------------------------------------------------------------------------
-void mglGraph::FaceX(float x0, float y0, float z0, float wy, float wz, const char *stl, float d1, float d2)
+void mglGraph::FaceX(mreal x0, mreal y0, mreal z0, mreal wy, mreal wz, const char *stl, mreal d1, mreal d2)
 {
 	Face(mglPoint(x0,y0,z0), mglPoint(x0,y0+wy,z0), mglPoint(x0,y0,z0+wz), mglPoint(x0,y0+wy+d1,z0+wz+d2), stl, 2);
 }
 //-----------------------------------------------------------------------------
-void mglGraph::FaceY(float x0, float y0, float z0, float wx, float wz, const char *stl, float d1, float d2)
+void mglGraph::FaceY(mreal x0, mreal y0, mreal z0, mreal wx, mreal wz, const char *stl, mreal d1, mreal d2)
 {
 	Face(mglPoint(x0,y0,z0), mglPoint(x0+wx,y0,z0), mglPoint(x0,y0,z0+wz), mglPoint(x0+wx+d1,y0,z0+wz+d2), stl, 2);
 }
 //-----------------------------------------------------------------------------
-void mglGraph::FaceZ(float x0, float y0, float z0, float wx, float wy, const char *stl, float d1, float d2)
+void mglGraph::FaceZ(mreal x0, mreal y0, mreal z0, mreal wx, mreal wy, const char *stl, mreal d1, mreal d2)
 {
 	Face(mglPoint(x0,y0,z0), mglPoint(x0,y0+wy,z0), mglPoint(x0+wx,y0,z0), mglPoint(x0+wx+d1,y0+wy+d2,z0), stl, 2);
 }
@@ -952,7 +978,8 @@ void mglGraph::ShowImage(const char *viewer, bool keep)
 	char fname[128], *cmd = new char [128];
 	sprintf(fname,"%s.png", tmpnam(NULL));
 	WritePNG(fname,"MathGL ShowImage file",false);
-	viewer = (viewer && viewer[0]) ? viewer : "kuickshow";
+	if(!viewer || !viewer[0])
+		viewer = MGL_DEF_VIEWER;
 	if(keep)
 	{
 		sprintf(cmd,"%s %s &", viewer,fname);
@@ -981,7 +1008,7 @@ void mglGraph::StartGroup(const char *name, int id)
 	StartAutoGroup(buf);
 }
 //-----------------------------------------------------------------------------
-void mglGraph::SetAutoRanges(float x1, float x2, float y1, float y2, float z1, float z2)
+void mglGraph::SetAutoRanges(mreal x1, mreal x2, mreal y1, mreal y2, mreal z1, mreal z2)
 {
 	if(x1!=x2)	{	Min.x = x1;	Max.x = x2;	}
 	if(y1!=y2)	{	Min.y = y1;	Max.y = y2;	}
@@ -991,16 +1018,77 @@ void mglGraph::SetAutoRanges(float x1, float x2, float y1, float y2, float z1, f
 void mglGraph::Colorbar(const char *sch,int where)
 {
 	SetScheme(sch);
-	float x=0,y=0;
+	mreal x=0,y=0;
 	if(where==2)	y=1;
 	if(where==0)	x=1;
 	Colorbar(where,x,y,1,1);
 }
 //-----------------------------------------------------------------------------
+void mglGraph::Colorbar(const mglData &v, const char *sch,int where)
+{
+	mreal x=0,y=0;
+	if(where==2)	y=1;
+	if(where==0)	x=1;
+	Colorbar(v,sch,where,x,y,1,1);
+}
+//-----------------------------------------------------------------------------
+void mglGraph::Colorbar(int where, mreal x, mreal y, mreal w, mreal h)
+{
+	float d = fabs(Cmax-Cmin);
+	d = floor(d*pow(10,-floor(log10(d))));
+	long n = 50*(d<4?int(2*d+0.5):int(d+0.5))+1;
+	if(d==1.f)	n = 101;
+	mglData v(n);	v.Fill(Cmin,Cmax);
+	mglColor *c=new mglColor[n];
+	for(long i=0;i<n;i++)	c[i] = GetC(v.a[i],true);
+	colorbar(v, c, where, x, y, w, h);
+	delete []c;
+}
+//-----------------------------------------------------------------------------
+void mglGraph::Colorbar(const mglData &v, const char *sch, int where, mreal x, mreal y, mreal w, mreal h)
+{
+	mglColor *c=new mglColor[v.nx];
+	if(!sch || !(*sch))	sch = PalNames;
+	for(long i=0;i<v.nx;i++)	c[i] = mglColor(sch[i%strlen(sch)]);
+	colorbar(v, c, where, x, y, w, h);
+	delete []c;
+}
+//-----------------------------------------------------------------------------
 void mglGraph::ColumnPlot(int num, int i)
 {
-	float d = i/(num+PlotFactor-1);
-	float w = PlotFactor/(num+PlotFactor-1);
+	mreal d = i/(num+PlotFactor-1);
+	mreal w = PlotFactor/(num+PlotFactor-1);
 	InPlot(0,1,d,d+w,true);
 }
 //-----------------------------------------------------------------------------
+void mglGraph::Axis(int how)
+{
+	switch(how)
+	{
+	case mglCartesian:	Axis(0,0,0);	break;
+	case mglPolar:
+		Axis("x*cos(y)","x*sin(y)",0);	break;
+	case mglSpherical:
+		Axis("x*sin(y)*cos(z)","x*sin(y)*sin(z)","x*cos(y)");	break;
+	case mglParabolic:
+		Axis("x*y","(x*x-y*y)/2",0);	break;
+	case mglParaboloidal:
+		Axis("(x*x-y*y)*cos(z)/2","(x*x-y*y)*sin(z)/2","x*y");	break;
+	case mglOblate:
+		Axis("cosh(x)*cos(y)*cos(z)","cosh(x)*cos(y)*sin(z)","sinh(x)*sin(y)");	break;
+//		Axis("x*y*cos(z)","x*y*sin(z)","(x*x-1)*(1-y*y)");	break;	
+	case mglProlate:
+		Axis("sinh(x)*sin(y)*cos(z)","sinh(x)*sin(y)*sin(z)","cosh(x)*cos(y)");	break;
+	case mglElliptic:
+		Axis("cosh(x)*cos(y)","sinh(x)*sin(y)",0);	break;
+	case mglToroidal:
+		Axis("sinh(x)*cos(z)/(cosh(x)-cos(y))","sinh(x)*sin(z)/(cosh(x)-cos(y))",
+		     "sin(y)/(cosh(x)-cos(y))");	break;
+	case mglBispherical:
+		Axis("sin(y)*cos(z)/(cosh(x)-cos(y))","sin(y)*sin(z)/(cosh(x)-cos(y))",
+		     "sinh(x)/(cosh(x)-cos(y))");	break;
+	case mglBipolar:
+		Axis("sinh(x)/(cosh(x)-cos(y))","sin(y)/(cosh(x)-cos(y))",0);	break;
+	default:	Axis(0,0,0);	break;
+	}
+}
