@@ -27,10 +27,7 @@
 
 #ifdef WIN32
 #include <io.h>
-wchar_t *wcstokw32(wchar_t *wcs, const wchar_t *delim){
-    wcstok(wcs,delim);
-}
-#define swprintf    _snwprintf
+wchar_t *wcstokw32(wchar_t *wcs, const wchar_t *delim)	{	wcstok(wcs,delim);	}
 #define wcstok(a,b,c) wcstokw32(a,b)
 #else
 #include <unistd.h>
@@ -43,12 +40,10 @@ void mgl_wcstrim(wchar_t *str)
 	wchar_t *c = mgl_wcsdup(str);
 	unsigned long n=wcslen(str);
 	long k;
-	for(k=0;k<long(wcslen(str));k++)	//   
-		if(str[k]>' ')	break;
+	for(k=0;k<long(wcslen(str));k++)		if(str[k]>' ')	break;
 	wcscpy(c,&(str[k]));
 	n = wcslen(c);
-	for(k=n-1;k>=0;k--)	//   
-		if(c[k]>' ')		break;
+	for(k=n-1;k>=0;k--)	if(c[k]>' ')	break;
 	c[k+1] = 0;
 	wcscpy(str,c);	free(c);
 }
@@ -78,7 +73,7 @@ bool check_for_name(const wchar_t *s)
 void mgl_wcstombs(char *dst, const wchar_t *src, int size)
 {
 	register int j;
-	for(j=0;src[j] && j<size-1;j++)
+	for(j=0;j<size-1 && src[j]!=0;j++)
 		dst[j] = src[j]<0x7f ? src[j] : ' ';
 	dst[j] = 0;
 }
@@ -89,7 +84,7 @@ int mglParse::Exec(mglGraph *gr, const wchar_t *com, long n, mglArg *a, const wc
 	int k[10], i;
 	for(i=0;i<10;i++)	k[i] = i<n ? a[i].type + 1 : 0;
 //	for(i=0;i<n;i++)	wcstombs(a[i].s, a[i].w, 1024);
-	for(i=0;i<n;i++)	mgl_wcstombs(a[i].s, a[i].w, 1024);
+	for(i=0;i<n;i++)		mgl_wcstombs(a[i].s, a[i].w, 1024);
 	mglCommand tst, *rts;
 	for(i=0;Cmd[i].name[0];i++);	// determine the number of symbols
 	tst.name = com;
@@ -133,13 +128,12 @@ mglVar::~mglVar()
 //-----------------------------------------------------------------------------
 mglParse::mglParse(bool setsize)
 {
-	DataList=0;		Cmd = mgls_base_cmd;
+	memset(this,0,sizeof(mglParse));
+	Cmd = mgls_base_cmd;
 	AllowSetSize=setsize;
-	Once = true;	Skip = false;	out = 0;
-	parlen=320;		Stop=false;		if_pos = 0;
-	for(long i=0;i<10;i++)	{	par[i] = 0;	for_stack[i]=0;	opt[i]=false;	}
-	opt[10] = false;
-	op1 = new wchar_t[2048];	op2 = new wchar_t[2048];
+	Once = true;	parlen=320;
+	op1 = new wchar_t[4096];	op2 = new wchar_t[4096];
+	fval = new mglData[10];
 }
 //-----------------------------------------------------------------------------
 mglParse::~mglParse()
@@ -150,7 +144,7 @@ mglParse::~mglParse()
 		delete DataList;
 	}
 	for(long i=0;i<10;i++)	if(par[i])	delete [] par[i];
-	delete []op1;	delete []op2;
+	delete []op1;	delete []op2;	delete []fval;
 	if(Cmd!=mgls_base_cmd)	delete []Cmd;
 }
 //-----------------------------------------------------------------------------
@@ -205,14 +199,15 @@ bool mglParse::AddParam(int n, const wchar_t *str, bool isstr)
 mglVar *mglParse::FindVar(const wchar_t *name)
 {
 	mglVar *v=DataList;
-	wchar_t *s = mgl_wcsdup(name),*p;
-	p = wcschr(s,'.');	if(p)	{	p[0]=0;	p++;	}
+//	wchar_t *s = mgl_wcsdup(name),*p;
+//	p = wcschr(s,'.');	if(p)	{	p[0]=0;	p++;	}
 	while(v)
 	{
-		if(!wcscmp(s, v->s))	{	free(s);	return v;	}
+//		if(!wcscmp(s, v->s))	{	free(s);	return v;	}
+		if(!wcscmp(name, v->s))	return v;
 		v = v->next;
 	}
-	free(s);
+//	free(s);
 	return 0;
 }
 //-----------------------------------------------------------------------------
@@ -234,7 +229,7 @@ int mglFindArg(const wchar_t *str)
 	{
 		if(l%2==0 && (str[i]=='#' || str[i]==';'))	return -i;
 		if(str[i]=='\'') l++;
-		if(l%2==0 && (str[i]==' ' || str[i]=='\t' || str[i]=='\n'))	return i;
+		if(l%2==0 && (str[i]<=' '))	return i;
 	}
 	return 0;
 }
@@ -242,9 +237,9 @@ int mglFindArg(const wchar_t *str)
 void wcstrim_mgl(wchar_t *str);
 void wcslwr_mgl(wchar_t *str);
 //-----------------------------------------------------------------------------
-bool mgls_suffix(const wchar_t *p, mglData *d, float *v)
+bool mgls_suffix(const wchar_t *p, mglData *d, mreal *v)
 {
-	float x,y,z,k;
+	mreal x,y,z,k;
 	bool ok=false;
 	if(!wcscmp(p,L"a"))			{	ok = true;	*v = d->a[0];	}
 	else if(!wcscmp(p,L"fst"))	{	ok = true;	int i=-1,j=-1,k=-1;	*v = d->Find(0,i,j,k);	}
@@ -278,86 +273,41 @@ bool mgls_suffix(const wchar_t *p, mglData *d, float *v)
 }
 //-----------------------------------------------------------------------------
 // convert substrings to arguments
+mglData mglFormulaCalc(const wchar_t *string, mglParse *arg);
 void mglParse::FillArg(int k, wchar_t **arg, mglArg *a)
 {
-	wchar_t *s, *t, *p=0, *b;
-	register long n,ii;
+	register long n;
 	for(n=1;n<k;n++)
 	{
 		mglVar *v, *u;
 		a[n-1].type = -1;
-		for(b=arg[n],ii=0,p=0;*b!=0;b++)
-		{
-			if(*b=='\'')	ii = 1-ii;
-			if(ii==0 && *b=='.')
-			{	p = b;	break;	}
-		}
-//		p = wcschr(arg[n],'.');
-		if(arg[n][0]!='\'' && (s=wcschr(arg[n],'(')))	// subdata or column in argument
-		{
-			// separate suffix only HERE !!!
-			if(p)	{	p[0]=0;	p++;	}
-			s = mgl_wcsdup(arg[n]);
-			t = wcstok (s, L"(,)",&b);
-			v = FindVar(t);
-			if(v==0)
-			{	a[n-1].type = 2;	a[n-1].v = NAN;		free(s);	return;	}
-			if(!wcschr(arg[n],'\''))	// this subdata
-			{
-				long k[3]={-1,-1,-1};
-				// continue parsing
-				t = wcstok (0, L"(,)",&b);	if(t)	k[0] = t[0]==':'?-1:wcstol(t,0,0);
-				t = wcstok (0, L"(,)",&b);	if(t)	k[1] = t[0]==':'?-1:wcstol(t,0,0);
-				t = wcstok (0, L"(,)",&b);	if(t)	k[2] = t[0]==':'?-1:wcstol(t,0,0);
-				u = new mglVar;		u->temp = true;
-				swprintf(a[n-1].w,2048,L"%ls.SubData(%d,%d,%d)",v->s,k[0],k[1],k[2]);
-				u->d = v->d.SubData(k[0],k[1],k[2]);
-			}
-			else	// this is column formula
-			{
-				wchar_t *tt = wcstok(0,L"'",&b);
-				t = wcstok(0,L"'",&b);	t[-1]=0;
-				u = new mglVar;		u->temp = true;
-				char *cc = new char[wcslen(tt)+1];
-//				wcstombs(cc,tt,wcslen(tt)+1);
-				mgl_wcstombs(cc,tt,wcslen(tt)+1);
-				swprintf(a[n-1].w,2048,L"%ls.Column(\"%s\")",v->s,cc);
-				u->d = v->d.Column(cc);
-				delete []cc;
-			}
-			free(s);
-			if(DataList)	u->MoveAfter(DataList);
-			else			DataList = u;
-			a[n-1].type = 0;	a[n-1].d = &(u->d);
-		}
-		else if(arg[n][0]=='|')	a[n-1].type = -1;
+		if(arg[n][0]=='|')	a[n-1].type = -1;
 		else if(arg[n][0]=='\'')
-		{
+		{	// this is string (simplest case)
 			a[n-1].type = 1;	arg[n][wcslen(arg[n])-1] = 0;
 			if(wcslen(arg[n]+1)>=2048)	arg[n][2048]=0;
 			wcscpy(a[n-1].w, arg[n]+1);
 		}
 		else if((v = FindVar(arg[n]))!=0)
-		{
-			p = wcschr(arg[n],'.');	if(p)	{	p[0]=0;	p++;	}
+		{	// have to find normal variables (for data creation)
 			a[n-1].type = 0;		a[n-1].d = &(v->d);
 			wcscpy(a[n-1].w, v->s);
 		}
 		else
-		{
-			a[n-1].type = 2;
-			if(!wcscmp(arg[n],L"nan"))	a[n-1].v = NAN;
-			else if(!wcscmp(arg[n],L":"))	a[n-1].v = -1;
-			else if(!wcscmp(arg[n],L"pi"))	a[n-1].v = M_PI;
-			else if(!wcscmp(arg[n],L"-pi"))	a[n-1].v = -M_PI;
-			else if(!wcscmp(arg[n],L"on"))	a[n-1].v = 1;
-			else if(!wcscmp(arg[n],L"off"))	a[n-1].v = 0;
-			else	a[n-1].v = wcstod(arg[n],0);
+		{	// parse all numbers and formulas by unified way
+			const mglData &d=mglFormulaCalc(arg[n], this);
+//printf("arg: %ls, -- dat: %ld x %ld x %ld, -- val: %g\n",arg[n],d.nx,d.ny,d.nz,d.a[0]);
+			if(d.nx*d.ny*d.nz==1)
+			{	a[n-1].type = 2;	a[n-1].v = d.a[0];	}
+			else
+			{
+				u=new mglVar;	u->temp=true;	u->d=d;
+				mglprintf(a[n-1].w,2048,L"/*%ls*/",arg[n]);
+				if(DataList)	u->MoveAfter(DataList);
+				else			DataList = u;
+				a[n-1].type = 0;	a[n-1].d = &(u->d);
+			}
 		}
-		// try to find numbers (momentum) for data
-		if(a[n-1].type == 0 && p)
-			if(mgls_suffix(p,a[n-1].d,&(a[n-1].v)))
-				a[n-1].type = 2;
 	}
 }
 //-----------------------------------------------------------------------------
@@ -419,7 +369,7 @@ int mglParse::PreExec(mglGraph *gr, long k, wchar_t **arg, mglArg *a)
 				{
 					b = a[i-1].d;
 					if(b==0 || nn!=b->nx*b->ny)	continue;
-					memcpy(v->d.a+j*nn,b->a,nn*(b->nz)*sizeof(float));
+					memcpy(v->d.a+j*nn,b->a,nn*(b->nz)*sizeof(mreal));
 					j+=b->nz;
 				}
 			}
@@ -431,7 +381,7 @@ int mglParse::PreExec(mglGraph *gr, long k, wchar_t **arg, mglArg *a)
 				{
 					b = a[i-1].d;
 					if(b==0 || nn!=b->nx)	continue;
-					memcpy(v->d.a+j*nn,b->a,nn*(b->ny)*sizeof(float));
+					memcpy(v->d.a+j*nn,b->a,nn*(b->ny)*sizeof(mreal));
 					j+=b->ny;
 				}
 			}
@@ -537,19 +487,20 @@ int mglParse::Parse(mglGraph *gr, const wchar_t *string, long pos)
 				}
 				else if(a[1].type==2 && a[2].type==2 && a[2].v>a[1].v)
 				{
-					float step = a[3].type==2?a[3].v:1;
+					mreal step = a[3].type==2?a[3].v:1;
 					int m = int(step>0 ? (a[2].v-a[1].v)/step : 0);
 					if(m>0)
 					{
 						n=0;	fval[r].Create(m+1);
-						fval[r].Fill(a[1].v, a[2].v);
+						for(int ii=0;ii<m+1;ii++)
+							fval[r].a[ii] = a[1].v + step*ii;
 					}
 				}
 				if(n==0)
 				{
 					for(int i=9;i>0;i--)	for_stack[i] = for_stack[i-1];
 					for_stack[0] = r+1;		fval[r].nz = pos;
-					wchar_t buf[32];		swprintf(buf,32,L"%g",fval[r].a[0]);
+					wchar_t buf[32];		mglprintf(buf,32,L"%g",fval[r].a[0]);
 					AddParam(r, buf,true);	fval[r].ny = 1;
 				}
 			}
@@ -584,9 +535,9 @@ int mglParse::FlowExec(mglGraph *gr, const wchar_t *com, long m, mglArg *a)
 		{
 			n = 0;
 			if(a[0].v)
-			{	Skip = !Once;	if(out)	swprintf(out,1024,L"if(!once)\t{\tonce = true;");	}
+			{	Skip = !Once;	if(out)	mglprintf(out,1024,L"if(!once)\t{\tonce = true;");	}
 			else
-			{	Skip = Once = false;	if(out)	swprintf(out,1024,L"}");	}
+			{	Skip = Once = false;	if(out)	mglprintf(out,1024,L"}");	}
 		}
 		else n = 1;
 	}
@@ -596,13 +547,13 @@ int mglParse::FlowExec(mglGraph *gr, const wchar_t *com, long m, mglArg *a)
 		if(a[0].type==2)
 		{
 			n = 0;	cond = (a[0].v!=0)?3:0;
-			if(out)	swprintf(out,1024,L"if(%g!=0)\t{", a[0].v);
+			if(out)	mglprintf(out,1024,L"if(%g!=0)\t{", a[0].v);
 		}
 		else if(m>1 && a[0].type==0 && a[1].type==1)
 		{
 			n = 0;	mgl_wcstombs(a[1].s, a[1].w, 1024);
 			cond = a[0].d->FindAny(a[1].s)?3:0;
-			if(out)	swprintf(out,1024,L"if(%ls.FindAny(\"%s\"))\t{", a[0].w, a[1].s);
+			if(out)	mglprintf(out,1024,L"if(%ls.FindAny(\"%s\"))\t{", a[0].w, a[1].s);
 		}
 		else n = 1;
 		if(n==0)
@@ -611,14 +562,14 @@ int mglParse::FlowExec(mglGraph *gr, const wchar_t *com, long m, mglArg *a)
 	else if(!wcscmp(com,L"endif"))
 	{
 		if_pos = if_pos>0 ? if_pos-1 : 0;
-		n = 0;	if(out)	swprintf(out,1024,L"}");
+		n = 0;	if(out)	mglprintf(out,1024,L"}");
 	}
 	else if(!wcscmp(com,L"else"))
 	{
 		if(if_pos>0)
 		{
 			n=0; if_stack[if_pos-1] = (if_stack[if_pos-1]&2)?2:3;
-			if(out)	swprintf(out,1024,L"}\telse\t{");
+			if(out)	mglprintf(out,1024,L"}\telse\t{");
 		}
 		else n = 1;
 	}
@@ -635,9 +586,9 @@ int mglParse::FlowExec(mglGraph *gr, const wchar_t *com, long m, mglArg *a)
 		{
 			if_stack[if_pos-1] = cond;
 			if(out && a[0].type==2)
-				swprintf(out,1024,L"else if(%g!=0)\t{", a[0].v);
+				mglprintf(out,1024,L"else if(%g!=0)\t{", a[0].v);
 			if(out && m>1 && a[0].type==0 && a[1].type==1)
-				swprintf(out,1024,L"else if(%ls.FindAny(\"%ls\"))\t{", a[0].w, a[1].w);
+				mglprintf(out,1024,L"else if(%ls.FindAny(\"%ls\"))\t{", a[0].w, a[1].w);
 		}
 	}
 	else if(!wcscmp(com,L"next"))
@@ -648,7 +599,7 @@ int mglParse::FlowExec(mglGraph *gr, const wchar_t *com, long m, mglArg *a)
 		{
 			if(fval[r].ny<fval[r].nx)
 			{
-				wchar_t buf[32];		swprintf(buf,32,L"%g",fval[r].a[fval[r].ny]);
+				wchar_t buf[32];		mglprintf(buf,32,L"%g",fval[r].a[fval[r].ny]);
 				AddParam(r, buf,true);	fval[r].ny += 1;
 				n = -fval[r].nz-1;
 			}
@@ -662,32 +613,32 @@ int mglParse::FlowExec(mglGraph *gr, const wchar_t *com, long m, mglArg *a)
 	return n+1;
 }
 //-----------------------------------------------------------------------------
+void mgl_error_print(int line, int r, mglGraph *gr)
+{
+	if(r==0)	printf("%s\n",gr->Message);
+	if(r==1)	printf("Wrong argument(s) in line %d\n", line);
+	if(r==2)	printf("Wrong command in line %d\n", line);
+	if(r==3)	printf("String too long in line %d\n", line);
+	if(r==4)	printf("Unbalanced ' in line %d\n", line);
+}
 void mglParse::Execute(mglGraph *gr, FILE *fp, bool print)
 {
 	if(gr==0 || fp==0)	return;
-	wchar_t str[8192];
-	int r, line=0;
-	Stop = false;
-	while(!feof(fp))
+	fseek(fp,0,SEEK_END);	// get file size
+	long len=ftell(fp),cur=0;
+	fseek(fp,0,SEEK_SET);	// restore back
+	wchar_t *str=new wchar_t[len+1];
+	if(len>0)	for(cur=0;cur<len;cur++)
 	{
-		if(!fgetws(str,8192,fp))	break;
-		line++;
-		if(gr->Message) gr->Message[0] = 0;
-		r = Parse(gr,str,ftell(fp));
-		if(r<0)	{	fseek(fp,-r-1,SEEK_SET);	continue;	}
-
-		if(print)
-		{
-			if(gr->Message && gr->Message[0])	printf("%s\n",gr->Message);
-			if(r==1)	printf("Wrong argument(s) in line %d -- %ls\n", line, str);
-			if(r==2)	printf("Wrong command in line %d -- %ls\n", line, str);
-			if(r==3)	printf("String too long in line %d -- %ls\n", line, str);
-			if(r==4)	printf("Unbalanced ' in line %d -- %ls\n", line, str);
-		}
+		str[cur] = fgetwc(fp);
+		if(cur==0 && str[cur]==wchar_t(-1))	cur--;	// Shaytan, but it works!!!
 	}
+	str[len]=0;
+	Execute(gr,str,print?mgl_error_print:NULL);
+	delete []str;
 }
 //-----------------------------------------------------------------------------
-void mglParse::Execute(mglGraph *gr, int n, const wchar_t **text, void (*error)(int line, int kind))
+void mglParse::Execute(mglGraph *gr, int n, const wchar_t **text, void (*error)(int line, int kind, mglGraph *gr))
 {
 	if(gr==0 || n<1 || text==0)	return;
 	long i, r;
@@ -697,13 +648,13 @@ void mglParse::Execute(mglGraph *gr, int n, const wchar_t **text, void (*error)(
 		if(r<0)	{	i = -r-2;	continue;	}
 		if(error)
 		{
-			if(r>0)	error(i, r);
-			if(gr->Message && gr->Message[0])	error(i,0);
+			if(r>0)	error(i, r, gr);
+			if(gr->Message && gr->Message[0])	error(i,0,gr);
 		}
 	}
 }
 //-----------------------------------------------------------------------------
-void mglParse::Execute(mglGraph *gr, const wchar_t *text, void (*error)(int line, int kind))
+void mglParse::Execute(mglGraph *gr, const wchar_t *text, void (*error)(int line, int kind, mglGraph *gr))
 {
 	unsigned s = wcslen(text)+1;
 	wchar_t *wcs = new wchar_t[s];
@@ -719,7 +670,7 @@ void mglParse::Execute(mglGraph *gr, const wchar_t *text, void (*error)(int line
 	delete []wcs;	free(str);
 }
 //-----------------------------------------------------------------------------
-void mglParse::Execute(mglGraph *gr, const char *text, void (*error)(int line, int kind))
+void mglParse::Execute(mglGraph *gr, const char *text, void (*error)(int line, int kind, mglGraph *gr))
 {
 	unsigned s = strlen(text)+1;
 	wchar_t *wcs = new wchar_t[s];
@@ -745,67 +696,67 @@ void mglParse::ProcOpt(mglGraph *gr, wchar_t *str)
 		if(opt[0])
 		{
 			gr->Min.x = val[0];	gr->Max.x = val[1];	gr->RecalcBorder();
-			if(out)	swprintf(buf,256,L"\tgr->Min.x = tx1;\tgr->Max.x = tx2;}");
+			if(out)	mglprintf(buf,256,L"\tgr->Min.x = tx1;\tgr->Max.x = tx2;}");
 			wcscat(op2,buf);
 		}
 		if(opt[1])
 		{
 			gr->Min.y = val[2];	gr->Max.y = val[3];	gr->RecalcBorder();
-			if(out)	swprintf(buf,256,L"\tgr->Min.y = ty1;\tgr->Max.y = ty2;}");
+			if(out)	mglprintf(buf,256,L"\tgr->Min.y = ty1;\tgr->Max.y = ty2;}");
 			wcscat(op2,buf);
 		}
 		if(opt[2])
 		{
 			gr->Min.z = val[4];	gr->Max.z = val[5];	gr->RecalcBorder();
-			if(out)	swprintf(buf,256,L"\tgr->Min.z = tz1;\tgr->Max.z = tz2;}");
+			if(out)	mglprintf(buf,256,L"\tgr->Min.z = tz1;\tgr->Max.z = tz2;}");
 			wcscat(op2,buf);
 		}
 		if(opt[3])
 		{
 			gr->Cmin = val[6];	gr->Cmax = val[7];
-			if(out)	swprintf(buf,256,L"\tgr->Cmin = tc1;\tgr->Cmax = tc2;}");
+			if(out)	mglprintf(buf,256,L"\tgr->Cmin = tc1;\tgr->Cmax = tc2;}");
 			wcscat(op2,buf);
 		}
 		if(opt[4])
 		{
 			gr->Ambient(val[8]);
-			if(out)	swprintf(buf,256,L"\tgr->Ambient(tam);}");
+			if(out)	mglprintf(buf,256,L"\tgr->Ambient(tam);}");
 			wcscat(op2,buf);
 		}
 		if(opt[5])
 		{
 			gr->Cut = val[10]!=0;
-			if(out)	swprintf(buf,256,L"\tgr->Cut = tct;}");
+			if(out)	mglprintf(buf,256,L"\tgr->Cut = tct;}");
 			wcscat(op2,buf);
 		}
 		if(opt[6])
 		{
 			gr->AlphaDef = val[12];
-			if(out)	swprintf(buf,256,L"\tgr->AlphaDef = tad;}");
+			if(out)	mglprintf(buf,256,L"\tgr->AlphaDef = tad;}");
 			wcscat(op2,buf);
 		}
 		if(opt[7])
 		{
 			gr->MeshNum  = int(val[14]);
-			if(out)	swprintf(buf,256,L"\tgr->MeshNum = tmn;}");
+			if(out)	mglprintf(buf,256,L"\tgr->MeshNum = tmn;}");
 			wcscat(op2,buf);
 		}
 		if(opt[8])
 		{
 			gr->FontSize = val[16];
-			if(out)	swprintf(buf,256,L"\tgr->FontSize = tfs;}");
+			if(out)	mglprintf(buf,256,L"\tgr->FontSize = tfs;}");
 			wcscat(op2,buf);
 		}
 		if(opt[9])
 		{
 			gr->MarkSize = val[17];
-			if(out)	swprintf(buf,256,L"\tgr->MarkSize = tms;}");
+			if(out)	mglprintf(buf,256,L"\tgr->MarkSize = tms;}");
 			wcscat(op2,buf);
 		}
 		if(opt[10])
 		{
 			gr->AddLegend(leg, gr->last_style);
-			if(out)	swprintf(buf,256,L"\tgr->AddLegend(\"%s\", \"%s\");", leg, gr->last_style);
+			if(out)	mglprintf(buf,256,L"\tgr->AddLegend(\"%s\", \"%s\");", leg, gr->last_style);
 			wcscat(op2,buf);
 		}
 		for(long i=0;i<16;i++)	opt[i]=false;
@@ -827,7 +778,7 @@ void mglParse::ProcOpt(mglGraph *gr, wchar_t *str)
 			n=mglFindArg(s);
 			if(n<1)	return;
 			s[n]=0;		a=s;	s=s+n+1;	wcstrim_mgl(a);
-			float ff = wcstod(s,0);
+			mreal ff = wcstod(s,0);
 			if(!wcscmp(a+1,L"range"))
 			{
 				n=mglFindArg(s);	if(n<1)	return;
@@ -835,68 +786,68 @@ void mglParse::ProcOpt(mglGraph *gr, wchar_t *str)
 				if(a[0]=='x')
 				{
 					opt[0]=true;	gr->Min.x = wcstod(b,0);	gr->Max.x = wcstod(s,0);
-					if(out)	swprintf(buf,256,L"{float tx1=gr->Min.x, tx2=gr->Max.x;\tgr->Min.x=%g;\tgr->Max.x=%g;", wcstod(b,0), wcstod(s,0));
+					if(out)	mglprintf(buf,256,L"{mreal tx1=gr->Min.x, tx2=gr->Max.x;\tgr->Min.x=%g;\tgr->Max.x=%g;", wcstod(b,0), wcstod(s,0));
 					wcscat(op1,buf);
 				}
 				else if(a[0]=='y')
 				{
 					opt[1]=true;	gr->Min.y = wcstod(b,0);	gr->Max.y = wcstod(s,0);
-					if(out)	swprintf(buf,256,L"{float ty1=gr->Min.y, ty2=gr->Max.y;\tgr->Min.y=%g;\tgr->Max.y=%g;", wcstod(b,0), wcstod(s,0));
+					if(out)	mglprintf(buf,256,L"{mreal ty1=gr->Min.y, ty2=gr->Max.y;\tgr->Min.y=%g;\tgr->Max.y=%g;", wcstod(b,0), wcstod(s,0));
 					wcscat(op1,buf);
 				}
 				else if(a[0]=='z')
 				{
 					opt[2]=true;	gr->Min.z = wcstod(b,0);	gr->Max.z = wcstod(s,0);
-					if(out)	swprintf(buf,256,L"{float tz1=gr->Min.z, tz2=gr->Max.z;\tgr->Min.z=%g;\tgr->Max.z=%g;", wcstod(b,0), wcstod(s,0));
+					if(out)	mglprintf(buf,256,L"{mreal tz1=gr->Min.z, tz2=gr->Max.z;\tgr->Min.z=%g;\tgr->Max.z=%g;", wcstod(b,0), wcstod(s,0));
 					wcscat(op1,buf);
 				}
 				else if(a[0]=='c')
 				{
 					opt[3]=true;	gr->Cmin = wcstod(b,0);		gr->Cmax = wcstod(s,0);
-					if(out)	swprintf(buf,256,L"{float tc1=gr->Cmin, tc2=gr->Cmax;\tgr->Cmin=%g;\tgr->Cmax=%g;", wcstod(b,0), wcstod(s,0));
+					if(out)	mglprintf(buf,256,L"{mreal tc1=gr->Cmin, tc2=gr->Cmax;\tgr->Cmin=%g;\tgr->Cmax=%g;", wcstod(b,0), wcstod(s,0));
 					wcscat(op1,buf);
 				}
 			}
 			else if(!wcscmp(a,L"cut"))
 			{
 				opt[5]=true;	gr->Cut = (ff!=0 || !wcsncmp(s,L"on",2));
-				if(out)	swprintf(buf,256,L"{bool tct=gr->Cut;\tgr->Cut=%s;", gr->Cut?"true":"false");
+				if(out)	mglprintf(buf,256,L"{bool tct=gr->Cut;\tgr->Cut=%s;", gr->Cut?"true":"false");
 				wcscat(op1,buf);
 			}
 			else if(!wcscmp(a,L"meshnum"))
 			{
 				opt[7]=true;	gr->MeshNum = int(ff);
-				if(out)	swprintf(buf,256,L"{int tmn=gr->MeshNum;\tgr->MeshNum=%d;", gr->MeshNum);
+				if(out)	mglprintf(buf,256,L"{int tmn=gr->MeshNum;\tgr->MeshNum=%d;", gr->MeshNum);
 				wcscat(op1,buf);
 			}
 			else if(!wcscmp(a,L"alphadef"))
 			{
 				opt[6]=true;	gr->AlphaDef = ff;
-				if(out)	swprintf(buf,256,L"{float tad=gr->AlphaDef;\tgr->AlphaDef=%g;", ff);
+				if(out)	mglprintf(buf,256,L"{mreal tad=gr->AlphaDef;\tgr->AlphaDef=%g;", ff);
 				wcscat(op1,buf);
 			}
 			else if(!wcscmp(a,L"alpha"))
 			{
 				opt[6]=true;	gr->AlphaDef = ff;
-				if(out)	swprintf(buf,256,L"{float tad=gr->AlphaDef;\tgr->AlphaDef=%g;", ff);
+				if(out)	mglprintf(buf,256,L"{mreal tad=gr->AlphaDef;\tgr->AlphaDef=%g;", ff);
 				wcscat(op1,buf);
 			}
 			else if(!wcscmp(a,L"fontsize"))
 			{
 				opt[8]=true;	gr->FontSize = ff>0 ? ff : -ff*gr->FontSize;
-				if(out)	swprintf(buf,256,L"{float tfs=gr->FontSize;\tgr->FontSize=%g;", gr->FontSize);
+				if(out)	mglprintf(buf,256,L"{mreal tfs=gr->FontSize;\tgr->FontSize=%g;", gr->FontSize);
 				wcscat(op1,buf);
 			}
 			else if(!wcscmp(a,L"ambient"))
 			{
 				opt[4]=true;	gr->Ambient(ff);
-				if(out)	swprintf(buf,256,L"{float tam=gr->AmbBr;\tgr->Ambient(%g);", ff);
+				if(out)	mglprintf(buf,256,L"{mreal tam=gr->AmbBr;\tgr->Ambient(%g);", ff);
 				wcscat(op1,buf);
 			}
 			else if(!wcscmp(a,L"marksize"))
 			{
 				opt[9]=true;	gr->MarkSize = ff/50;
-				if(out)	swprintf(buf,256,L"{float tad=gr->MarkSize;\tgr->MarkSize=%g/50;", ff);
+				if(out)	mglprintf(buf,256,L"{mreal tad=gr->MarkSize;\tgr->MarkSize=%g/50;", ff);
 				wcscat(op1,buf);
 			}
 			else if(!wcscmp(a,L"legend"))
