@@ -17,11 +17,6 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
-#ifdef USE_GETTEXT
-	#include <libintl.h>
-#else
-	#define gettext(x)	(x)
-#endif
 
 #include <FL/Fl_Pixmap.H>
 #include <FL/fl_ask.H>
@@ -35,6 +30,16 @@
 #include "mgl/mgl_eps.h"
 #include "mgl/mgl_c.h"
 #include "mgl/mgl_f.h"
+
+#ifdef USE_GETTEXT
+	#include <libintl.h>
+#else
+// Workaround for gcc 4.2
+	#ifndef _LIBINTL_H
+	#define gettext(x)	(x)
+	#endif
+#endif
+
 //-----------------------------------------------------------------------------
 #include "xpm/alpha_on.xpm"
 #include "xpm/light_on.xpm"
@@ -91,7 +96,7 @@ void Fl_MathGL::draw()
 void Fl_MathGL::update(mglGraph *gr)
 {
 	if(gr==0)	gr=graph;
-	if(gr==0)	return;
+	if(gr==0 || draw_func==0)	return;
 	gr->DefaultPlotParam();
 	gr->Alpha(flag&1);
 	gr->Light(flag&2);
@@ -100,8 +105,8 @@ void Fl_MathGL::update(mglGraph *gr)
 	gr->DrawFace = !rotate;
 
 	gr->Message = new char[2048];	gr->Message[0] = 0;
-	if(draw_func)	draw_func(gr, draw_par);
-	if(gr->Message[0] != 0)			fl_message(gr->Message);
+	draw_func(gr, draw_par);
+	if(gr->Message[0] != 0)			fl_message("%s",gr->Message);
 	delete []gr->Message;			gr->Message = 0;
 
 	if(gr==graph && (graph->GetWidth()!=w() || graph->GetHeight()!=h()))
@@ -205,7 +210,7 @@ int Fl_MathGL::handle(int code)
 	if(code==FL_DRAG)
 	{
 		xe=Fl::event_x();	ye=Fl::event_y();
-		float ff = 240/sqrt(float(w()*h()));
+		mreal ff = 240/sqrt(mreal(w()*h()));
 		if(rotate)
 		{
 			phi += int((x0-xe)*ff);
@@ -222,11 +227,11 @@ int Fl_MathGL::handle(int code)
 		if(zoom)
 		{
 			int w1=w(),h1=h();
-			float _x1,_x2,_y1,_y2;
-			_x1 = x1+(x2-x1)*(x0-x())/float(w1);
-			_y1 = y2-(y2-y1)*(ye-y())/float(h1);
-			_x2 = x1+(x2-x1)*(xe-x())/float(w1);
-			_y2 = y2-(y2-y1)*(y0-y())/float(h1);
+			mreal _x1,_x2,_y1,_y2;
+			_x1 = x1+(x2-x1)*(x0-x())/mreal(w1);
+			_y1 = y2-(y2-y1)*(ye-y())/mreal(h1);
+			_x2 = x1+(x2-x1)*(xe-x())/mreal(w1);
+			_y2 = y2-(y2-y1)*(y0-y())/mreal(h1);
 			x1=_x1;		x2=_x2;		y1=_y1;		y2=_y2;
 			if(x1>x2)	{	_x1=x1;	x1=x2;	x2=_x1;	}
 			if(y1>y2)	{	_x1=y1;	y1=y2;	y2=_x1;	}
@@ -428,7 +433,7 @@ void oncemore_cb(Fl_Widget*, void*v)	{	((mglGraphFLTK*)v)->ReLoad(true);	}
 void su_cb(Fl_Widget*, void* v)
 {
 	mglGraphFLTK* e = (mglGraphFLTK*)v;
-	float x1,x2,y1,y2,d;
+	mreal x1,x2,y1,y2,d;
 	e->FMGL->get_zoom(&x1,&y1,&x2,&y2);
 	d = (y2-y1)/3;	y1 += d;	y2 += d;
 	e->FMGL->set_zoom(x1,y1,x2,y2);
@@ -438,7 +443,7 @@ void su_cb(Fl_Widget*, void* v)
 void sd_cb(Fl_Widget*, void* v)
 {
 	mglGraphFLTK* e = (mglGraphFLTK*)v;
-	float x1,x2,y1,y2,d;
+	mreal x1,x2,y1,y2,d;
 	e->FMGL->get_zoom(&x1,&y1,&x2,&y2);
 	d = (y2-y1)/3;	y1 -= d;	y2 -= d;
 	e->FMGL->set_zoom(x1,y1,x2,y2);
@@ -448,7 +453,7 @@ void sd_cb(Fl_Widget*, void* v)
 void sr_cb(Fl_Widget*, void* v)
 {
 	mglGraphFLTK* e = (mglGraphFLTK*)v;
-	float x1,x2,y1,y2,d;
+	mreal x1,x2,y1,y2,d;
 	e->FMGL->get_zoom(&x1,&y1,&x2,&y2);
 	d = (x2-x1)/3;	x1 += d;	x2 += d;
 	e->FMGL->set_zoom(x1,y1,x2,y2);
@@ -458,7 +463,7 @@ void sr_cb(Fl_Widget*, void* v)
 void sl_cb(Fl_Widget*, void* v)
 {
 	mglGraphFLTK* e = (mglGraphFLTK*)v;
-	float x1,x2,y1,y2,d;
+	mreal x1,x2,y1,y2,d;
 	e->FMGL->get_zoom(&x1,&y1,&x2,&y2);
 	d = (x2-x1)/3;	x1 -= d;	x2 -= d;
 	e->FMGL->set_zoom(x1,y1,x2,y2);
@@ -468,7 +473,7 @@ void sl_cb(Fl_Widget*, void* v)
 void sz_cb(Fl_Widget*, void* v)
 {
 	mglGraphFLTK* e = (mglGraphFLTK*)v;
-	float x1,x2,y1,y2,d;
+	mreal x1,x2,y1,y2,d;
 	e->FMGL->get_zoom(&x1,&y1,&x2,&y2);
 	d = (y2-y1)/4;	y1 += d;	y2 -= d;
 	d = (x2-x1)/4;	x1 += d;	x2 -= d;
@@ -479,7 +484,7 @@ void sz_cb(Fl_Widget*, void* v)
 void so_cb(Fl_Widget*, void* v)
 {
 	mglGraphFLTK* e = (mglGraphFLTK*)v;
-	float x1,x2,y1,y2,d;
+	mreal x1,x2,y1,y2,d;
 	e->FMGL->get_zoom(&x1,&y1,&x2,&y2);
 	d = (y2-y1)/2;	y1 -= d;	y2 += d;
 	d = (x2-x1)/2;	x1 -= d;	x2 += d;
@@ -570,7 +575,7 @@ void mglGraphFLTK::Window(int argc, char **argv, int (*draw)(mglGraph *gr, void
 {
 	NumFig=0;	CurFig=0;
 	CurFrameId = 0;
-	int n = draw(this,par);
+	int n = draw ? draw(this,par) : 0;
 	if(n<NumFig && n>=0)	NumFig = n;
 	DrawFunc = draw;		FuncPar = par;
 	LoadFunc = reload;
@@ -704,11 +709,13 @@ void mgl_fltk_run()	{	mglFlRun();	}
 	pthread_detach(tmp);
 }*/
 //-----------------------------------------------------------------------------
-uintptr_t mgl_create_graph_fltk_(int (*draw)(uintptr_t *gr), const char *title, int l)
+//uintptr_t mgl_create_graph_fltk_(int (*draw)(uintptr_t *gr), const char *title, int l)
+uintptr_t mgl_create_graph_fltk_(const char *title, int l)
 {
 	mglGraphFLTK *g = new mglGraphFLTK;
 	char *s = new char[l+1];	memcpy(s,title,l);	s[l]=0;
-	g->Window(0,0,mgl_fortran_func,s,(void*)draw);
+//	g->Window(0,0,mgl_fortran_func,s,(void*)draw);
+	g->Window(0,0,0,s,0,0);
 	delete []s;
 	return uintptr_t(g);
 }
