@@ -1,5 +1,5 @@
 /***************************************************************************
- * mgl_qt.cpp is part of Math Graphic Library
+ * mgl_qt.cpp is part of Math Graphic Library                              *
  * Copyright (C) 2007 Alexey Balakin <balakin@appl.sci-nnov.ru>            *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -152,16 +152,16 @@ void QMathGL::setRotate(bool r)
 }
 //-----------------------------------------------------------------------------
 void QMathGL::shiftDown()
-{	float d=(y2-y1)/3;	y1+=d;	y2+=d;	update();	}
+{	mreal d=(y2-y1)/3;	y1+=d;	y2+=d;	update();	}
 //-----------------------------------------------------------------------------
 void QMathGL::shiftUp()
-{	float d=(y2-y1)/3;	y1-=d;	y2-=d;	update();	}
+{	mreal d=(y2-y1)/3;	y1-=d;	y2-=d;	update();	}
 //-----------------------------------------------------------------------------
 void QMathGL::shiftRight()
-{	float d=(x2-x1)/3;	x1-=d;	x2-=d;	update();	}
+{	mreal d=(x2-x1)/3;	x1-=d;	x2-=d;	update();	}
 //-----------------------------------------------------------------------------
 void QMathGL::shiftLeft()
-{	float d=(x2-x1)/3;	x1+=d;	x2+=d;	update();	}
+{	mreal d=(x2-x1)/3;	x1+=d;	x2+=d;	update();	}
 //-----------------------------------------------------------------------------
 void QMathGL::restore()
 {
@@ -173,7 +173,7 @@ void QMathGL::restore()
 //-----------------------------------------------------------------------------
 void QMathGL::zoomIn()
 {
-	float d;
+	mreal d;
 	d = (y2-y1)/4;	y1 += d;	y2 -= d;
 	d = (x2-x1)/4;	x1 += d;	x2 -= d;
 	update();
@@ -181,7 +181,7 @@ void QMathGL::zoomIn()
 //-----------------------------------------------------------------------------
 void QMathGL::zoomOut()
 {
-	float d;
+	mreal d;
 	d = (y2-y1)/2;	y1 -= d;	y2 += d;
 	d = (x2-x1)/2;	x1 -= d;	x2 += d;
 	update();
@@ -190,7 +190,7 @@ void QMathGL::zoomOut()
 void QMathGL::update(mglGraph *gr)
 {
 	if(gr==0)	gr = graph;
-	if(gr==0)	return;
+	if(gr==0 || draw_func==0)	return;
 	gr->DefaultPlotParam();
 	gr->Alpha(alpha);	gr->Light(light);
 	gr->View(tet,phi);	gr->Org = mglPoint(NAN,NAN,NAN);
@@ -199,7 +199,7 @@ void QMathGL::update(mglGraph *gr)
 	gr->DrawFace = !rotate;
 
 	char *buf=new char[2048];	buf[0]=0;	gr->Message = buf;
-	if(draw_func)	draw_func(gr, draw_par);
+	draw_func(gr, draw_par);
 	if(buf[0] != 0)	QMessageBox::warning(this, appName, buf);
 	gr->Message = 0;		delete []buf;
 	mousePos="";
@@ -220,6 +220,7 @@ void QMathGL::mousePressEvent(QMouseEvent *ev)
 		mglPoint p = graph->CalcXYZ(ev->x(), ev->y());
 		mousePos.sprintf("x=%g, y=%g, z=%g",p.x,p.y,p.z);
 		repaint();
+		emit mouseClick(p.x,p.y,p.z);
 	}
 	xe=x0=ev->x();	ye=y0=ev->y();	ev->accept();
 }
@@ -231,9 +232,9 @@ void QMathGL::mouseReleaseEvent(QMouseEvent *ev)
 		if(zoom)
 		{
 			int w1=width(),h1=height();
-			float _x1,_x2,_y1,_y2;
-			_x1 = x1+(x2-x1)*(x0-x())/float(w1);	_y1 = y2-(y2-y1)*(ye-y())/float(h1);
-			_x2 = x1+(x2-x1)*(xe-x())/float(w1);	_y2 = y2-(y2-y1)*(y0-y())/float(h1);
+			mreal _x1,_x2,_y1,_y2;
+			_x1 = x1+(x2-x1)*(x0-x())/mreal(w1);	_y1 = y2-(y2-y1)*(ye-y())/mreal(h1);
+			_x2 = x1+(x2-x1)*(xe-x())/mreal(w1);	_y2 = y2-(y2-y1)*(y0-y())/mreal(h1);
 			x1=_x1;		x2=_x2;		y1=_y1;		y2=_y2;
 			if(x1>x2)	{	_x1=x1;	x1=x2;	x2=_x1;	}
 			if(y1>y2)	{	_x1=y1;	y1=y2;	y2=_x1;	}
@@ -253,7 +254,7 @@ void QMathGL::mouseMoveEvent(QMouseEvent *ev)
 	{
 		if(ev->buttons()&Qt::LeftButton)	// rotate
 		{
-			float ff = 240/sqrt(float(width()*height()));
+			mreal ff = 240/sqrt(mreal(width()*height()));
 			phi += int((x0-xe)*ff);
 			tet += int((y0-ye)*ff);
 			if(phi>180)	phi-=360;		if(phi<-180)	phi+=360;
@@ -263,8 +264,8 @@ void QMathGL::mouseMoveEvent(QMouseEvent *ev)
 		}
 		if(ev->buttons()&Qt::RightButton)	// zoom and perspective
 		{
-			float ff = 2.*(y0-ye)/width(), gg = 0.5*(xe-x0)/height();
-			float cx = (x1+x2)/2, cy = (y1+y2)/2;
+			mreal ff = 2.*(y0-ye)/width(), gg = 0.5*(xe-x0)/height();
+			mreal cx = (x1+x2)/2, cy = (y1+y2)/2;
 			x1 = cx+(x1-cx)*exp(-ff);	x2 = cx+(x2-cx)*exp(-ff);
 			y1 = cy+(y1-cy)*exp(-ff);	y2 = cy+(y2-cy)*exp(-ff);
 			per = per + gg;
@@ -274,8 +275,8 @@ void QMathGL::mouseMoveEvent(QMouseEvent *ev)
 		}
 		if(ev->buttons()&Qt::MidButton)	// shift
 		{
-			float ff = 1./sqrt(float(width()*height()));
-			float dx = (x0-xe)*ff*(x2-x1), dy = (y0-ye)*ff*(y2-y1);
+			mreal ff = 1./sqrt(mreal(width()*height()));
+			mreal dx = (x0-xe)*ff*(x2-x1), dy = (y0-ye)*ff*(y2-y1);
 			x1 += dx;	x2 += dx;	y1 -= dy;	y2 -= dy;
 		}
 		x0 = xe;	y0 = ye;
@@ -426,7 +427,7 @@ void QMathGL::prevSlide()
 void QMathGL::animation(bool st)
 {
 	mglGraphQT *gr = dynamic_cast<mglGraphQT *>(graph);
-	if(st)	timer->start(gr ? gr->Delay*1000 : animDelay);
+	if(st)	timer->start(gr ? int(gr->Delay*1000) : animDelay);
 	else	timer->stop();
 }
 //-----------------------------------------------------------------------------
@@ -560,7 +561,7 @@ void mglGraphQT::Window(int argc, char **argv, int (*draw)(mglGraph *gr, void *p
 {
 	NumFig=0;	CurFig=0;
 	CurFrameId = 0;
-	int n = draw(this,par);
+	int n = draw ? draw(this,par) : 0;
 	if(n<NumFig && n>=0)	NumFig = n;
 	DrawFunc = draw;		FuncPar = par;
 	LoadFunc = reload;
@@ -573,17 +574,17 @@ void mglGraphQT::Window(int argc, char **argv, int (*draw)(mglGraph *gr, void *p
 		return;
 	}
 
-	if(!qApp && argc>0)
+	if(!qApp)
 	{
 		QApplication *a;
 		if(!argv)
 		{
 			static char tmp[2][1];
-			tmp[0][0]=tmp[1][0]=0;	argc=1;
-			a = new QApplication(argc, (char **)tmp);
+			tmp[0][0]=tmp[1][0]=0;
+			static int aa=1;
+			a = new QApplication(aa, (char **)tmp);
 		}
-		else
-			a = new QApplication(argc, argv);
+		else	a = new QApplication(argc, argv);
 		a->connect(a, SIGNAL(lastWindowClosed()), a, SLOT(quit()));
 	}
 
@@ -593,9 +594,10 @@ void mglGraphQT::Window(int argc, char **argv, int (*draw)(mglGraph *gr, void *p
 	QMGL = new QMathGL(Wnd);
 	QMGL->setPopup(popup);	QMGL->setGraph(this);
 	QMGL->setDraw(draw, par);
-	QMGL->update();			makeMenu();
+	makeMenu();	qApp->processEvents();
 	scroll->setWidget(QMGL);
 	Wnd->setCentralWidget(scroll);
+	QMGL->update();
 	if(maximize)
 	{	Wnd->showMaximized();	}
 	else	Wnd->show();
@@ -615,11 +617,13 @@ HMGL mgl_create_graph_qt(int (*draw)(HMGL gr, void *p), const char *title, void
 	return g;
 }
 //-----------------------------------------------------------------------------
-uintptr_t mgl_create_graph_qt_(int (*draw)(uintptr_t *gr), const char *title, int l)
+//uintptr_t mgl_create_graph_qt_(int (*draw)(uintptr_t *gr), const char *title, int l)
+uintptr_t mgl_create_graph_qt_(const char *title, int l)
 {
 	mglGraphQT *g = new mglGraphQT;
 	char *s = new char[l+1];	memcpy(s,title,l);	s[l]=0;
-	g->Window(0,0,mgl_fortran_func,s,(void*)draw);
+//	g->Window(0,0,mgl_fortran_func,s,(void*)draw);
+	g->Window(0,0,0,s,0,0);
 	delete []s;
 	return uintptr_t(g);
 }
@@ -653,9 +657,7 @@ void mglGraphQT::makeMenu()
 		oo = new QMenu(TR("&Export as ..."),Wnd);
 		oo->addAction(TR("PNG"), QMGL, SLOT(exportPNG()),Qt::ALT+Qt::Key_P);
 		oo->addAction(TR("solid PNG"), QMGL, SLOT(exportPNGs()),Qt::ALT+Qt::Key_F);
-#ifndef WIN32
 		oo->addAction(TR("JPEG"), QMGL, SLOT(exportJPG()),Qt::ALT+Qt::Key_J);
-#endif
 		oo->addAction(TR("bitmap EPS"), QMGL, SLOT(exportBPS()));
 		oo->addAction(TR("vector EPS"), QMGL, SLOT(exportEPS()),Qt::ALT+Qt::Key_E);
 		oo->addAction(TR("SVG"), QMGL, SLOT(exportSVG()),Qt::ALT+Qt::Key_S);
@@ -791,100 +793,3 @@ void mglGraphQT::makeMenu()
 	o->addAction(TR("About &Qt"), QMGL, SLOT(aboutQt()));
 }
 //-----------------------------------------------------------------------------
-/*//
-//	Function for mglGraphAB::ShowImage()
-//
-//-----------------------------------------------------------------------------
-void mgl_show_img_qt(const char *title, mglGraphAB *gr, bool same, bool thread)
-{
-	static QMainWindow *Wnd=NULL;
-	static mglQShowImg *img=NULL;
-	if(same && Wnd)		// if prev window then show it
-	{
-		convertFromGraph(img->pic, gr, &(img->buf));
-		Wnd->setWindowTitle(title);
-		Wnd->show();	return;
-	}
-	if(!qApp)	// if no application then create it
-	{
-		QApplication *a;
-//		if(!argv)
-		{
-			static char tmp[2][1];
-			tmp[0][0]=tmp[1][0]=0;
-			int argc=1;
-			a = new QApplication(argc, (char **)tmp);
-		}
-		a->connect(a, SIGNAL(lastWindowClosed()), a, SLOT(quit()));
-	}
-	// create new window
-	Wnd = new QMainWindow;	Wnd->resize(gr->GetWidth(),gr->GetHeight());
-	Wnd->setWindowTitle(title);
-	img = new mglQShowImg(Wnd);
-	convertFromGraph(img->pic, gr, &(img->buf));
-	Wnd->setCentralWidget(img);
-	// setup menu
-	QMenu *o = Wnd->menuBar()->addMenu(TR("&File"));
-	o->addAction(TR("Save"), img, SLOT(save()), Qt::CTRL+Qt::Key_S);
-	o->addAction(TR("Copy"), img, SLOT(copy()), Qt::CTRL+Qt::Key_C);
-	o->addAction(TR("Print"), img, SLOT(print()), Qt::CTRL+Qt::Key_P);
-	o->addSeparator();
-	o->addAction(TR("&Close"), Wnd, SLOT(close()), Qt::CTRL+Qt::Key_W);
-	Wnd->menuBar()->addSeparator();
-	o = Wnd->menuBar()->addMenu(TR("&Help"));
-	o->addAction(TR("About"), img, SLOT(about()));
-	o->addAction(TR("About &Qt"), img, SLOT(aboutQt()));
-	// now show it
-	Wnd->show();
-	if(thread)	mgl_qt_thread();	else mgl_qt_run();
-}
-//-----------------------------------------------------------------------------
-mglQShowImg::mglQShowImg(QWidget *parent, Qt::WindowFlags f) : QWidget(parent, f)
-{	buf=NULL;	}
-//-----------------------------------------------------------------------------
-void mglQShowImg::paintEvent(QPaintEvent *)
-{
-	QPainter paint;
-	paint.begin(this);
-	paint.drawPixmap(0,0,pic);
-	paint.end();
-}
-//-----------------------------------------------------------------------------
-void mglQShowImg::resizeEvent(QResizeEvent *)
-{	resize(pic.width(), pic.height());	}
-//-----------------------------------------------------------------------------
-void mglQShowImg::about()
-{
-	QString s = tr("MathGL v. 1.") + QString::number(MGL_VERSION) +
-			tr("\n(c) Alexey Balakin, 2007\nhttp://mathgl.sourceforge.net/");
-	QMessageBox::about(this, tr("MathGL - about"), s);
-}
-//-----------------------------------------------------------------------------
-void mglQShowImg::aboutQt()	{	QMessageBox::aboutQt(this, tr("About Qt"));	}
-//-----------------------------------------------------------------------------
-void mglQShowImg::copy()
-{
-	QClipboard *cb = QApplication::clipboard();
-	cb->setPixmap(pic, QClipboard::Clipboard);
-}
-//-----------------------------------------------------------------------------
-void mglQShowImg::print()
-{
-	QPrinter *printer = new QPrinter;
-	QPrintDialog printDlg(printer, this);
-	if (printDlg.exec() == QDialog::Accepted)
-	{
-		QPainter p;
-		if(!p.begin(printer))	return;	// paint on printer
-		p.drawPixmap(0,0,pic);
-	}
-	delete printer;
-}
-//-----------------------------------------------------------------------------
-void mglQShowImg::save()
-{
-	QString fn = QFileDialog::getSaveFileName(this, tr("MathGL - save image"), "", tr("PNG files (*.png)\nJPEG files (*.jpg)\nAll files (*.*)"));
-	if(!fn.isEmpty())	pic.save(fn);
-}
-//-----------------------------------------------------------------------------
-*/
\ No newline at end of file
